// Archer animations

#include "ArcherCommon.as"
#include "FireParticle.as"

const f32 config_offset = -4.0f;

void onInit( CSprite@ this )
{
    LoadSprites( this );
}

void LoadSprites( CSprite@ this )
{
    string texname = this.getBlob().getSexNum() == 0 ?
                     "Entities/Characters/Archer/ArcherMale.png" :
                     "Entities/Characters/Archer/ArcherFemale.png";
    this.ReloadSprite( texname, this.getConsts().frameWidth, this.getConsts().frameHeight,
                       this.getBlob().getTeamNum(), this.getBlob().getSkinNum() );
    this.RemoveSpriteLayer("frontarm");
    CSpriteLayer@ frontarm = this.addSpriteLayer( "frontarm", texname , 32, 16, this.getBlob().getTeamNum(), this.getBlob().getSkinNum() );

    if (frontarm !is null)
    {
        Animation@ animcharge = frontarm.addAnimation( "charge", 0, false );
        animcharge.AddFrame(17);
        animcharge.AddFrame(25);
        animcharge.AddFrame(33);
        Animation@ animshoot = frontarm.addAnimation( "fired", 0, false );
        animshoot.AddFrame(41);
        Animation@ animnoarrow = frontarm.addAnimation( "no_arrow", 0, false );
        animnoarrow.AddFrame(26);
        frontarm.SetOffset( Vec2f(-1.0f,5.0f+config_offset) );
        frontarm.SetAnimation("fired");
        frontarm.SetVisible(false);
    }

    this.RemoveSpriteLayer("backarm");
    CSpriteLayer@ backarm = this.addSpriteLayer( "backarm", texname , 32, 16, this.getBlob().getTeamNum(), this.getBlob().getSkinNum() );

    if (backarm !is null)
    {
        Animation@ anim = backarm.addAnimation( "default", 0, false );
        anim.AddFrame(18);
        backarm.SetOffset( Vec2f(-1.0f,5.0f+config_offset) );
        backarm.SetAnimation("default");
        backarm.SetVisible(false);
    }

    this.RemoveSpriteLayer("quiver");
    CSpriteLayer@ quiver = this.addSpriteLayer( "quiver", texname , 16, 16, this.getBlob().getTeamNum(), this.getBlob().getSkinNum() );

    if (quiver !is null)
    {
        Animation@ anim = quiver.addAnimation( "default", 0, false );
        anim.AddFrame(68);
        Animation@ emptyanim = quiver.addAnimation( "empty", 0, false );
        emptyanim.AddFrame(69);
        quiver.SetOffset( Vec2f(-10.0f,2.0f+config_offset) );
        quiver.SetAnimation("empty");
        quiver.SetRelativeZ(-0.1f);
    }
}

void setArmValues(CSpriteLayer@ arm, bool visible, f32 angle, f32 relativeZ, string anim, Vec2f around, Vec2f offset)
{
    if (arm !is null)
    {
        arm.SetVisible(visible);

        if (!arm.isAnimation(anim)) {
            arm.SetAnimation(anim);
        }

        arm.ResetTransform( );
        arm.SetRelativeZ( relativeZ );
        arm.RotateBy( angle, around );
        arm.SetOffset(offset);
    }
}

void onTick( CSprite@ this )
{
    // store some vars for ease and speed
    CBlob@ blob = this.getBlob();
    u8 knocked = blob.get_u8("knocked");
    Vec2f pos = blob.getPosition();
    Vec2f aimpos = blob.getAimPos();
    CMovement @movement = blob.getMovement();

    if (movement is null) { // can't animate without keys
        return;
    }

	bool has_arrows = blob.get_bool("has_arrow");

    bool inair = (!blob.isOnGround() && !blob.isOnLadder());
    Vec2f vel = blob.getVelocity();

    if (this.getBlob().hasTag("dead"))
    {
        this.SetAnimation("dead");
        this.RemoveSpriteLayer("frontarm");
        this.RemoveSpriteLayer("backarm");
        
        doQuiverUpdate(this,has_arrows, true);
        
        Vec2f oldvel = this.getBlob().getOldVelocity();

        if (vel.y < -1.0f) {
            this.SetFrameIndex( 0 );
        }
        else if (vel.y > 1.0f) {
            this.SetFrameIndex( 1 );
        }
        else if (oldvel.y > 1.0f && this.getBlob().isOnGround()) {
            this.SetFrameIndex( 3 );
        }
        else {
            this.SetFrameIndex( 2 );
        }

        return;
    }

	if (knocked > 0)
    {
		blob.Tag("dead head");
	}
    else if (blob.isKeyPressed(key_action1))
    {
        blob.Tag("attack head");
    }
    else
    {
        blob.Untag("attack head");
        blob.Untag("dead head");
    }
    // set facing

	bool facingLeft = this.isFacingLeft();	

    // get the angle of aiming with mouse
    Vec2f vec = aimpos - pos;
    f32 angle = vec.Angle();
    int direction;

    if ((angle > 330 && angle < 361) || (angle > -1 && angle < 30) ||
            (angle > 150 && angle < 210)) {
        direction = 0;
    }
    else if (aimpos.y < pos.y) {
        direction = -1;
    }
    else {
        direction = 1;
    }

    // animations
    bool ended = this.isAnimationEnded();
    u8 charge_state = blob.get_u8("charge_state");
    s8 charge_time = blob.get_s8("charge_time");
	const bool stab =  charge_state == ArcherParams::stabbing;
    bool firing = blob.isKeyPressed(key_action1) || (!stab && charge_state > 0);
    bool quiver = true;
	const bool left = blob.isKeyPressed(key_left);
	const bool right = blob.isKeyPressed(key_right);
	const bool up = blob.isKeyPressed(key_up);
	const bool down = blob.isKeyPressed(key_down);

	bool crouch = false;

    if (knocked > 0)
    {
        if (inair) {
            this.SetAnimation("knocked_air");
        }
        else {
            this.SetAnimation("knocked");
        }
    }
    else if (blob.hasTag( "seated" ))
    {
        this.SetAnimation("default");
    }
    else if (firing)
    {
        this.SetAnimation("shoot");
    }
    else if (stab)
	{
		this.SetAnimation("stab");
	}
    else if (inair)
    {
        this.SetAnimation("fall");
        f32 vy = vel.y;
        this.animation.timer = 0;

        if (vy < -1.5 || up) {
            this.animation.frame = 0;
        }
        else if (vy > 1.5 || down) {
            this.animation.frame = 2;
        }
        else {
            this.animation.frame = 1;
        }
    }
    else if ((left || right) ||
             (blob.isOnLadder() && (up || down) ) ) {
        this.SetAnimation("run");
    }
    else if (blob.isKeyPressed(key_down) && ended) {
        this.SetAnimation("crouch");
        crouch = true;
    }
    else
    {
        this.SetAnimation("default");
    }

    //arm anims
    Vec2f armOffset = Vec2f(-1.0f,4.0f+config_offset);
    u8 arrowType = blob.get_u8( "arrow type" );

    if (firing)
    {
        f32 armangle = -angle;

        if (this.isFacingLeft()) {
            armangle = 180.0f-angle;
        }

        while (armangle > 180.0f) {
            armangle -= 360.0f;
        }

        while (armangle < -180.0f) {
            armangle += 360.0f;
        }

        f32 sign = (this.isFacingLeft()?1.0f:-1.0f);
        CSpriteLayer@ frontarm = this.getSpriteLayer( "frontarm" );

        if (charge_state == ArcherParams::readying)
        {
            u16 frontframe = 0;
            f32 temp = charge_time;
            f32 ready_tween = temp / ArcherParams::ready_time;
            armangle = armangle * ready_tween;
            armOffset = Vec2f(-1.0f,4.0f+config_offset+2.0f*(1.0f-ready_tween));
            setArmValues(frontarm,true,armangle,0.1f,"charge",Vec2f(-4.0f*sign,0.0f), armOffset );
            frontarm.animation.frame = frontframe;
        }
        else if (charge_state == ArcherParams::no_arrows)
        {
            string animname = "no_arrow";

            if (charge_time == ArcherParams::ready_time)
            {
                animname = "fired";
            }

            u16 frontframe = 0;
            f32 temp = charge_time;
            f32 ready_tween = temp / ArcherParams::ready_time;
            armangle = armangle * ready_tween;
            armOffset = Vec2f(-1.0f,4.0f+config_offset+2.0f*(1.0f-ready_tween));
            setArmValues(frontarm,true,armangle,0.1f,animname,Vec2f(-4.0f*sign,0.0f), armOffset );
            frontarm.animation.frame = frontframe;
        }
        else if (charge_state != ArcherParams::fired)
        {
            u16 frontframe = Maths::Min((charge_time/((ArcherParams::shoot_period/3)+1)),2);
            setArmValues(frontarm,true,armangle,0.1f,"charge",Vec2f(-4.0f*sign,0.0f), armOffset);
            frontarm.animation.frame = frontframe;
        }
        else
        {
            setArmValues(frontarm,true,armangle,0.1f,"fired",Vec2f(-4.0f*sign,0.0f), armOffset);
        }

        frontarm.SetRelativeZ(1.5f);
        setArmValues(this.getSpriteLayer( "backarm" ),true,armangle,-0.1f,"default",Vec2f(-4.0f*sign,0.0f), armOffset);

        // fire arrow particles

        if (arrowType == ArrowType::fire && getGameTime() % 6 == 0 && blob.get_bool("arrow reqs"))
        {
            Vec2f offset = Vec2f(12.0f,0.0f);

            if (this.isFacingLeft()) {
                offset.x = -offset.x;
            }

            offset.RotateBy( armangle );
            makeFireParticle( frontarm.getWorldTranslation() + offset, 4  );
        }
    }
    else
    {
        setArmValues(this.getSpriteLayer( "frontarm" ),false,0.0f,0.1f,"fired",Vec2f(0,0), armOffset);
        setArmValues(this.getSpriteLayer( "backarm" ),false,0.0f,-0.1f,"default",Vec2f(0,0), armOffset);
    }

    // set fire light

    if (arrowType == ArrowType::fire)
    {
        if (firing && blob.get_bool("arrow reqs"))
        {
            blob.SetLight(true);
            blob.SetLightRadius( blob.getRadius()*2.0f );
        }
        else
        {
            blob.SetLight(false);
        }
    }

    //quiver
    doQuiverUpdate(this,has_arrows, quiver);

	//set the head anim
    if (knocked > 0 || crouch)
    {
		blob.Tag("dead head");
	}
    else if (blob.isKeyPressed(key_action1))
    {
        blob.Tag("attack head");
    }
    else
    {
        blob.Untag("attack head");
        blob.Untag("dead head");
    }


}

void doQuiverUpdate(CSprite@ this, bool has_arrows, bool quiver)
{
	CSpriteLayer@ quiverLayer = this.getSpriteLayer( "quiver" );

    if (quiverLayer !is null)
    {
        if (quiver)
        {
            quiverLayer.SetVisible(true);
            f32 quiverangle = -45.0f;

            if (this.isFacingLeft()) {
                quiverangle *= -1.0f;
            }

			PixelOffset @po = getDriver().getPixelOffset( this.getFilename(), this.getFrame() );

			bool down = (this.isAnimation("crouch") || this.isAnimation("dead"));
			bool easy = false;
			Vec2f off;
			if (po !is null)
			{
				easy = true;
				off.Set( this.getFrameWidth()/2, -this.getFrameHeight()/2 );
				off += this.getOffset();
				off += Vec2f( -po.x, po.y );
				
				
				f32 y = ( down ? 3.0f : 7.0f);
				f32 x = ( down ? 5.0f : 4.0f);
				off += Vec2f( x, y+config_offset );
			}

			if (easy)
			{
				quiverLayer.SetOffset( off );
			}

            quiverLayer.ResetTransform();
            quiverLayer.RotateBy(quiverangle, Vec2f(0.0f,0.0f));

            if (has_arrows) {
                quiverLayer.SetAnimation("default");
            }
            else {
                quiverLayer.SetAnimation("empty");
            }
        }
        else
        {
            quiverLayer.SetVisible(false);
        }
    }
}

void onGib(CSprite@ this)
{
    print("TODO ARCHER GIBBING");

    if (g_kidssafe) {
        return;
    }

    CBlob@ blob = this.getBlob();
    Vec2f pos = blob.getPosition();
    Vec2f vel = blob.getVelocity();
	vel.y -= 3.0f;
    f32 hp = Maths::Min(Maths::Abs(blob.getHealth()),2.0f) + 1.0f;
	const u8 team = blob.getTeamNum();
    CParticle@ Body     = makeGibParticle( "Entities/Characters/Archer/ArcherGibs.png", pos, vel + getRandomVelocity( 90, hp , 80 ), 0, 0, Vec2f (16,16), 2.0f, 20, "/BodyGibFall", team );
    CParticle@ Arm      = makeGibParticle( "Entities/Characters/Archer/ArcherGibs.png", pos, vel + getRandomVelocity( 90, hp - 0.2 , 80 ), 1, 0, Vec2f (16,16), 2.0f, 20, "/BodyGibFall", team );
    CParticle@ Shield   = makeGibParticle( "Entities/Characters/Archer/ArcherGibs.png", pos, vel + getRandomVelocity( 90, hp , 80 ), 2, 0, Vec2f (16,16), 2.0f, 0, "Sounds/material_drop.ogg", team );
    CParticle@ Sword    = makeGibParticle( "Entities/Characters/Archer/ArcherGibs.png", pos, vel + getRandomVelocity( 90, hp + 1 , 80 ), 3, 0, Vec2f (16,16), 2.0f, 0, "Sounds/material_drop.ogg", team );
}
