// Archer logic

#include "ArcherCommon.as"
#include "ThrowCommon.as"
#include "Knocked.as"
#include "Hitters.as"
#include "RunnerCommon.as"
#include "Help.as";

const int FLETCH_COOLDOWN = 45;
const int PICKUP_COOLDOWN = 15;
const int fletch_num_arrows = 1;
const int STAB_DELAY = 24;

void onInit( CBlob@ this )
{
    this.set_s8( "charge_time", 0 );
    this.set_u8( "charge_state", ArcherParams::not_aiming );
    this.set_bool( "has_arrow", false );
    this.set_f32("gib health", -1.5f);
    this.Tag("player");
    this.Tag("flesh");
    this.set_u8( "arrow type", ArrowType::normal );
    this.set_bool( "arrow reqs", true );
	this.set_Vec2f("inventory offset", Vec2f(0.0f, 76.0f));
	this.set_u8( "stab delay", 0 );
    //no spinning
    this.getShape().SetRotationsAllowed(false);
    this.getSprite().SetEmitSound( "Entities/Characters/Archer/BowPull.ogg" );
    this.getSprite().SetEmitSoundPaused( true );
    this.addCommandID("shoot arrow");
    this.addCommandID("pickup arrow");
    this.addCommandID("get satchel");
    //for activating satchel
    this.push("names to activate", "satchel");
																			   
	AddIconToken( "$Arrow$", "Entities/Characters/Archer/ArcherIcons.png", Vec2f(16,16), 0 );
	AddIconToken( "$FireArrow$", "Entities/Characters/Archer/ArcherIcons.png", Vec2f(16,16), 3 );

	SetHelp( this, "help self action", "archer", "Hold to fire arrow \n\n         $LMB$", "" ); 
	SetHelp( this, "help self action2", "archer", "Press to stab \n\n         $RMB$" ); 

    setKnockable( this );

    for (uint i = 0; i < arrowTypeNames.length; i++) {
        this.addCommandID( "pick " + arrowTypeNames[i]);
    }

	this.getCurrentScript().runFlags |= Script::tick_not_attached;
	this.getCurrentScript().removeIfTag = "dead";
}

bool hasArrows( CBlob@ this )
{
    u8 arrowType = this.get_u8( "arrow type" );
    return this.getBlobCount( arrowTypeNames[arrowType] ) > 0;
}

bool hasArrows( CBlob@ this, u8 arrowType )
{
	return this.getBlobCount( arrowTypeNames[arrowType] ) > 0;
}

bool hasReqs( CBlob@ this, u8 arrowType )
{
	bool reqs = true;
	
	if (arrowType == ArrowType::fire)
	{
		reqs = false;
		//check held
		CBlob@ b = this.getCarriedBlob();
		if (b !is null && b.hasTag("fire source"))
		{
			reqs = true;
		}
		//check touching
		int tcount = this.getTouchingCount();
		while(tcount --> 0 && !reqs)
		{
			@b = this.getTouchingByIndex(tcount);
			if (b is null) continue;
			if (b.hasTag("fire source") || b.isInFlames())
			{
				reqs = true;
			}
		}
	}
	
    this.set_bool("arrow reqs", reqs);
	return reqs;
}

void onTick( CBlob@ this )
{
    Knocked(this);

    u8 charge_state = this.get_u8( "charge_state" );
    
	const bool stab = charge_state == ArcherParams::not_aiming && this.isKeyJustPressed( key_action2 ) && !this.isKeyPressed( key_down );
	if (stab) {
		charge_state = ArcherParams::stabbing;
		this.set_u8( "charge_state", charge_state );
		this.set_u8( "stab delay", STAB_DELAY );
	}

	if (charge_state == ArcherParams::stabbing)			// pick arrow from tree anim
	{
		CSprite@ sprite = this.getSprite();

		u8 delay = this.get_u8( "stab delay" );
		if (delay == STAB_DELAY) 
		{		
			// fletch arrow
			if ( this.getMap().getSectorAtPosition( this.getPosition() + Vec2f(this.isFacingLeft() ? -5.0f : 5.0f, 0.0f), "tree" ) !is null )	// make arrow from tree
			{
				if (getNet().isServer())
				{
					CBlob@ mat_arrows = server_CreateBlob( "mat_arrows", this.getTeamNum(), this.getPosition() );
					if (mat_arrows !is null)
					{
						mat_arrows.server_SetQuantity(fletch_num_arrows);
						mat_arrows.Tag("do not set materials");
						this.server_PutInInventory( mat_arrows );
					}
				}
				this.getSprite().PlaySound( "Entities/Items/Projectiles/Sounds/ArrowHitGround.ogg" );
			}

			// STAB
			sprite.PlaySound("KnifeStab");
			if (getNet().isServer()){
				Stab(this);
			}
		}

		if (delay == 0 || delay > STAB_DELAY) {
			charge_state = ArcherParams::not_aiming;
			this.set_u8( "charge_state", charge_state );
		}

		delay--;
		this.set_u8( "stab delay", delay );
	}

	// vvvvvvvvvvvvvv CLIENT-SIDE ONLY vvvvvvvvvvvvvvvvvvv

	if (!getNet().isClient()) return;

	RunnerMoveVars@ moveVars;
	if (!this.get( "moveVars", @moveVars )) {
		return;
	}		  
	bool ismyplayer = this.isMyPlayer();
	bool hasarrow = this.get_bool( "has_arrow" );
	s8 charge_time = this.get_s8( "charge_time" );

	if (charge_state != ArcherParams::stabbing)
	{
		if ((getGameTime()+this.getNetworkID())%10 == 0)
		{
			hasarrow = hasArrows( this );

			if (!hasarrow)  {                // set back to default
				this.set_u8( "arrow type", ArrowType::normal );
			}
		}

		if (this.isKeyPressed(key_action1))
		{
			moveVars.walkFactor *= 0.7f;
			moveVars.canVault = false;

			if (ismyplayer && hasarrow && getCamera() !is null)
			{
				getCamera().mousecamstyle = 1;
			}

			if (charge_state == ArcherParams::not_aiming || charge_state == ArcherParams::fired)
			{
				charge_state = ArcherParams::readying;
				hasarrow = hasArrows( this );

				if (!hasarrow)
				{
					charge_state = ArcherParams::no_arrows;
					charge_time = 0;

					if (this.isMyPlayer() && !this.wasKeyPressed(key_action1)) { // playing annoying no ammo sound
						Sound::Play("Entities/Characters/Sounds/NoAmmo.ogg");
					}

					// set back to default
					this.set_u8( "arrow type", ArrowType::normal );
				}
				else
				{
					if (ismyplayer)
					{
				
						if (!hasReqs( this, this.get_u8( "arrow type" ) ) && !this.wasKeyPressed(key_action1)) { // playing annoying no ammo sound
							Sound::Play("Entities/Characters/Sounds/NoAmmo.ogg");
						}
					}
					
					this.getSprite().RewindEmitSound();
					this.getSprite().SetEmitSoundPaused( false );

					if (!ismyplayer) { // lower the volume of other players charging  - ooo good idea
						this.getSprite().SetEmitSoundVolume( 0.5f );
					}
				}
			}
			else if (charge_state == ArcherParams::readying)
			{
				charge_time++;

				if(charge_time > ArcherParams::ready_time)
				{
					charge_time = 1;
					charge_state = ArcherParams::charging;
				}
			}
			else if (charge_state == ArcherParams::charging)
			{
				charge_time++;

				if(charge_time >= ArcherParams::shoot_period)
				{
					charge_state = ArcherParams::decharging;
					this.getSprite().SetEmitSoundPaused( true );
				}
			}
			else if (charge_state == ArcherParams::decharging)
			{
				charge_time--;

				if(charge_time == 0)
				{
					charge_state = ArcherParams::charging;
					this.getSprite().RewindEmitSound();
					this.getSprite().SetEmitSoundPaused( false );
				}
			}
			else if (charge_state == ArcherParams::no_arrows)
			{
				if(charge_time < ArcherParams::ready_time) {
					charge_time++;
				}
			}
		}
		else
		{
			if (charge_state > ArcherParams::readying)
			{
				if (charge_state < ArcherParams::fired)
				{
					//time to fire!
					if (hasarrow && ismyplayer) // client-logic
					{
						f32 arrowspeed;

						if (charge_time < (ArcherParams::shoot_period/3)) {
							arrowspeed = ArcherParams::shoot_max_vel * (1.0f/3.0f);
						}
						else if (charge_time < (2*ArcherParams::shoot_period/3)) {
							arrowspeed = ArcherParams::shoot_max_vel * (2.0f/3.0f);
						}
						else {
							arrowspeed = ArcherParams::shoot_max_vel;
						}

						ShootArrow( this, this.getPosition() + Vec2f(0.0f,-2.0f), this.getAimPos()+Vec2f(0.0f,-2.0f), arrowspeed );
					}

					charge_time = ArcherParams::fired_time;
					charge_state = ArcherParams::fired;
				}
				else //fired..
				{
					charge_time--;

					if (charge_time <= 0) {
						charge_state = ArcherParams::not_aiming;
						charge_time = 0;
					}
				}
			}
			else {
				charge_state = ArcherParams::not_aiming;    //set to not aiming either way
				charge_time = 0;
			}

			this.getSprite().SetEmitSoundPaused( true );
		}
	}

	// my player!

    if ( ismyplayer )
    {			
		// activate/throw

        if (this.isKeyJustPressed(key_action3))
        {
            CBlob@ carried = this.getCarriedBlob();

            if (carried !is null)
            {
                client_SendThrowOrActivateCommand( this );
            }
            else if (this.getBlobCount("satchel") > 0)
            {
                this.SendCommand( this.getCommandID("get satchel") );
            }
        }
    		
		//this.Sync( "has_arrow" , false );

		// pick up arrow

		u8 cooldown = this.get_u8( "fletch_cooldown" );
		if (cooldown > 0) {
			cooldown--;
			this.set_u8( "fletch_cooldown", cooldown );
		}

		// pickup from ground

        if (cooldown == 0 && this.isKeyPressed(key_action2))
        {
            if (getPickupArrow( this ) !is null /*|| canPickSpriteArrow( this, false ) doesnt work*/) // pickup arrow from ground
            {
                this.SendCommand( this.getCommandID("pickup arrow") );
				this.set_u8( "fletch_cooldown", PICKUP_COOLDOWN );
            }
        }
    }   

	this.set_s8( "charge_time", charge_time );
	this.set_u8( "charge_state", charge_state );
	this.set_bool( "has_arrow" , hasarrow );
}

void ShootArrow( CBlob @this, Vec2f arrowPos, Vec2f aimpos, f32 arrowspeed )
{
    if (!this.isMyPlayer()) {
        return;
    }

    f32 randomInn = 0.0f -2.0f + (( f32(Random(2048)) / 2048.0f) * 4.0f);
    Vec2f arrowVel = (aimpos- arrowPos).RotateBy(randomInn,Vec2f(0,0));
    arrowVel.Normalize();
    arrowVel *= arrowspeed;
    //print("arrowspeed " + arrowspeed);
    CBitStream params;
    params.write_Vec2f( arrowPos );
    params.write_Vec2f( arrowVel );
	params.write_u8( this.get_u8( "arrow type" ) );
    this.SendCommand( this.getCommandID("shoot arrow"), params );
}

CBlob@ getPickupArrow( CBlob@ this )
{
	CBlob@[] blobsInRadius;
	if (this.getMap().getBlobsInRadius( this.getPosition(), this.getRadius()*1.5f, @blobsInRadius ))
	{
		for (uint i = 0; i < blobsInRadius.length; i++)
		{
			CBlob @b = blobsInRadius[i];
			if (b.getName() == "arrow")
			{
				return b;
			}
		}
	}
    return null;
}

bool canPickSpriteArrow( CBlob@ this, bool takeout )
{
	CBlob@[] blobsInRadius;
	if (this.getMap().getBlobsInRadius( this.getPosition(), this.getRadius()*1.5f, @blobsInRadius ))
	{
		for (uint i = 0; i < blobsInRadius.length; i++)
		{
			CBlob @b = blobsInRadius[i];
			{
				CSprite@ sprite = b.getSprite();
				if (sprite.getSpriteLayer("arrow") !is null)
				{
					if (takeout)
						sprite.RemoveSpriteLayer("arrow");
					return true;
				}
			}
		}
	}
	return false;
}

void onCommand( CBlob@ this, u8 cmd, CBitStream @params )
{
    if (cmd == this.getCommandID("shoot arrow"))
    {
        Vec2f arrowPos = params.read_Vec2f();
        Vec2f arrowVel = params.read_Vec2f();
        u8 arrowType = params.read_u8();
		this.set_u8( "arrow type", arrowType );

		// return to normal arrow - server didnt have this synced
		if (!hasArrows( this, arrowType )) {
			return;
		}

        if (getNet().isServer())
        {
            CBlob@ arrow = server_CreateBlobNoInit( "arrow" );	
            if (arrow !is null)
            {
				// fire arrow?
				bool hadreqs = hasReqs(this, arrowType);
				if (arrowType == ArrowType::fire && hadreqs)
				{
					arrow.Tag("fire arrow");
				}
				arrow.Init();

				arrow.IgnoreCollisionWhileOverlapped( this );
                arrow.SetDamageOwnerPlayer( this.getPlayer() );
				arrow.setTeamNum( this.getTeamNum() );
				arrow.setPosition( arrowPos );
                arrow.setVelocity( arrowVel );
            }
        }

        this.getSprite().PlaySound( "Entities/Characters/Archer/BowFire.ogg" );
        this.TakeBlob( arrowTypeNames[ arrowType ], 1 );

		this.set_u8( "fletch_cooldown", FLETCH_COOLDOWN ); // just don't allow shoot + make arrow
    }
    else if (cmd == this.getCommandID("pickup arrow"))
    {
        CBlob@ arrow = getPickupArrow( this );
		bool spriteArrow = canPickSpriteArrow( this, false );
        if (arrow !is null || spriteArrow)
        {
			CBlob@ mat_arrows = server_CreateBlob( "mat_arrows", this.getTeamNum(), this.getPosition() );
			if (mat_arrows !is null)
			{
				mat_arrows.server_SetQuantity(fletch_num_arrows);
				mat_arrows.Tag("do not set materials");
				this.server_PutInInventory( mat_arrows );

				if (arrow !is null) {
					arrow.server_Die();
				}
				else{
					canPickSpriteArrow( this, true );
				}
			}
			this.getSprite().PlaySound( "Entities/Items/Projectiles/Sounds/ArrowHitGround.ogg" );
        }
    }
    else if (cmd == this.getCommandID("get satchel") && getNet().isServer())
    {
        CInventory@ inv = this.getInventory();

        if (inv !is null)
        {
            CBlob@ blob = this.server_PutOutInventory("satchel");

            if (blob !is null)
            {
                this.server_Pickup( blob );
                server_ActivateCommand( this, blob );
            }
        }
    }
    else 
    {
        for (uint i = 0; i < arrowTypeNames.length; i++)
        {
            if (cmd == this.getCommandID( "pick " + arrowTypeNames[i]))
            {
                this.set_u8( "arrow type", i );
                break;
            }
        }
    }
}

// arrow pick menu

void onCreateInventoryMenu( CBlob@ this, CBlob@ forBlob, CGridMenu @gridmenu )
{
    if (arrowTypeNames.length == 0) {
        return;
    }

    this.ClearGridMenusExceptInventory();
    Vec2f pos( gridmenu.getUpperLeftPosition().x + 0.5f*(gridmenu.getLowerRightPosition().x - gridmenu.getUpperLeftPosition().x),
               gridmenu.getUpperLeftPosition().y - 32 * 1 - 24 );
    CGridMenu@ menu = CreateGridMenu( pos, this, Vec2f( arrowTypeNames.length, 1 ), "Current arrow" );
    u8 arrowSel = this.get_u8( "arrow type" );

    if (menu !is null)
    {
		menu.deleteAfterClick = false;

        for (uint i = 0; i < arrowTypeNames.length; i++)
        {
            string matname = arrowTypeNames[i];
            CGridButton @button = menu.AddButton( i == 0 ? "$Arrow$" : "$FireArrow$", arrowNames[i], this.getCommandID( "pick " + matname) );

            if (button !is null)
            {
				bool enabled = this.getBlobCount( arrowTypeNames[i] ) > 0;
                button.SetEnabled( enabled );
				button.selectOneOnClick = true;
			
				if (enabled && i == ArrowType::fire && !hasReqs(this, i))
				{
					button.hoverText = "Requires a fire source $lantern$";
					//button.SetEnabled( false );
				}
			
                if (arrowSel == i) {
                    button.SetSelected(1);
                }
			}				
        }
    }
}

// auto-switch to appropriate arrow when picked up
void onAddToInventory( CBlob@ this, CBlob@ blob )
{
	CInventory@ inv = this.getInventory();
	if (inv.getItemsCount() == 0)
	{
		string name = blob.getName();
		for (uint i = 0; i < arrowTypeNames.length; i++)
		{
			if (name == arrowTypeNames[i])
			{
				this.set_u8( "arrow type", i );
			}
		}
	}
}


void Stab( CBlob@ this )
{	   				
	Vec2f blobPos = this.getPosition();
	Vec2f aimPos = this.getAimPos();
	Vec2f aimDir = aimPos - blobPos;
	f32 aimangle = aimDir.Angle();
	Vec2f pos = blobPos - Vec2f(2,0).RotateBy(-aimangle);
	f32 attack_distance = this.getRadius() + 16.0f;
	f32 damage;
	f32 radius = this.getRadius();
	CMap@ map = this.getMap();

	HitInfo@[] hitInfos;  
	if (map.getHitInfosFromArc( pos, -aimangle, 180.0f, attack_distance, this, @hitInfos ))
	{
		//HitInfo objects are sorted, first come closest hits
		for (uint i = 0; i < hitInfos.length; i++)
		{
			HitInfo@ hi = hitInfos[i];
			CBlob@ b = hi.blob;
			if (b !is null) // blob
			{						  
				const bool large = b.getMass() > 500;
				if (b.getTeamNum() == this.getTeamNum() && !b.hasTag("dead")) { // no TK
					if (large) return;
					else continue;
				}
				if (b.hasTag("invincible"))	{
					continue;
				}

				Vec2f otherPos = b.getPosition();
				if (b.getHealth() > 0.0f && b.hasTag("flesh") && !b.hasTag("dead") &&
					((otherPos.x > blobPos.x && !this.isFacingLeft() && !b.isFacingLeft()) ||
					 (otherPos.x < blobPos.x && this.isFacingLeft() && b.isFacingLeft())
					)
					)
				{
					damage = 3.0f;
				}
				else
				{
					damage = 0.25f;
				}

				Vec2f velocity = otherPos - pos;
				this.server_Hit( b, hi.hitpos, velocity, damage, Hitters::stab, true); // server_Hit() is server-side only

				if (large) return;
			}
		}
	}	   
}
