// Builder animations

#include "BuilderCommon.as"

void onInit( CSprite@ this )
{
	const string texname = this.getBlob().getSexNum() == 0 ?
		"Entities/Characters/Builder/BuilderMale.png" :
		"Entities/Characters/Builder/BuilderFemale.png";
	this.ReloadSprite( texname );

	this.getCurrentScript().runFlags |= Script::tick_not_infire;
}


void onTick( CSprite@ this )
{
    // store some vars for ease and speed
    CBlob@ blob = this.getBlob();
    u8 knocked = blob.get_u8("knocked");
    Vec2f pos = blob.getPosition();
    Vec2f aimpos;
    CMovement @movement = blob.getMovement();
    if (movement is null) { // can't animate without keys
        return;
    }
    aimpos = movement.getVars().aimpos;
    bool inair = (!blob.isOnGround() && !blob.isOnLadder());

	Vec2f vel = blob.getVelocity();

    if (this.getBlob().hasTag("dead"))
    {
        this.SetAnimation("dead");
        Vec2f oldvel = this.getBlob().getOldVelocity();

        if (vel.y < -1.0f) {
            this.SetFrameIndex( 0 );
        }
        else if (vel.y > 1.0f) {
            this.SetFrameIndex( 2 );
        }
        else {
            this.SetFrameIndex( 1 );
        }		 
        return;
    }

	// set facing
    bool facingLeft = this.isFacingLeft();	
    // get the angle of aiming with mouse
    Vec2f vec = aimpos - pos;
    f32 angle = vec.Angle();
    int direction;

    if ((angle > 330 && angle < 361) || (angle > -1 && angle < 30) ||
            (angle > 150 && angle < 210)) {
        direction = 0;
    }
    else if (aimpos.y < pos.y) {
        direction = -1;
    }
    else {
        direction = 1;
    }

	const bool left = blob.isKeyPressed(key_left);
	const bool right = blob.isKeyPressed(key_right);
	const bool up = blob.isKeyPressed(key_up);
	const bool down = blob.isKeyPressed(key_down);

    // animations

    //if (!blob.isInFlames()) //give way to burning anim
    {
        bool ended = this.isAnimationEnded();

        if (knocked > 0)
        {
            if (inair) {
                this.SetAnimation("knocked_air");
            }
            else {
                this.SetAnimation("knocked");
            }
        }
        else if (blob.hasTag( "seated" ))
        {
            this.SetAnimation("crouch");
        }
        //wall sliding and climbing anims
        else if (blob.hasTag( "wallsliding" ))
        {
            this.SetAnimation("wall_slide");
        }
        else if (blob.hasTag("wallclimbing"))
        {
            this.SetAnimation("wall_climb");
        }
        //actual builder anims
        else if (blob.isKeyPressed(key_action2))
        {
            this.SetAnimation("strike");
        }
        else if (inair)
        {
            this.SetAnimation("fall");
            f32 vy = vel.y;
			this.animation.timer = 0;

			if (vy < -1.5 || up) {
				this.animation.frame = 0;
			}
			else if (vy > 1.5 || down) {
				this.animation.frame = 2;
			}
			else {
				this.animation.frame = 1;
			}
        }
        else if ((blob.isKeyPressed(key_left) || blob.isKeyPressed(key_right)) ||
                 (blob.isOnLadder() && (blob.isKeyPressed(key_up) || blob.isKeyPressed(key_down)) ) )
        {
            this.SetAnimation("run");
        }
        else if (blob.isKeyPressed(key_down) && ended)
        {
            this.SetAnimation("crouch");
        }
        else
        {
            this.SetAnimation("default");
        }
    }
    
    //set the attack head

	if (knocked > 0)
	{
		blob.Tag("dead head");
	}
    else if (blob.isKeyPressed(key_action2) || blob.isInFlames())
    {
        blob.Tag("attack head");
    }
    else
    {
        blob.Untag("attack head");
        blob.Untag("dead head");
    }
}

void DrawCursorAt( Vec2f pos, string& in filename )
{
    Vec2f aligned = getDriver().getScreenPosFromWorldPos( getMap().getAlignedWorldPos( pos ) );
    GUI::DrawIcon( filename, aligned, getCamera().targetDistance );
}

// render cursors

const string cursorTexture = "Entities/Characters/Sprites/TileCursor.png";

void onRender( CSprite@ this )
{
    CBlob@ blob = this.getBlob();

    if (blob.getControls() is null) {
        return;
    }

    if (!blob.isMyPlayer()) {
        return;
    }

    if (getHUD().hasButtons()) {
        return;
    }

    // draw tile cursor

    if (this.isAnimation("strike"))
    {
        Vec2f pos = blob.getPosition();
        Vec2f vector = blob.getAimPos() - pos;
        Vec2f normal = vector;
        normal.Normalize();
        f32 aimLength = vector.getLength();
        f32 angle = vector.Angle();
        f32 attack_distance = blob.getRadius() + blob.get_f32( "pickaxe_distance" );
        f32 attack_power = 0.5f;
        Vec2f attackVel(1.0f, 0.0f);
        attackVel.RotateBy( angle );
        CMap@ map = blob.getMap();
        HitData@ hitdata;
        blob.get("hitdata", @hitdata);
        CBlob@ hitBlob = hitdata.blobID > 0 ? getBlobByNetworkID( hitdata.blobID ) : null;

        if (hitBlob !is null) // blob hit
        {
            if (!hitBlob.hasTag("flesh")) {
                hitBlob.RenderForHUD( RenderStyle::outline );
            }
        }
        else // map hit
        {
            DrawCursorAt( hitdata.tilepos, cursorTexture );
        }
    }
}

void onGib(CSprite@ this)
{
    if (g_kidssafe) {
        return;
    }

    CBlob@ blob = this.getBlob();
    Vec2f pos = blob.getPosition();
    Vec2f vel = blob.getVelocity();
	vel.y -= 3.0f;
    f32 hp = Maths::Min(Maths::Abs(blob.getHealth()), 2.0f) + 1.0;
	const u8 team = blob.getTeamNum();
    CParticle@ Body     = makeGibParticle( "Entities/Characters/Builder/BuilderGibs.png", pos, vel + getRandomVelocity( 90, hp , 80 ), 0, 0, Vec2f (16,16), 2.0f, 20, "/BodyGibFall", team );
    CParticle@ Arm1     = makeGibParticle( "Entities/Characters/Builder/BuilderGibs.png", pos, vel + getRandomVelocity( 90, hp - 0.2 , 80 ), 1, 0, Vec2f (16,16), 2.0f, 20, "/BodyGibFall", team );
    CParticle@ Arm2     = makeGibParticle( "Entities/Characters/Builder/BuilderGibs.png", pos, vel + getRandomVelocity( 90, hp - 0.2 , 80 ), 1, 0, Vec2f (16,16), 2.0f, 20, "/BodyGibFall", team );
    CParticle@ Shield   = makeGibParticle( "Entities/Characters/Builder/BuilderGibs.png", pos, vel + getRandomVelocity( 90, hp , 80 ), 2, 0, Vec2f (16,16), 2.0f, 0, "Sounds/material_drop.ogg", team );
    CParticle@ Sword    = makeGibParticle( "Entities/Characters/Builder/BuilderGibs.png", pos, vel + getRandomVelocity( 90, hp + 1 , 80 ), 3, 0, Vec2f (16,16), 2.0f, 0, "Sounds/material_drop.ogg", team );
}
