// Builder logic

#include "/Entities/Common/Requirements.as"
#include "BuilderCommon.as"

namespace Builder
{
enum Cmd
{
    nil = 0,
    make_workshop = 59,
    make_block = 60,
    make_reserved = 99
};
}

const bool MAKE_WORKSHOPS = false;
Vec2f MENU_SIZE( 3, 4 );

void onInit( CInventory@ this )
{
    AddIconToken( "$EmptyShop$", "Entities/Workshops/Sprites/WorkshopIcons.png", Vec2f(16,16), 0 );
    BuildBlock[] blocks;
	
    
	{   // stone_block
        BuildBlock b( CMap::tile_castle, "stone_block", "$stone_block$", "Stone Block" );
        AddRequirement( b.reqs, "blob", "mat_stone", "Stone", 10 );
        blocks.push_back( b );
    }
    {   // back_stone_block
        BuildBlock b( CMap::tile_castle_back, "back_stone_block", "$back_stone_block$", "Back Stone Wall" );
        AddRequirement( b.reqs, "blob", "mat_stone", "Stone", 2 );
        blocks.push_back( b );
    }
	{   // stone_door
		BuildBlock b( 0, "stone_door", "$stone_door$", "Stone Door" );
		AddRequirement( b.reqs, "blob", "mat_stone", "Stone", 50 );
		blocks.push_back( b );
	}    

	{   // wood_block
		BuildBlock b( CMap::tile_wood, "wood_block", "$wood_block$", "Wood Block" );
		AddRequirement( b.reqs, "blob", "mat_wood", "Wood", 10 );
		blocks.push_back( b );
	}
	{   // back_wood_block
		BuildBlock b( CMap::tile_wood_back, "back_wood_block", "$back_wood_block$", "Back Wood Wall" );
		AddRequirement( b.reqs, "blob", "mat_wood", "Wood", 2 );
		blocks.push_back( b );
	}
	{   // wooden_door
		BuildBlock b( 0, "wooden_door", "$wooden_door$", "Wooden Door" );
		AddRequirement( b.reqs, "blob", "mat_wood", "Wood", 20 );
		blocks.push_back( b );
	}

	{   // trap
		BuildBlock b( 0, "trap_block", "$trap_block$", "Trap Block" );
		AddRequirement( b.reqs, "blob", "mat_stone", "Stone", 20 );
		blocks.push_back( b );
	}	
	{   // ladder
		BuildBlock b( 0, "ladder", "$ladder$", "Ladder" );
		AddRequirement( b.reqs, "blob", "mat_wood", "Wood", 10 );
		blocks.push_back( b );
	}
	{   // platform
		BuildBlock b( 0, "wooden_platform", "$wooden_platform$", "Wooden Platform" );
		AddRequirement( b.reqs, "blob", "mat_wood", "Wood", 20 );
		blocks.push_back( b );
	}

	{   // workbench
		BuildBlock b( 0, "workbench", "$workbench$", "Workbench" );
		AddRequirement( b.reqs, "blob", "mat_wood", "Wood", 80 );
		b.buildOnGround = true;
		blocks.push_back( b );
	}
	{   // spikes
		BuildBlock b( 0, "spikes", "$spikes$", "Spikes" );
		AddRequirement( b.reqs, "blob", "mat_stone", "Stone", 10 );
		blocks.push_back( b );
	}
	{   // fireplace
		BuildBlock b( 0, "fireplace", "$fireplace$", "Campfire" );
		AddRequirement( b.reqs, "blob", "mat_wood", "Wood", 50 );
		b.buildOnGround = true;
		blocks.push_back( b );
	}

    this.getBlob().set( "blocks", blocks );
    this.getBlob().set_Vec2f("inventory offset", Vec2f(0.0f, 176.0f));
    this.getBlob().set_u8( "buildblob", 255 );
	this.getCurrentScript().removeIfTag = "dead";
}

void MakeWorkshopButton( CInventory@ this, CBlob@ forBlob, CGridMenu @gridmenu )
{
    // make workshop
    CBitStream reqs, missing;
    AddRequirement( reqs, "blob", "mat_stone", "Stones", 10 );
    CGridButton @button = gridmenu.AddButton( "$EmptyShop$", "Make Workshop", Builder::make_workshop );

    if (button !is null)
    {
        if (this.hasRequirements( reqs, missing )) {
            button.hoverText = getButtonRequirementsText( reqs, false );
        }
        else {
            button.hoverText = getButtonRequirementsText( missing, true );
        }
    }
}

void MakeBlocksMenu( CInventory@ this, CGridMenu @invmenu )
{
    int buildtile = this.getBlob().get_TileType("buildtile");
    BuildBlock[]@ blocks;
    this.getBlob().get( "blocks", @blocks );

    if (blocks !is null)
    {
        f32 fl = blocks.length;
        Vec2f pos( invmenu.getUpperLeftPosition().x + 0.5f*(invmenu.getLowerRightPosition().x - invmenu.getUpperLeftPosition().x),
                   invmenu.getUpperLeftPosition().y - 32 * MENU_SIZE.y - 0 );
        CGridMenu@ menu = CreateGridMenu( pos, this.getBlob(), MENU_SIZE, "Build" );

        if (menu !is null)
        {
			menu.deleteAfterClick = false;

            for (uint i = 0; i < blocks.length; i++)
            {
                BuildBlock@ b = blocks[i];
                CGridButton @button = menu.AddButton( b.icon, b.description, Builder::make_block + i );

                if (button !is null)
                {
					button.selectOneOnClick = true;
                    CBitStream missing;

                    if (this.hasRequirements( b.reqs, missing )) {
                        button.hoverText = b.description + "  " + getButtonRequirementsText( b.reqs, false );
                    }
                    else
                    {
                        button.hoverText = b.description + "  " + getButtonRequirementsText( missing, true );
                        button.SetEnabled( false );
                    }

                    if (int(b.tile) == buildtile && b.tile != 0) {
                        button.SetSelected(1);
                    }

					CBlob @carryBlob = this.getBlob().getCarriedBlob(); 
					if (carryBlob !is null && carryBlob.getName() == b.name)
						button.SetSelected(1);
                }

				//if (i == 8) { // gap before spikes 
				//	menu.AddEmptyButton();
				//}
            }
        }
    }
}

void onCreateInventoryMenu( CInventory@ this, CBlob@ forBlob, CGridMenu @gridmenu )
{
    this.getBlob().ClearGridMenusExceptInventory();
    // blocks
    MakeBlocksMenu( this, gridmenu );

    if (MAKE_WORKSHOPS) {
        MakeWorkshopButton( this, forBlob, gridmenu );
    }
}

void onCommand( CInventory@ this, u8 cmd, CBitStream @params )
{
    string dbg = "BuilderInventory.as: Unknown command ";
    CBlob@ blob = this.getBlob();

    if( cmd  == Builder::make_workshop && getNet().isServer())
    {
        CBitStream reqs, missing;
        AddRequirement( reqs, "blob", "mat_stone", "Stones", 10 );

        if (this.hasRequirements( reqs, missing ))
        {
            this.server_TakeRequirements( reqs );
            CBlob @shopBlob = server_CreateBlob( "workshop", blob.getTeamNum(), blob.getPosition() );
        }
        else
        {
            print("doesn't meet requirements");
        }
    }

    if (cmd >= Builder::make_block && cmd < Builder::make_reserved)
    {
        const bool isServer = getNet().isServer();
        BuildBlock[]@ blocks;
        blob.get( "blocks", @blocks );
        uint i = cmd - Builder::make_block;

        if (blocks !is null && i >= 0 && i < blocks.length)
        {
            BuildBlock@ b = blocks[i];

            // put carried in inventory thing first

            if (isServer)
            {
                CBlob @carryBlob = blob.getCarriedBlob();

                if (carryBlob !is null)
                {
                    // check if this isn't what we wanted to create
                    if (carryBlob.getName() == b.name) {
                        return;    // this is it
                    }

					// kill this 

					if (carryBlob.hasTag("temp blob"))
					{
						carryBlob.Untag("temp blob");
						carryBlob.server_Die();			
					}
					else // try put into inventory whatever was in hands
					{
						carryBlob.server_PutInInventory( blob );
					}
                }
            }

            if (b.tile == 0) // blob block
            {
				server_BuildBlob( blob, @blocks, i );	 
            }
            else // block
            {
               blob.set_TileType( "buildtile", b.tile );
            }
        }
    }

// blob.set_u8("lastcmd", cmd );
}
