// Knight logic

#include "ThrowCommon.as"	
#include "KnightCommon.as";
#include "RunnerCommon.as";	
#include "Hitters.as";
#include "ShieldCommon.as";	
#include "Knocked.as"
#include "Help.as";

//specialised timer for code compression

void set_delay(CBlob@ this, string field, u32 delay)
{
    this.set_s32(field,getGameTime()+delay);
}

//attacks limited to the one time per-actor before reset.

void knight_actorlimit_setup(CBlob@ this)
{
    u16[] networkIDs;
    this.set( "LimitedActors", networkIDs );
}

bool knight_has_hit_actor(CBlob@ this, CBlob@ actor)
{
    u16[]@ networkIDs;
    this.get( "LimitedActors", @networkIDs );
    return networkIDs.find(actor.getNetworkID()) >= 0;
}

u32 knight_hit_actor_count(CBlob@ this)
{
    u16[]@ networkIDs;
    this.get( "LimitedActors", @networkIDs );
    return networkIDs.length;
}

void knight_add_actor_limit(CBlob@ this, CBlob@ actor)
{
    this.push( "LimitedActors", actor.getNetworkID() );
}

void knight_clear_actor_limits(CBlob@ this)
{
    this.clear( "LimitedActors" );
}

//attacking/bashing functions

const int DELTA_BEGIN_ATTACK = 4;

bool isJab(f32 damage)
{
	return damage < 1.5f;
}

void DoAttack(CBlob@ this, f32 damage, f32 aimangle, f32 arcdegrees, u8 type, int deltaInt)
{
    if (!getNet().isServer()) {
        return;
    }

    if (aimangle < 0.0f) {
        aimangle += 360.0f;
    }

    Vec2f blobPos = this.getPosition();
    Vec2f pos = blobPos - Vec2f(6,0).RotateBy(aimangle);
    f32 attack_distance = 16.0f;
    f32 radius = this.getRadius();
    CMap@ map = this.getMap();
	bool dontHitMore = false;
	bool dontHitMoreMap = false;
	const bool jab = isJab(damage);
    // this gathers HitInfo objects which contain blob or tile hit information
    HitInfo@[] hitInfos;

    if (map.getHitInfosFromArc( pos, aimangle, arcdegrees, radius + attack_distance, this, @hitInfos ))
    {
        //HitInfo objects are sorted, first come closest hits
        for (uint i = 0; i < hitInfos.length; i++)
        {
            HitInfo@ hi = hitInfos[i];
			CBlob@ b = hi.blob;	  
            if (b !is null) // blob
            {		
				const bool large = b.getMass() > 500;
				if (b.getTeamNum() == this.getTeamNum() && !b.hasTag("dead")) { // no TK
					if (large) return;
					else continue;
				}		 				
	
				if (knight_has_hit_actor(this,b)) {
					if (large) return;
					else continue;
				}							

				knight_add_actor_limit( this, b );
				if (!dontHitMore)
				{	
					Vec2f velocity = b.getPosition() - pos;
					this.server_Hit( b, hi.hitpos, velocity, damage, type, true); // server_Hit() is server-side only

					// end hitting if we hit something solid, dont if its flesh
					if (b.isCollidable() && (jab || !b.hasTag("flesh")) && b.hasTag("invincible"))
					{
						dontHitMore = true;
					}

				}
				if (large) return;
            }
			else  // hitmap
			if (!dontHitMore && !dontHitMoreMap && (deltaInt == DELTA_BEGIN_ATTACK+1))
			{  
				if (map.isTileGround( hi.tile ))
				{
					if (!jab) {
						map.server_DestroyTile( hi.hitpos, 0.1f );
						dontHitMoreMap = true;
					}
				}
			}
        }
    }

    // destroy grass

    if (((aimangle >= 0.0f && aimangle <= 180.0f) || damage > 1.0f) &&    // aiming down or slash
            (deltaInt == DELTA_BEGIN_ATTACK+1)) // hit only once
    {
        f32 tilesize = map.tilesize;
        int steps = Maths::Ceil(2*radius / tilesize);
        int sign = this.isFacingLeft()? -1 : 1;

        for (int y = 0; y < steps; y++)
            for (int x = 0; x < steps; x++)
            {
                Vec2f tilepos = blobPos + Vec2f( x * tilesize * sign, y * tilesize);
                TileType tile = map.getTile( tilepos );

                if (map.isTileGrass( tile ))
                {
                    map.server_DestroyTile( tilepos, damage );

                    if (damage <= 1.0f) {
                        return;
                    }
                }
            }
    }
}

void DoShieldBash( CBlob@ this, CBlob@ hitblob )
{
    if (!getNet().isServer()) {
        return;
    }

    bool facing_left = this.isFacingLeft();
    const f32 bash_vel = 3.0f;
    Vec2f force = Vec2f( (facing_left ? -bash_vel : bash_vel) , -bash_vel / 3 );
    Vec2f pos = this.getPosition();
    Vec2f theirpos = hitblob.getPosition();

    //printf("VEL " + this.getVelocity().Length());

    if ( this.getVelocity().Length() > 3.0f && (facing_left ? theirpos.x < pos.x : theirpos.x > pos.x) && !knight_has_hit_actor(this,hitblob) )
    {
        knight_add_actor_limit(this,hitblob);
        this.server_Hit( hitblob, pos, force , 0.0f, Hitters::shield );
    }
}

//a little push forward

void pushForward(CBlob@ this, f32 normalForce, f32 pushingForce, f32 verticalForce)
{
    f32 facing_sign = this.isFacingLeft() ? -1.0f : 1.0f ;
    bool pushing_in_facing_direction =
        (facing_sign < 0.0f && this.isKeyPressed(key_left)) ||
        (facing_sign > 0.0f && this.isKeyPressed(key_right));
    f32 force = normalForce;

    if (pushing_in_facing_direction) {
        force = pushingForce;
    }

    this.AddForce(Vec2f( force * facing_sign , verticalForce));
}

//bomb management

bool hasBomb( CBlob@ this ) //expensive as hell do not do this shit every frame
{
    CBitStream reqs, missing;
    AddRequirement( reqs, "blob", "mat_bombs", "Bombs", 1 );
    CInventory@ inv = this.getInventory();

    if (inv !is null)
    {
        return inv.hasRequirements( reqs, missing );
    }
    else {
        warn("our inventory was null! KnightLogic.as");
    }

    return false;
}

void takeBomb( CBlob@ this )
{
	CBlob@ carried = this.getCarriedBlob();
	if (carried !is null)
	{
		if (carried.getName() == "mat_bombs")
		{
			carried.server_Die();
			return;
		}
	}

    CBitStream reqs, missing;
    AddRequirement( reqs, "blob", "mat_bombs", "Bombs", 1 );
    CInventory@ inv = this.getInventory();

    if (inv !is null)
    {
        if (inv.hasRequirements( reqs, missing )) {
            inv.server_TakeRequirements( reqs );
        }
        else {
            warn("took a bomb even though we dont have one! KnightLogic.as");
        }
    }
    else {
        warn("our inventory was null! KnightLogic.as");
    }
}

//Actual hook functions

void onInit( CBlob@ this )
{
	KnightInfo knight;

    knight.state = KnightStates::normal;
    knight.swordTimer = 0;
	knight.shieldTimer = 0;
	knight.doubleslash = false;

	this.set("knightInfo", @knight);

	this.set_f32("gib health", -1.5f);
    this.set_s32("shield down", getGameTime());
    addShieldVars(this, 220.0f, 2.0f, 5.0f);
    knight_actorlimit_setup(this);
    this.getShape().SetRotationsAllowed(false);
    this.Tag("player");
    this.Tag("flesh");

    this.addCommandID("get bomb");
    this.push("names to activate", "bomb");								       
    setKnockable( this );								
	
	SetHelp( this, "help self action", "knight", "Press to jab\nHold to slash \n\n         $LMB$", "", 2 ); 
	SetHelp( this, "help self action2", "knight", "Hold to shield \n\n         $RMB$" ); 

	this.getCurrentScript().runFlags |= Script::tick_not_attached;
	this.getCurrentScript().removeIfTag = "dead";
}

void onTick( CBlob@ this )
{
    u8 knocked = Knocked(this);
    Vec2f pos = this.getPosition();
    Vec2f aimpos = this.getAimPos();
    const bool inair = (!this.isOnGround() && !this.isOnLadder());
	Vec2f vec;
	const int direction = this.getAimDirection( vec );
	const f32 side = (this.isFacingLeft() ? 1.0f : -1.0f);

	//knight logic stuff
    //get the vars to turn various other scripts on/off
    RunnerMoveVars@ moveVars;
    if (!this.get( "moveVars", @moveVars )) {
        return;
    }

	KnightInfo@ knight;
	if (!this.get( "knightInfo", @knight )) {
        return;
    }

	bool shieldState = isShieldState(knight.state);
	bool specialShieldState = isSpecialShieldState(knight.state);
	bool swordState = isSwordState(knight.state);

	bool pressed_a1 = this.isKeyPressed(key_action1);
	bool pressed_a2 = this.isKeyPressed(key_action2);

	bool walking = (this.isKeyPressed(key_left) || this.isKeyPressed(key_right));

    if ( knocked > 0 )
    {
        knight.state = KnightStates::normal; //cancel any attacks or shielding
        knight.swordTimer = 0;
		knight.shieldTimer = 0;

		pressed_a1 = false;
		pressed_a2 = false;
		walking = false;
    }
    else if ( !pressed_a1 && !swordState &&
			( pressed_a2 || (specialShieldState && knight.state != KnightStates::shieldbashingprep) ) )
    {
        moveVars.jumpFactor *= 0.5f;
        moveVars.walkFactor *= 0.8f;
		knight.swordTimer = 0;

        if (!canRaiseShield(this))
        {
            if (knight.state != KnightStates::normal) {
                set_delay(this, "shield down", 40);
            }

            knight.state = KnightStates::normal;

            if (pressed_a2 && ((this.get_s32("shield down") - getGameTime()) <= 0))
            {
                resetShieldKnockdown( this ); //re-put up the shield
            }
        }
        else
        {
            if (pressed_a2 && /*state != KnightStates::shieldbashing &&*/ inair && !this.isInWater())
            {
                if (direction == -1 && !getMap().isInWater( pos + Vec2f(0,16) ) && !this.hasTag("wallsliding"))
                {
                    knight.state = KnightStates::shieldgliding;
                    knight.shieldTimer = 1;
                }
                else if (direction == 1)
                {
                    knight.state = KnightStates::shielddropping;
                    knight.shieldTimer = 5;
                }
            }

            if (knight.state == KnightStates::shieldbashing)
            {
                
            }

            else if (knight.state == KnightStates::shieldgliding && !this.isInWater())
            {
                if (this.getVelocity().y > -1.0f) {
                    this.AddForce(Vec2f( 0,-20.0f));
                }

                if (!inair || !this.isKeyPressed(key_action2)) {
                    knight.state = KnightStates::shielding;
                }
            }
            else if (knight.state == KnightStates::shielddropping)
            {
                if (this.isInWater() )
                {
                    Vec2f vel = this.getVelocity();

                    if (vel.y > 1.5f && Maths::Abs(vel.x)*3 > Maths::Abs(vel.y))
                    {
                        vel.y = Maths::Max(-Maths::Abs(vel.y) + 1.0f,-8.0);
                        this.setVelocity(vel);
                    }
                    else
                    {
                        knight.state = KnightStates::shielding;
                    }
                }

                if (!inair || !this.isKeyPressed(key_action2)) {
                    knight.state = KnightStates::shielding;
                }
            }
            else
            {
                knight.state = KnightStates::shielding;
				knight.shieldTimer = 2;
            }
        }
    }
    else if ( (pressed_a1 || swordState) && !this.hasTag("wallsliding") ) //no attacking during a slide
    {
		if(knight.swordTimer == KnightVars::slash_charge_level2)
		{
			Sound::Play("AnimeSword.ogg", pos);
		}
		
		if(knight.swordTimer > KnightVars::slash_charge_limit)
		{
			Sound::Play("/Stun", pos);
			SetKnocked(this, 15);
		}
		
		bool strong = ( knight.swordTimer > KnightVars::slash_charge_level2 );
        moveVars.jumpFactor *= ( strong ? 0.5f : 0.8f);
        moveVars.walkFactor *= ( strong ? 0.3f : 0.7f);
		knight.shieldTimer = 0;

        if (!inair)
		{
            this.AddForce(Vec2f(this.getVelocity().x * -5.0,0.0f));    //horizontal slowing force (prevents SANICS)
        }

        if ( this.isKeyJustPressed(key_action1) && (!inMiddleOfAttack(knight.state) ||
                shieldState || knight.state == KnightStates::normal ) )
        {
            knight.state = KnightStates::sword_drawn;
			knight.swordTimer = 0;
        }

        if (knight.state == KnightStates::sword_drawn && getNet().isServer()) {
            knight_clear_actor_limits(this);
        }

        //responding to releases/noaction
        s32 delta = knight.swordTimer;
		if (knight.swordTimer < 128)
			knight.swordTimer++;

        if (knight.state == KnightStates::sword_drawn && !pressed_a1 &&
			!this.isKeyJustReleased(key_action1) && delta > KnightVars::resheath_time)
        {
            knight.state = KnightStates::normal;
        }
        else if (this.isKeyJustReleased(key_action1) && knight.state == KnightStates::sword_drawn )
        {
			knight.swordTimer = 0;

            if (delta < KnightVars::slash_charge)
            {
                if (direction == -1) {
                    knight.state = KnightStates::sword_cut_up;
                }
                else if (direction == 0)
                {
                    if (aimpos.y < pos.y) {
                        knight.state = KnightStates::sword_cut_mid;
                    }
                    else {
                        knight.state = KnightStates::sword_cut_mid_down;
                    }
                }
                else {
                    knight.state = KnightStates::sword_cut_down;
                }
            }
            else if (delta < KnightVars::slash_charge_level2)
            {
                knight.state = KnightStates::sword_power;
                Vec2f aiming_direction = this.getVelocity();
                aiming_direction.y *= 2;
                aiming_direction.Normalize();
                this.set_Vec2f("slash_direction", aiming_direction);
            }
			else if (delta < KnightVars::slash_charge_limit)
            {
                knight.state = KnightStates::sword_power_super;
                Vec2f aiming_direction = this.getVelocity();
                aiming_direction.y *= 2;
                aiming_direction.Normalize();
                this.set_Vec2f("slash_direction", aiming_direction);
            }
            else
            {
                //knock?
            }
        }
        else if (knight.state >= KnightStates::sword_cut_mid &&
				knight.state <= KnightStates::sword_cut_down) // cut state
        {
            if (delta == DELTA_BEGIN_ATTACK) {
                Sound::Play( "/SwordSlash", this.getPosition() );
            }

            if (delta > DELTA_BEGIN_ATTACK && delta < 8)
            {
                f32 attackarc = 80.0f;
                f32 attackAngle = getCutAngle( this, knight.state );

                if (knight.state == KnightStates::sword_cut_down)
                {
                    attackarc *= 0.9f;
                }

                DoAttack(this, 1.0f, attackAngle, attackarc, Hitters::sword, delta);
            }
            else if (delta >= 12)
            {
                knight.swordTimer = 0;
				knight.state = KnightStates::sword_drawn;
            }
        }
        else if (knight.state == KnightStates::sword_power || knight.state == KnightStates::sword_power_super)
        {
            //attacking + noises
            if (delta == 2)
            {
                Sound::Play( "/ArgLong", this.getPosition() );
                Sound::Play( "/SwordSlash", this.getPosition() );
            }
            else if (delta > DELTA_BEGIN_ATTACK && delta < 10)
            {
                DoAttack(this, 2.0f, -(vec.Angle()), 120.0f, Hitters::sword, delta );
            }
            else if (delta >= KnightVars::slash_time)
            {
				knight.swordTimer = 0;
                knight.state = knight.doubleslash ? KnightStates::sword_power : KnightStates::sword_drawn;
				
				if (knight.doubleslash)
				{
					knight_clear_actor_limits(this);
					knight.doubleslash = false;
				}
            }

			if (delta > KnightVars::slash_time / 2 )//&& knight_hit_actor_count(this) > 0)
			{
				//double 
				if (knight.state == KnightStates::sword_power_super &&
					this.isKeyJustReleased(key_action1))
				{
					knight.doubleslash = true;
				}
			}
        }

        //special slash movement

		Vec2f vel = this.getVelocity();
        if ( (knight.state == KnightStates::sword_power && delta < KnightVars::slash_move_time) &&
			 Maths::Abs(vel.x) < KnightVars::slash_move_max_speed &&
			 vel.y > -KnightVars::slash_move_max_speed )
        {
            Vec2f slash_vel = this.get_Vec2f("slash_direction") * this.getMass() * 0.7f;
            this.AddForce(slash_vel);
        }
        
        moveVars.canVault = false;
		
    }
    else if (this.isKeyJustReleased(key_action2) || this.isKeyJustReleased(key_action1) || this.get_u32("knight_timer") <= getGameTime())
    {
        knight.state = KnightStates::normal;
    }

    //throwing bombs

    if ( this.isMyPlayer() )
    {
        if (this.isKeyJustPressed(key_action3))
        {
            CBlob @carried = this.getCarriedBlob();

            if (carried !is null)
            {
				if (carried.getName() == "mat_bombs")
					this.SendCommand( this.getCommandID("get bomb") );
				else
					client_SendThrowOrActivateCommand( this );
            }
            else if (hasBomb(this))
            {
                this.SendCommand( this.getCommandID("get bomb") );
            }
            else
            {
                if (this.isMyPlayer()) { // playing annoying no ammo sound
                    Sound::Play("NoAmmo.ogg");
                }
            }
        }
    }

    //setting the shield direction properly
    if (shieldState)
    {
        int horiz = this.isFacingLeft() ? -1 : 1;
        setShieldEnabled( this, true );

        if (specialShieldState)
        {
            if (knight.state == KnightStates::shieldgliding)
            {
                setShieldDirection( this, Vec2f(0,-1) );
            }
            else if (knight.state == KnightStates::shieldbashing)
            {
                setShieldDirection( this, Vec2f(horiz,0) );
            }
            else //shield dropping
            {
                setShieldDirection( this, Vec2f(horiz,2) );
            }
        }
        else if (walking)
        {
            if (direction == 0) //forward
            {
                setShieldDirection( this, Vec2f(horiz,0) );
            }
            else if (direction == 1) { //down
                setShieldDirection( this, Vec2f(horiz,2) );
            }
            else
            {
                setShieldDirection( this, Vec2f(horiz,-2) );
            }
        }
        else
        {
            if (direction == 0) { //forward
                setShieldDirection( this, Vec2f(horiz,0) );
            }
            else if (direction == 1) { //down
                setShieldDirection( this, Vec2f(horiz,2) );
            }
            else //up
            {
                if (vec.y < -0.97) {
                    setShieldDirection( this, Vec2f(0,-1) );
                }
                else {
                    setShieldDirection( this, Vec2f(horiz,-2) );
                }
            }
        }

        // shield up = ladder

        if (knight.state == KnightStates::shielding && direction == -1 && !inair)
        {
            this.getShape().getVars().isladder = true;
        }
        else
        {
            this.getShape().getVars().isladder = false;
        }
    }
    else
    {
        setShieldEnabled( this, false );
		this.getShape().getVars().isladder = false;
    }

    if (!swordState && knight.state != KnightStates::shieldbashing && getNet().isServer()) {
        knight_clear_actor_limits(this);
    }
}

void onCommand( CBlob@ this, u8 cmd, CBitStream @params )
{
    if (cmd == this.getCommandID("get bomb") && hasBomb(this))
    {
		if (getNet().isServer())
		{
			CBlob @blob = server_CreateBlob( "bomb", this.getTeamNum(), this.getPosition() ); 
			if (blob !is null)
			{
				takeBomb(this);
				this.server_Pickup( blob );
				// activated - in Bomb.as
			}
		}
		this.Tag( "mat_bombs" + " done activate" );
    }
}


f32 onHit( CBlob@ this, Vec2f worldPoint, Vec2f velocity, f32 damage, CBlob@ hitterBlob, u8 customData )
{
    // play cling sound if other knight attacked us
    // dmg could be taken out here if we ever want to

    if (hitterBlob.getPosition().x < this.getPosition().x && hitterBlob.getName() == "knight") // knight and the left one (to play only once)
    {
        CSprite@ sprite = this.getSprite();
        CSprite@ hsprite = hitterBlob.getSprite();

        if (hsprite.isAnimation( "strike_power_ready" ) || hsprite.isAnimation( "strike_mid" ) ||
                hsprite.isAnimation( "strike_mid_down" ) || hsprite.isAnimation( "strike_up" ) ||
                hsprite.isAnimation( "strike_down" ) || hsprite.isAnimation( "strike_up" ) )
        {
            if (sprite.isAnimation( "strike_power_ready" ) || sprite.isAnimation( "strike_mid" ) ||
                    sprite.isAnimation( "strike_mid_down" ) || sprite.isAnimation( "strike_up" ) ||
                    sprite.isAnimation( "strike_down" ) || sprite.isAnimation( "strike_up" )) {
                this.getSprite().PlaySound( "SwordCling" );
            }
        }
    }

    return damage; //no block, damage goes through
}

f32 getCutAngle( CBlob@ this, int state )
{
    f32 attackAngle = (this.isFacingLeft()?180.0f:0.0f);

    if (state == KnightStates::sword_cut_mid) {
        attackAngle += (this.isFacingLeft()?30.0f:-30.0f);
    }
    else if (state == KnightStates::sword_cut_mid_down) {
        attackAngle -= (this.isFacingLeft()?30.0f:-30.0f);
    }
    else if (state == KnightStates::sword_cut_up) {
        attackAngle += (this.isFacingLeft()?80.0f:-80.0f);
    }
    else if (state == KnightStates::sword_cut_down) {
        attackAngle -= (this.isFacingLeft()?80.0f:-80.0f);
    }

    return attackAngle;
}

f32 getCutAngle( CBlob@ this )
{
    Vec2f aimpos = this.getMovement().getVars().aimpos;
    int tempState;
    Vec2f vec;
    int direction = this.getAimDirection( vec );

    if (direction == -1) {
        tempState = KnightStates::sword_cut_up;
    }
    else if (direction == 0)
    {
        if (aimpos.y < this.getPosition().y) {
            tempState = KnightStates::sword_cut_mid;
        }
        else {
            tempState = KnightStates::sword_cut_mid_down;
        }
    }
    else {
        tempState = KnightStates::sword_cut_down;
    }

    return getCutAngle( this, tempState );
}



void onHitBlob( CBlob@ this, Vec2f worldPoint, Vec2f velocity, f32 damage, CBlob@ hitBlob, u8 customData )
{
	if (isJab(damage) && blockAttack(hitBlob, velocity, 0.0f))
	{
		this.getSprite().PlaySound("/Stun");
		SetKnocked( this, 20 );
	}
}
