
// We want to be able to climb dirt walls since there's no notching and
// it can be annoying to get around

#include "RunnerCommon.as";

void onInit( CMovement@ this )
{
	this.getCurrentScript().runFlags |= Script::tick_not_inwater;
	this.getCurrentScript().runFlags |= Script::tick_onmap;
	this.getCurrentScript().runFlags |= Script::tick_not_attached;
	this.getCurrentScript().removeIfTag = "dead";
}

void onTick( CMovement@ this )
{
    CBlob@ blob = this.getBlob();

	if (blob.isKeyPressed(key_action1) || blob.isKeyPressed(key_action2) || blob.isKeyPressed(key_action3))
    {
        //blob.Untag("wallclimbing");
        return;
    }
	RunnerMoveVars@ moveVars;
	if (!blob.get( "moveVars", @moveVars )) {
		return;	
	}
	CMap@ map = getMap();


    Vec2f pos = blob.getPosition();
    Vec2f vel = blob.getVelocity();
    Vec2f force;
    bool pressed = false;

    //must be pressing onto facing wall
    if (blob.isFacingLeft() && blob.isKeyPressed( key_left ))
    {
        force.x -= 0.5f;
    }
    else if (!blob.isFacingLeft() && blob.isKeyPressed( key_right ))
    {
        force.x += 0.5f;
    }
    else
    {
        return;
    }

    bool climbing = false;

    if ( //blob.isOnMap() && blob.isOnWall() &&	 script opt
            !blob.isOnCeiling() && !blob.isOnGround() &&
            !blob.isKeyPressed(key_down))
    {
        CMap@ map = blob.getMap();
        f32 offset = force.x == 0.0f ? 0.0f : (force.x > 0) ? 1.2f : -1.2f;

        if (climbableWallAhead(blob, map, offset ))
        {
            if (vel.y > -2 && vel.y < 4)
            {
                blob.Tag("wallclimbing");
                climbing = true;
            }
        }
    }

    //if (!climbing)				// done RunnerDefault.as;
    //{
    //    blob.Untag("wallclimbing");
    //}

    if (climbing)
    {
        if (blob.isKeyPressed( key_up ) && vel.y > -1.0f)
        {
            force.y -= 1.0f;
        }

        blob.AddForce(force * moveVars.overallScale * 40);
        //turn off regular movement
        moveVars.jumpFactor = 0.0f;
        moveVars.walkFactor = 0.0f;
        moveVars.canVault = false;
    }
}

bool isTileClimbable(CMap@ map, TileType tile)
{
    return ((!map.isTileCastle(tile) && !map.isTileWood(tile)) && map.isTileSolid(tile));
}

bool climbableWallAhead(CBlob@ blob, CMap@ map, f32 movingside)
{
    Vec2f pos = blob.getPosition();
    f32 tilesize = map.tilesize;
    return isTileClimbable( map , map.getTile( Vec2f( pos.x + movingside * tilesize, pos.y + tilesize*(-0.2) ) )) ||
           isTileClimbable( map , map.getTile( Vec2f( pos.x + movingside * tilesize, pos.y + tilesize*(0.3) ) )) ;
}
