// Runner Movement Stopping

#include "RunnerCommon.as"

void onInit( CMovement@ this )
{
	this.getCurrentScript().removeIfTag = "dead";
	this.getCurrentScript().runFlags |= Script::tick_not_attached;
}

void onTick( CMovement@ this )
{
    CBlob@ blob = this.getBlob();
    RunnerMoveVars@ moveVars;  
    if (!blob.get( "moveVars", @moveVars )) {
        return;
    }

    if (moveVars.jumpFactor < 0.01f)
    {
        moveVars.jumpFactor = 1.0f;
        return; 
    }

    Vec2f vel = blob.getVelocity();
    f32 vellen = blob.getShape().vellen;
    bool onground = blob.isOnGround() || blob.isOnLadder();

    if (onground)
    {
        moveVars.jumpCount = 0;
    }
    else
    {
        moveVars.jumpCount++;

        if (moveVars.jumpCount < 15) {
            moveVars.walkFactor *= 0.8f;
        }
        else {
            moveVars.walkFactor *= 0.6f;
        }
    }

    if (blob.isKeyPressed(key_up)
            && (blob.isOnGround() || !blob.isOnLadder()) //ladder movement overrides
            && vel.y > -moveVars.jumpMaxVel
            && !blob.hasTag("shielded")
       )
    {
        moveVars.jumpStart = 0.7f;
        moveVars.jumpMid = 0.2f;
        moveVars.jumpEnd = 0.1f;
        bool crappyjump = false;

        //todo what constitutes a crappy jump? maybe carrying heavy?
        if ( crappyjump )
        {
            moveVars.jumpStart *= 0.79f;
            moveVars.jumpMid *= 0.69f;
            moveVars.jumpEnd *= 0.59f;
        }

        Vec2f force = Vec2f(0,0);
        f32 side = 0.0f;

        if (blob.isFacingLeft() && blob.isKeyPressed(key_left)) {
            side = -1.0f;
        }
        else if (!blob.isFacingLeft() && blob.isKeyPressed(key_right)) {
            side = 1.0f;
        }

        // jump
        if (moveVars.jumpCount <= 0)
        {
            force.y -= 1.5f;
        }
        else if (moveVars.jumpCount < 3)
        {
            force.y -= moveVars.jumpStart;
            //force.x += side * moveVars.jumpMid;
        }
        else if (moveVars.jumpCount < 6)
        {
            force.y -= moveVars.jumpMid;
            //force.x += side * moveVars.jumpEnd;
        }
        else if (moveVars.jumpCount < 8)
        {
            force.y -= moveVars.jumpEnd;
        }

        //if (blob.isOnWall()) {
          //  force.y *= 1.1f;
        //}

        force *= moveVars.jumpFactor * moveVars.overallScale * 60.0f;


        blob.AddForce(force);

        // sound

        if (moveVars.jumpCount == 1)
        {
            TileType tile = blob.getMap().getTile( blob.getPosition() + Vec2f( 0.0f, blob.getRadius() + 4.0f ));

            if (blob.getMap().isTileGroundStuff( tile )) {
                blob.getSprite().PlaySound("/EarthJump");
            }
            else {
                blob.getSprite().PlaySound("/StoneJump");
            }
        }
    }

    moveVars.jumpFactor = 1.0f;
}
