// Runner Movement Walking

#include "RunnerCommon.as"

void onInit( CMovement@ this )
{
	this.getCurrentScript().removeIfTag = "dead";
	this.getCurrentScript().runFlags |= Script::tick_not_attached;
}

void onTick( CMovement@ this )
{
    CBlob@ blob = this.getBlob();
    RunnerMoveVars@ moveVars;
    if (!blob.get( "moveVars", @moveVars )) {
        return;
    }
	
	const bool left	= blob.isKeyPressed(key_left);
	const bool right = blob.isKeyPressed(key_right);
	if (!left && !right)
	{
		moveVars.walkFactor = 1.0f;
		return;
	}

    if (moveVars.walkFactor < 0.01f)
    {
        moveVars.walkFactor = 1.0f;
        return; //do nothing!
    }

    // carrying heavy
    CBlob@ carryBlob = blob.getCarriedBlob();		  
    if (carryBlob !is null)
    {
        if (carryBlob.hasTag("medium weight"))
        {
            moveVars.walkFactor *= 0.6f;
            moveVars.jumpFactor *= 0.8f;
        }
        else if (carryBlob.hasTag("heavy weight"))
        {
            moveVars.walkFactor *= 0.3f;
            moveVars.jumpFactor *= 0.5f;
        }
    }

    Vec2f vel = blob.getVelocity();
    f32 vellen = vel.Length();
    bool facingleft = blob.isFacingLeft();
    bool stand = blob.isOnGround() || blob.isOnLadder();
    Vec2f walkDirection;
    const f32 turnaroundspeed = 1.3f;
    const f32 normalspeed = 1.0f;
    const f32 backwardsspeed = 0.8f;

    if (right)
    {
        if (vel.x < -0.1f) {
            walkDirection.x += turnaroundspeed;
        }
        else if (facingleft) {
            walkDirection.x += backwardsspeed;
        }
        else {
            walkDirection.x += normalspeed;
        }
    }

    if (left)
    {
        if (vel.x > 0.1f) {
            walkDirection.x -= turnaroundspeed;
        }
        else if (!facingleft) {
            walkDirection.x -= backwardsspeed;
        }
        else {
            walkDirection.x -= normalspeed;
        }
    }

    f32 force = 1.0f;
    f32 lim = moveVars.walkSpeed;

    if (!stand) {
        lim = moveVars.walkSpeedInAir;
    }

    lim *= moveVars.walkFactor;
    f32 forcemul = 1.0f;
    if ((vel.x > 0.0f && vel.x > lim) || (vel.x < 0.0f && vel.x < -lim))
    {
		forcemul = (Maths::Pow((Maths::Abs(vel.x) - lim) / lim, 2.0f)*7.0f) + 1.0f;
		if (Maths::Abs(forcemul) < 0.01f) {
			moveVars.walkFactor = 1.0f;
			return;
		}
	}

    force *= 1.0f / (forcemul);
    force *= moveVars.walkFactor * moveVars.overallScale * 30.0f;

    if (Maths::Abs(force) > 0.01f) {
        blob.AddForce(walkDirection*force);
    }

    moveVars.walkFactor = 1.0f;

    //for debugging different speeds
    //if (getGameTime() % 30 ==  0)
    //print("vel: "+blob.getVelocity().x);
}

