// generic character head script

void onInit( CSprite@ this )
{
    CBlob@ blob = this.getBlob();
    LoadHead( this, blob.getHeadNum() );
}

void LoadHead( CSprite@ this, u8 headIndex )
{
    this.RemoveSpriteLayer( "head" );
    // add head
    string texname = "Entities/Characters/Sprites/Heads.png";
    CSpriteLayer@ head = this.addSpriteLayer( "head", texname, 16, 16,
                         this.getBlob().getTeamNum(),
                         this.getBlob().getSkinNum() );
    CBlob@ blob = this.getBlob();

    // set defaults
    if (headIndex == 255)
    {
        if (blob.getConfig() == "builder") {
            headIndex = 15;
        }
        else if (blob.getConfig() == "knight") {
            headIndex = 16;
        }
        else if (blob.getConfig() == "archer") {
            headIndex = 17;
        }
    }

    s32 headFrame = ((headIndex * 2) + (blob.getSexNum() == 0 ? 0 : 1))*NUM_HEADFRAMES;
	blob.set_s32("head index", headFrame );	 
    if (head !is null)
    {
        Animation@ anim = head.addAnimation( "default", 0, false );
        anim.AddFrame(headFrame);
        anim.AddFrame(headFrame+1);
        anim.AddFrame(headFrame+2);
        head.SetAnimation( anim );
    }
}

const s32 NUM_HEADFRAMES = 4;
const int FRAMES_WIDTH = 8 * NUM_HEADFRAMES;

void onGib( CSprite@ this )
{
    if (g_kidssafe) {
        return;
    }

    CBlob@ blob = this.getBlob();
	if (blob !is null)
    {
		int frame = blob.get_s32("head index");
        int framex = frame % FRAMES_WIDTH;
        int framey = frame / FRAMES_WIDTH;
        
		Vec2f pos = blob.getPosition();
		Vec2f vel = blob.getVelocity();
		f32 hp = Maths::Min(Maths::Abs(blob.getHealth()),2.0f) + 1.5;
		makeGibParticle( "Entities/Characters/Sprites/Heads.png",
							pos, vel + getRandomVelocity( 90, hp , 30 ),
							framex, framey, Vec2f (16,16),
							2.0f, 20, "/BodyGibFall", blob.getTeamNum() );
    }
}

void onTick( CSprite@ this )
{
    CBlob@ blob = this.getBlob();
    // head animations
    CSpriteLayer@ head = this.getSpriteLayer( "head" );

    if (head !is null)
    {
        // set the head offset and Z value according to the pink/yellow pixels
        PixelOffset @po = getDriver().getPixelOffset( this.getFilename(), this.getFrame() );

        if (po !is null)
        {
            // behind, in front or not drawn
            if (po.level == 0)
            {
                head.SetVisible( false );
            }
            else
            {
                head.SetVisible( this.isVisible() );
                head.SetRelativeZ( po.level * 0.05f );
            }

            // set the proper offset
            Vec2f headoffset( this.getFrameWidth()/2, -this.getFrameHeight()/2 );
            headoffset += this.getOffset();
            headoffset += Vec2f( -po.x, po.y );
            headoffset += Vec2f( 0, -2 );
            head.SetOffset( headoffset );

            if (blob.hasTag("dead") || blob.hasTag("dead head"))
            {
                head.animation.frame = 2;

				// sparkle blood if cut throat
				if (getGameTime() % 2 == 0 && blob.hasTag("cutthroat"))
				{
					Vec2f vel = getRandomVelocity(90.0f, 1.3f * 0.1f*Random(40), 2.0f);
					ParticleBlood( blob.getPosition()+Vec2f(this.isFacingLeft() ? headoffset.x : -headoffset.x, headoffset.y), vel, SColor(255, 126,0,0) );
					if (Random(100) == 0)
						blob.Untag("cutthroat");
				}
            }
            else if (blob.hasTag("attack head"))
            {
                head.animation.frame = 1;
            }
            else
            {
                head.animation.frame = 0;
            }
        }
        else {
            head.SetVisible( false );
        }
    }
}

