// Shield hit - make sure to set up the shield vars elsewhere

#include "ShieldCommon.as";

void sparks(Vec2f at, f32 angle, f32 damage)
{
    int amount = damage*5 + Random(5);

    for (int i = 0; i < amount; i++)
    {
        Vec2f vel = getRandomVelocity(angle, damage * 3.0f, 180.0f);
        vel.y = -Maths::Abs( vel.y ) + Maths::Abs(vel.x)/3.0f - 2.0f - float(Random(100))/100.0f;
        ParticlePixel( at, vel, SColor( 255, 255, 255, 0), true );
    }
}

#include "Hitters.as";

bool canBlockThisType(u8 type) // this function needs to use a tag on the hitterBlob, like ("bypass shield")
{
    return type == Hitters::stomp ||
           type == Hitters::builder ||
           type == Hitters::sword ||
           type == Hitters::shield ||
           type == Hitters::arrow ||
           type == Hitters::bite ||
           type == Hitters::stab ||
           isExplosionHitter(type);
}

bool isExplosionHitter(u8 type)
{
	return type == Hitters::bomb ||
           type == Hitters::explosion;
}

f32 onHit( CBlob@ this, Vec2f worldPoint, Vec2f velocity, f32 damage, CBlob@ hitterBlob, u8 customData )
{
    if (this.hasTag("dead") || !this.hasTag("shielded") || !canBlockThisType(customData) || this is hitterBlob )
    {
        //print("dead " + this.hasTag("dead") + "shielded " + this.hasTag("shielded") + "cant " + canBlockThisType(customData));
        return damage;
    }

    if (blockAttack(this, velocity, 0.0f))
    {
        if (isExplosionHitter(customData)) //bomb jump
        {
			Vec2f vel = this.getVelocity();
			this.setVelocity(Vec2f(0.0f,Maths::Min(0.0f, vel.y)));

            Vec2f bombforce = Vec2f(0.0f, ((velocity.y > 0) ? 0.7f : -1.3f) );

            bombforce.Normalize();
            bombforce *= 2.0f * Maths::Sqrt(damage) * this.getMass();
            bombforce.y -= 2;

            if (this.isFacingLeft() && vel.x > 0) {
                bombforce.x += 5;
            }
            else if (!this.isFacingLeft() && vel.x < 0) {
                bombforce.x -= 5;
            }

            this.AddForce(bombforce);
			this.Tag("dont stop til ground");
            //TODO: force server pos and vel here
        }
        else if (exceedsShieldBreakForce(this,damage))
        {
            knockShieldDown(this);
            this.Tag("force_knock");
        }

        Sound::Play( "Entities/Characters/Knight/ShieldHit.ogg", worldPoint );
        sparks (worldPoint, -velocity.Angle(), damage);
        return 0.0f;
    }

    return damage; //no block, damage goes through
}
