// Blob can place blocks on grid

#include "ThrowCommon.as"
#include "PlacementCommon.as"
#include "BuildBlock.as"
#include "CheckSpam.as"

//server-only
void PlaceBlob( CBlob@ this, CBlob @blob, Vec2f cursorPos )
{
    if (blob !is null)
    {
		blob.Tag("temp blob placed");
		if (this.server_DetachFrom( blob ))
		{
			blob.setPosition( cursorPos );	
			if (blob.isSnapToGrid())  {
				blob.getShape().SetStatic( true );
			}
		}
    }
}

Vec2f getBottomOfCursor( Vec2f cursorPos, CBlob@ carryBlob )
{
    // check at bottom of cursor
    f32 w = getMap().tilesize / 2.0f;
    f32 h = getMap().tilesize / 2.0f;
    int offsetY = 0;//Maths::Max( 1, Maths::Round(carryBlob.getHeight() / getMap().tilesize) ) - 1;
    h -= offsetY * getMap().tilesize / 2.0f;
    return Vec2f(cursorPos.x + w, cursorPos.y + h);
}

void PositionCarried( CBlob@ this, CBlob@ carryBlob )
{
    // rotate towards mouse if object allows
    if (carryBlob.hasTag("place45"))
    {
		f32 distance = 8.0f;
		if (carryBlob.exists("place45 distance"))
			distance = f32(carryBlob.get_s8("place45 distance"));
		
		f32 angleOffset = 0.0f;
		if (!carryBlob.hasTag("place45 perp"))
			angleOffset = 90.0f;
		
        Vec2f aimpos = this.getAimPos();
        Vec2f pos = this.getPosition();
        Vec2f aim_vec = (pos - aimpos);
        aim_vec.Normalize();
        f32 angle_step = 45.0f;
        f32 mouseAngle = (int( aim_vec.getAngleDegrees() + (angle_step*0.5) )/int(angle_step))*angle_step ;
        if(!this.isFacingLeft()) mouseAngle += 180;
        
        carryBlob.setAngleDegrees( -mouseAngle + angleOffset );
        AttachmentPoint@ hands = this.getAttachments().getAttachmentPointByName("PICKUP");

        aim_vec *= distance;

        if (hands !is null)
        {
            hands.offset.x = 0 + (aim_vec.x * 2 * (this.isFacingLeft()?1.0f:-1.0f));  // if blob config has offset other than 0,0 there is a desync on client, dont know why
            hands.offset.y = -(aim_vec.y * (distance < 0 ? 1.0f : 1.0f));
        }
    }
    else
    {
		if (!carryBlob.hasTag("place norotate")) {
            carryBlob.setAngleDegrees( 0.0f );
        }

        AttachmentPoint@ hands = this.getAttachments().getAttachmentPointByName("PICKUP");

        if (hands !is null)
        {
			// set the pickup offset according to the pink pixel
			CSprite@ sprite = this.getSprite();
			PixelOffset @po = getDriver().getPixelOffset( sprite.getFilename(), sprite.getFrame() );  
			if (po !is null)
			{
				// set the proper offset
				Vec2f headoffset( sprite.getFrameWidth()/2, -sprite.getFrameHeight()/2 );
				headoffset += Vec2f( -po.x, po.y );
				headoffset.x *= -1.0f;
				hands.offset = headoffset;
			}
			else {
				hands.offset.Set(0, 0);
			}

            if (this.isKeyPressed( key_down ) ) { // hack for crouch
                hands.offset.y += 2;
            }
            //else {
            //    hands.offset.Set(0, 0);
            //}
        }
    }
}

void onInit( CBlob@ this )
{
    AddCursor( this );
    SetupBuildDelay( this );
    this.addCommandID("placeBlob");
    this.addCommandID("settleLadder");
    
    this.addCommandID("rotateBlob");
    
    this.set_u16("build_angle", 0);

	this.getCurrentScript().runFlags |= Script::tick_not_attached;
	this.getCurrentScript().runFlags |= Script::tick_hasattached;
	//this.getCurrentScript().runFlags |= Script::tick_onscreen;	  wont run on server
	this.getCurrentScript().removeIfTag = "dead";
}

void onTick( CBlob@ this )
{
    if (this.getControls() is null || this.isInInventory()) {
        return;
    }		  

    CBlob @carryBlob = this.getCarriedBlob();  
    if (carryBlob is null) {
        return;
    }

    if (carryBlob !is null)
    {
        if (!carryBlob.isSnapToGrid()) // hide block in hands when placing close
        {
            PositionCarried( this, carryBlob );
        }
        else
        {
			if (carryBlob.hasTag("place norotate"))
			{
				this.getCarriedBlob().setAngleDegrees( 0.0f );
			}
			else
			{
				this.getCarriedBlob().setAngleDegrees( this.get_u16("build_angle") );
			}
		}
    }

    if (!this.isMyPlayer()) {
        return;
    }
////                     ONLY MYPLAYER STUFF BEYOND THIS LINE                   ////
    BlockCursor @bc;
    this.get( "blockCursor", @bc );

    if (bc is null) {
        return;
    }

    if (isBuildDelayed( this )) {

		// don't draw blob while waiting to build
		if (carryBlob !is null)	{
			carryBlob.SetVisible( false );
		}	  
        return;
    }

    SetTileAimpos( this, bc );
    // check buildable
    bc.buildable = false;
    bc.supported = false;

    if (carryBlob !is null)
    {
		CMap@ map = this.getMap();
        bool snap = carryBlob.isSnapToGrid();

        // hide block in hands when placing close

        carryBlob.SetVisible( !snap );

        /*if (!snap)   // ladder can be thrown at all times
        {
            bc.buildable = true; //!carryBlob.isOverlappedAtPosition( carryBlob.getPosition(), carryBlob.getAngleDegrees() );
            bc.supported = true;
        }*/
        
        bool onetile = false;
        if (carryBlob.getName() == "ladder")
        {
			onetile = true;
		}

        if (bc.cursorClose)
        {
            if (snap) // if snaps to grid make cursor
            {
				Vec2f halftileoffset(map.tilesize*0.5f, map.tilesize*0.5f);
				
                CMap@ map = this.getMap();
                TileType buildtile = carryBlob.getMapTile();
                Vec2f bottomPos = getBottomOfCursor( bc.tileAimPos, carryBlob );
                
                bool overlapped;
                
                if (onetile)
                {
					Vec2f ontilepos = halftileoffset + bc.tileAimPos;
					
					overlapped = false;
					CBlob@[] b;
					
					f32 tsqr = halftileoffset.LengthSquared() - 1.0f;
					
					if (map.getBlobsInRadius(ontilepos, 0.5f, @b))
					{
						for (uint nearblob_step = 0; nearblob_step < b.length && !overlapped; ++nearblob_step)
						{
							CBlob@ blob = b[nearblob_step];
							
							if (blob is carryBlob || blob is this) continue;
							
							overlapped = (blob.getPosition()-ontilepos).LengthSquared() < tsqr;
						}
					}
				}
				else
				{
					overlapped = carryBlob.isOverlappedAtPosition( bottomPos, carryBlob.getAngleDegrees() );
				}
                
                bc.buildable = isBuildableAtPos( this, bottomPos, buildtile, carryBlob ) && !overlapped &&
								!isBuildRayBlocked( this.getPosition(), bc.tileAimPos + halftileoffset );
                bc.supported = carryBlob.getShape().getConsts().support > 0 ? map.hasSupportAtPos( bc.tileAimPos, buildtile ) : true;
            }
        }

        // place blob with action1 key

        if (!getHUD().hasButtons() && !carryBlob.hasTag("custom drop") )
        {
			if (this.isKeyPressed( key_action1 ))
			{
				if (snap && bc.cursorClose && bc.buildable && bc.supported)
				{
					CBitStream params;
					params.write_u16( carryBlob.getNetworkID() );
					params.write_Vec2f( getBottomOfCursor(bc.tileAimPos, carryBlob) );
					this.SendCommand( this.getCommandID("placeBlob"), params );
					u32 delay = this.get_u32("build delay");
					if (this.isKeyPressed(key_left)  || this.isKeyPressed(key_right) )
						delay *= 2;
					SetBuildDelay( this, delay );
				}
				/*else if (!getHUD().hasButtons() && !this.isKeyPressed( key_pickup ) && bc.buildable && bc.supported && carryBlob.hasTag("place45"))
				{	
					CBitStream params;
					params.write_u16( carryBlob.getNetworkID() );
					params.write_Vec2f( this.getAimPos() );
					this.SendCommand( this.getCommandID("settleLadder"), params );
					
					SetBuildDelay( this, 40 );
				}*/
				else if (this.isKeyJustPressed( key_action1 )) {
					Sound::Play("NoAmmo.ogg");
				}
			}
			
			if (this.isKeyJustPressed( key_action3 ))
			{
				CBitStream params;
				params.write_u16( (this.get_u16("build_angle") + 90) % 360 );
				this.SendCommand( this.getCommandID("rotateBlob"), params );
			}
        }
    }
}

void onInit( CSprite@ this )
{
	this.getCurrentScript().runFlags |= Script::tick_not_attached;
	this.getCurrentScript().runFlags |= Script::tick_hasattached;
	this.getCurrentScript().runFlags |= Script::tick_myplayer;
	this.getCurrentScript().removeIfTag = "dead";
}

// render block placement
void onRender( CSprite@ this )
{
    CBlob@ blob = this.getBlob();

    if (getHUD().hasButtons()) {
        return;
    }	
    if (blob.isKeyPressed(key_action2) || blob.isKeyPressed(key_pickup)) { //hack: dont show when builder is attacking
        return;
    }	 
    if (isBuildDelayed( blob )) {
        return;
    }

    // draw a map block or other blob that snaps to grid
    CBlob@ carryBlob = blob.getCarriedBlob();

    if (carryBlob !is null) // && carryBlob.isSnapToGrid()
    {
        if (!carryBlob.isSnapToGrid()) {
            return;
        }

        BlockCursor @bc;
        blob.get( "blockCursor", @bc );

        if (bc !is null)
        {
            if (bc.cursorClose && bc.buildable)
            {
                SColor color;

                if (bc.buildable && bc.supported)
                {
                    color.set(255, 255, 255, 255);
                    carryBlob.RenderForHUD( getBottomOfCursor( bc.tileAimPos, carryBlob ) - carryBlob.getPosition(), 0.0f, color, RenderStyle::normal );
                }
                else
                {
                    color.set(255, 255, 46, 50);
                    Vec2f offset(0.0f, -1.0f + 1.0f * ((getGameTime()*0.8f) % 8) );
                    carryBlob.RenderForHUD( getBottomOfCursor( bc.tileAimPos, carryBlob ) + offset - carryBlob.getPosition(), 0.0f, color, RenderStyle::normal );
                }
            }
            else
            {
                f32 halfTile = getMap().tilesize / 2.0f;
                Vec2f aimpos = blob.getMovement().getVars().aimpos;
                carryBlob.RenderForHUD( Vec2f(aimpos.x - halfTile, aimpos.y - halfTile) - carryBlob.getPosition(), 0.0f,
                                        (blob.isKeyPressed ( key_action1 ) || bc.cursorClose) ? SColor(255, 255, 46, 50) : color_white,
                                        RenderStyle::normal );
            }
        }
    }
}

void onCommand( CBlob@ this, u8 cmd, CBitStream @params )
{
	if (cmd == this.getCommandID("rotateBlob") )
    {
		this.set_u16("build_angle", params.read_u16());
		return;
    }
	
	if (!getNet().isServer()) {
        return;
    }

    if (cmd == this.getCommandID("placeBlob") )
    {
        CBlob @carryBlob = getBlobByNetworkID( params.read_u16() );
        Vec2f pos = params.read_Vec2f();
        PlaceBlob( this, carryBlob, pos );
    }
    else if (cmd == this.getCommandID("settleLadder") )
    {
        CBlob @carryBlob = getBlobByNetworkID( params.read_u16() );
        Vec2f pos = params.read_Vec2f();	   
        if (carryBlob !is null)
        {
			carryBlob.Tag("temp blob placed");
            carryBlob.server_DetachFrom( this );
			carryBlob.getShape().SetStatic( true );
        }
    }
}

void onDetach( CBlob@ this, CBlob@ detached, AttachmentPoint@ attachedPoint )
{
    // set visible in case of detachment and was invisible for HUD
    detached.SetVisible( true );

    if (detached.hasTag("temp blob placed"))
    {
        // override ignore collision so we can step on our ladder
        this.IgnoreCollisionWhileOverlapped( null );
        detached.Untag("temp blob placed");
    }
}
