#include "PlacementCommon.as"
#include "BuildBlock.as"

//server-only
void PlaceBlock( CBlob@ this, u8 index, Vec2f cursorPos )
{
    BuildBlock @bc = getBlockByIndex( this, index );

    if (bc is null)
    {
        warn("BuildBlock is null " + index);
        return;
    }

    CBitStream missing;

    if (bc.tile > 0 && this.getInventory().hasRequirements( bc.reqs, missing ))
    {
        this.getInventory().server_TakeRequirements( bc.reqs );
        getMap().server_SetTile( cursorPos, bc.tile );
        
        if ( !(this.getInventory().hasRequirements(bc.reqs, missing)) )
        {
			this.set_TileType( "buildtile", 0 );
		}
    }
}

void onInit( CBlob@ this )
{
    AddCursor( this );
    SetupBuildDelay( this );
    this.addCommandID("placeBlock");

	this.getCurrentScript().runFlags |= Script::tick_not_attached;
	this.getCurrentScript().runFlags |= Script::tick_myplayer;
	this.getCurrentScript().removeIfTag = "dead";
}

void onTick( CBlob@ this )
{
    if (this.isInInventory()) {
        return;
    }

    CBlob @carryBlob = this.getCarriedBlob();  
    if (carryBlob !is null) {
        return;
    }

    BlockCursor @bc;
    this.get( "blockCursor", @bc );

    if (bc is null) {
        return;
    }

    if (isBuildDelayed( this )) {
        return;
    }

    SetTileAimpos( this, bc );
    // check buildable
    bc.buildable = false;
    bc.supported = false;
    TileType buildtile = this.get_TileType("buildtile");

    if (buildtile > 0)
    {
        CMap@ map = this.getMap();
        u8 blockIndex = getBlockIndexByTile( this, buildtile );
        bc.hasReqs = false;
        BuildBlock @block = getBlockByIndex( this, blockIndex );

        if (block !is null)
        {
            CBitStream missing;
            bc.hasReqs = this.getInventory().hasRequirements( block.reqs, missing );
        }

        if (bc.cursorClose)
        {
            bc.buildable = isBuildableAtPos( this, bc.tileAimPos, buildtile, null )
							&& !isBuildRayBlocked( this.getPosition(), bc.tileAimPos + Vec2f(map.tilesize*0.5f, map.tilesize*0.5f) )
							;
            bc.supported = !bc.buildable || map.hasSupportAtPos( bc.tileAimPos, buildtile );
        }

        // place block

        if (!getHUD().hasButtons() && this.isKeyPressed( key_action1 ) )
        {
            if (bc.cursorClose && bc.buildable && bc.supported )
            {
                CBitStream params;
                params.write_u8( blockIndex );
                params.write_Vec2f( bc.tileAimPos );
                this.SendCommand( this.getCommandID("placeBlock"), params );
				u32 delay = this.get_u32("build delay");
				if (this.isKeyPressed(key_left)  || this.isKeyPressed(key_right) )
					delay *= 2;
                SetBuildDelay( this, map.isTileSolidNoBlobs( block.tile ) ? delay : delay / 3 );
            }
            else if (this.isKeyJustPressed( key_action1 )) {
                Sound::Play("NoAmmo.ogg");
            }
        }
    }
}

// render block placement

void onInit( CSprite@ this )
{
	this.getCurrentScript().runFlags |= Script::tick_not_attached;
	this.getCurrentScript().runFlags |= Script::tick_myplayer;
	this.getCurrentScript().removeIfTag = "dead";
}

void onRender( CSprite@ this )
{
    CBlob@ blob = this.getBlob();
    if (getHUD().hasButtons()) {
        return;
    }

    if (blob.isKeyPressed(key_action2) || blob.isKeyPressed(key_pickup)) { //hack: dont show when builder is attacking
        return;
    }

    CBlob @carryBlob = blob.getCarriedBlob();	 
    if (carryBlob !is null) {
        return;
    }

    if (isBuildDelayed( blob )) {
        return;
    }

    // draw a map block or other blob that snaps to grid
    TileType buildtile = blob.get_TileType("buildtile");

    if (buildtile > 0)
    {
        BlockCursor @bc;
        blob.get( "blockCursor", @bc );

        if (bc !is null)
        {
            if (bc.cursorClose && bc.hasReqs && bc.buildable)
            {
                SColor color;

                if (bc.buildable && bc.supported)
                {
                    color.set(255, 255, 255, 255);
                    getMap().DrawTile( bc.tileAimPos, buildtile, color, getCamera().targetDistance, true );
                }
                else
                {
                    color.set(255, 255, 46, 50);
                    Vec2f offset(0.0f, -1.0f + 1.0f * ((getGameTime()*0.8f) % 8) );
                    getMap().DrawTile( bc.tileAimPos + offset, buildtile, color, getCamera().targetDistance, true );
                }
            }
            else
            {
                f32 halfTile = getMap().tilesize / 2.0f;
                Vec2f aimpos = blob.getAimPos();
                getMap().DrawTile( Vec2f(aimpos.x - halfTile, aimpos.y - halfTile), buildtile,
                                   ( bc.hasReqs && (!blob.isKeyPressed( key_action1 ) && !bc.cursorClose )) ? color_white : SColor(255, 255, 46, 50),
                                   getCamera().targetDistance, true );
            }
        }
    }
}

void onCommand( CBlob@ this, u8 cmd, CBitStream @params )
{
    if( getNet().isServer() && cmd == this.getCommandID("placeBlock") )
    {
        u8 index = params.read_u8();
        Vec2f pos = params.read_Vec2f();
        PlaceBlock( this, index, pos );
    }
}

