
shared class BuildBlock
{
    TileType tile;
    string name;
    CBitStream reqs;
    string icon;
    string description;
	bool buildOnGround;
    bool temporaryBlob;

    BuildBlock() {} // required for handles to work

    BuildBlock( TileType _tile, string _name, string _icon, string _desc )
    {
        tile = _tile;
        name = _name;
        icon = _icon;
        description = _desc;
        temporaryBlob = true;
		buildOnGround = false;
    }
};

u8 getBlockIndexByTile( CBlob@ this, TileType tile )
{
    BuildBlock[]@ blocks;

    if (this.get( "blocks", @blocks ))
    {
        for (uint i = 0; i < blocks.length; i++)
        {
            BuildBlock@ b = blocks[i];

            if (b.tile == tile) {
                return i;
            }
        }
    }

    warn("getBlockIndexByTile() blocks not found");
    return 255;
}

TileType getTileByBlockIndex( CBlob@ this, u8 index )
{
    BuildBlock[]@ blocks;

    if (this.get( "blocks", @blocks ))
    {
        if (index >= 0 && index < blocks.length) {
            return blocks[index].tile;
        }
    }

    warn("getTileByBlockIndex() blocks not found");
    return 0;
}

BuildBlock@ getBlockByIndex( CBlob@ this, u8 index )
{
    BuildBlock[]@ blocks;

    if (this.get( "blocks", @blocks ))
    {
        return @blocks[index];
    }

    warn("getBlockByIndex() blocks not found");
    return null;
}