//generic shop menu

// properties:
//      shop offset - Vec2f - used to offset things bought that spawn into the world, like vehicles

#include "ShopCommon.as";
#include "Requirements.as";
#include "MakeCrate.as";
#include "CheckSpam.as";

void onInit( CBlob@ this )
{
	this.addCommandID( "shop menu" );
	this.addCommandID( "shop buy" );

	if (!this.exists("shop available"))
		this.set_bool("shop available", true);
	if (!this.exists("shop offset"))
		this.set_Vec2f("shop offset", Vec2f(0,0));
	if (!this.exists("shop menu size"))
		this.set_Vec2f("shop menu size", Vec2f(7,7));
	if (!this.exists("shop description"))
		this.set_string("shop description", "Workbench");
}

void GetButtonsFor( CBlob@ this, CBlob@ caller )
{
	ShopItem[]@ shop_items;		   
	if (!this.get(SHOP_ARRAY, @shop_items)) { return; }
	if (shop_items.length > 0 && this.get_bool("shop available"))
	{
		CBitStream params;
		params.write_u16( caller.getNetworkID() );																				
		caller.CreateGenericButton( 15, this.get_Vec2f("shop offset"), this, this.getCommandID("shop menu"), this.get_string("shop description"), params );
	}
}

bool isInRadius( CBlob@ this, CBlob @caller )
{
	return ((this.getPosition() - caller.getPosition()).Length() < this.getRadius()*3.0f + caller.getRadius());
}

void onCommand( CBlob@ this, u8 cmd, CBitStream @params )
{
	bool isServer = getNet().isServer();

	if (cmd == this.getCommandID("shop menu"))
    {
        // build menu for them
        CBlob@ caller = getBlobByNetworkID( params.read_u16() );
		BuildShopMenu( this, caller, "Shop", Vec2f(200,-50), this.get_Vec2f("shop menu size") );
    }
	else
	if (cmd == this.getCommandID("shop buy"))
    {
        u16 callerID;
		if (!params.saferead_u16(callerID))
			return;
        bool spawnToInventory = params.read_bool();
        bool spawnInCrate = params.read_bool();
		bool producing = params.read_bool();
        string blobName = params.read_string();		
        u8 s_index = params.read_u8();

        CBlob@ caller = getBlobByNetworkID(callerID);
        if (caller is null) { return; }
        CInventory@ inv = caller.getInventory();

        if (inv !is null && isInRadius( this, caller ))
        {
            ShopItem[]@ shop_items;		   
            if (!this.get(SHOP_ARRAY, @shop_items)) { return; }
            if (s_index >= shop_items.length) { return; }  
            ShopItem@ s = shop_items[s_index];

			// production?
			if (s.ticksToMake > 0)
			{
				s.producing = producing;
				return;
			}

			if (!getNet().isServer()) { return; } //only do this on server

			// check spam

			if (isSpammed( blobName, this.getPosition() ))
				return;

			bool tookReqs = false;

			// try taking from the caller + this shop first
            CBitStream missing;
			if (hasRequirements( inv, this.getInventory(), s.requirements, missing )) 
            {
				server_TakeRequirementsCombined( inv, this.getInventory(), s.requirements );
				tookReqs = true;
			}
			// try taking from caller + storages second
			if (!tookReqs)
			{
				const s32 team = this.getTeamNum();	  
				CBlob@[] storages;
				if (getBlobsByName( "storage", @storages ))
					for (uint step = 0; step < storages.length; ++step)
					{
						CBlob@ storage = storages[step];
						if (storage.getTeamNum() == team)
						{																
							CBitStream missing;
							if (hasRequirements( inv, storage.getInventory(), s.requirements, missing )) 
							{
								server_TakeRequirementsCombined( inv, storage.getInventory(), s.requirements );
								tookReqs = true;
								break;
							}
						}
					}
			}

			if (tookReqs)
			{
                //inv.server_TakeRequirements(s.requirements);
                Vec2f spawn_offset = Vec2f();

                if (this.exists("shop offset")) { spawn_offset = this.get_Vec2f("shop offset"); }
                if (this.isFacingLeft()) { spawn_offset.x *= -1; }

                if (spawnInCrate)
                {
                    CBlob@ crate = server_MakeCrate( blobName, s.name, 1 + Random(2), caller.getTeamNum(), caller.getPosition() );

                    if (crate !is null)
                    {
                        if (spawnToInventory && caller.canBePutInInventory(crate)) {
                            caller.server_PutInInventory(crate);
                        }
                        else {
                            caller.server_Pickup(crate);
                        }
                    }
                }
                else
                {
                    CBlob@ blob = server_CreateBlob(blobName, caller.getTeamNum(), this.getPosition() + spawn_offset);

                    if (blob !is null)
                    {
                        if (spawnToInventory && caller.canBePutInInventory(blob)) {
                            caller.server_PutInInventory(blob);
                        }
                        else {
							if (blob.getAttachments() !is null && blob.getAttachments().getAttachmentPointByName("PICKUP") !is null)
								caller.server_Pickup(blob);
                        }
                    }
                }
            }
        }
	}   
}

