#include "Requirements.as";

shared class ShopItem
{
    string name;
    string iconName;
    string blobName;
    string description;
    CBitStream requirements;
	// creating
    bool spawnToInventory;
    bool spawnInCrate;
	// production
	u32 ticksToMake;
	u32 timeCreated;
	bool producing;
	bool inProductionNow;
	bool hasRequirements;
	bool inStock;
	u8 quantityLimit;
	CBitStream requirementsMissing;
	// food production
	u8 customData;
	
	void Setup(string _name, string _iconName, string _blobName, string _description, bool _spawnToInventory, bool _spawnInCrate)
	{
		name = _name;
		iconName = _iconName;
		blobName = _blobName;
		description = _description;
		spawnToInventory = _spawnToInventory;
		spawnInCrate = _spawnInCrate;
		producing = false;
		timeCreated = 0;
		ticksToMake = 0;
		inProductionNow = hasRequirements = inStock = false;
		quantityLimit = 0;
		customData = 0;
	}
	
	ShopItem()
	{
		Setup("","","","",false,false);
	}	
	
	ShopItem( CBitStream @bt )
	{
		if (!Unserialise(bt)) {
			warn("Error unserializing ShopItem");
			name = "";
		}
	}
	
	void Serialise( CBitStream@ stream )
	{
		stream.write_string( name );
		stream.write_string( iconName );
		stream.write_string( blobName );
		stream.write_string( description );
		stream.write_u32( ticksToMake );
		stream.write_bool( spawnInCrate );
		stream.write_u8( quantityLimit );
		stream.write_u32( timeCreated );
		stream.write_CBitStream( requirements );
		stream.write_bool( producing );
		stream.write_u8( customData );
	}
	
	bool Unserialise( CBitStream@ stream )
	{
		if (!stream.saferead_string(name)) return false;
		if (!stream.saferead_string(iconName)) return false;		
		if (!stream.saferead_string(blobName)) return false;		
		if (!stream.saferead_string(description)) return false;							
		if (!stream.saferead_u32(ticksToMake)) return false;										
		if (!stream.saferead_bool(spawnInCrate)) return false;													
		if (!stream.saferead_u8(quantityLimit)) return false;						
		if (!stream.saferead_u32(timeCreated)) return false;																				
		if (!stream.saferead_CBitStream(requirements)) return false;
		if (!stream.saferead_bool(producing)) return false;																											
		if (!stream.saferead_u8(customData)) return false;							
		return true;																																						
	}


};

const string SHOP_ARRAY = "shop array";

//adding a item to a blobs list of items

ShopItem@ addShopItem( CBlob@ this, const string &in name, const string &in iconName, const string &in blobName, const string &in description )
{
    return addShopItem( this, SHOP_ARRAY, name, iconName, blobName, description, false, false );
}

ShopItem@ addShopItem( CBlob@ this, const string &in name, const string &in iconName, const string &in blobName, const string &in description, bool spawnToInventory )
{
    return addShopItem( this, SHOP_ARRAY, name, iconName, blobName, description, spawnToInventory, false );
}

ShopItem@ addShopItem( CBlob@ this, const string &in name, const string &in  iconName, const string &in  blobName, const string &in description, bool spawnToInventory, bool spawnInCrate )
{
	return addShopItem( this, SHOP_ARRAY, name, iconName, blobName, description, spawnToInventory, spawnInCrate );
}

ShopItem@ addShopItem( CBlob@ this, const string &in shopArray, const string &in name, const string &in iconName, const string &in blobName, const string &in description, bool spawnToInventory, bool spawnInCrate )
{
    if (!this.exists(shopArray))
    {
        ShopItem[] items;
        this.set( shopArray, items );
    }

	// check if we are not duplicating
	ShopItem[]@ shop_items;	   
	if (this.get( shopArray, @shop_items ))
	{
		for (uint i = 0 ; i < shop_items.length; i++)
		{
			ShopItem @item = shop_items[i];	
			if (item.blobName == blobName && item.name == name)
			{
				item.name = name;
				item.iconName = iconName;
				item.blobName = blobName;
				item.description = description;
				item.spawnToInventory = spawnToInventory;
				item.spawnInCrate = spawnInCrate;
				return item;
			}
		}
	}

	// create a new one
    ShopItem p;
    p.Setup( name, iconName, blobName, description, spawnToInventory, spawnInCrate );
    
    this.push(shopArray, p);
    ShopItem@ p_ref;
    this.getLast( shopArray, @p_ref );
    return p_ref;
}

//helper for building menus of shopitems

void addShopItemsToMenu(CBlob@ this, CGridMenu@ menu, CBlob@ caller )
{
    ShopItem[]@ shop_items;

    if (this.get( SHOP_ARRAY, @shop_items ))
    {
        for (uint i = 0 ; i < shop_items.length; i++)
        {
            ShopItem @s_item = shop_items[i];		  
            if (s_item is null || caller is null) { continue; }
            CBitStream params;

            params.write_u16( caller.getNetworkID() );
            params.write_bool( s_item.spawnToInventory );
            params.write_bool( s_item.spawnInCrate );
			params.write_bool( s_item.producing );
            params.write_string( s_item.blobName );
            params.write_u8(u8(i));

			CGridButton@ button = menu.AddButton( s_item.iconName, s_item.name, this.getCommandID("shop buy"), params );	   
			if (button !is null)
			{
				if (s_item.producing)		  // !! no click for production items
					button.clickable = false;

				button.selectOnClick = true;

				bool tookReqs = false;
				CBlob@ storageReq = null;
				// try taking from the caller + this shop first
				CBitStream missing;
				if (hasRequirements( this.getInventory(), caller.getInventory(), s_item.requirements, missing )) 
				{
					tookReqs = true;
				}
				// try taking from caller + storages second
				if (!tookReqs)
				{
					const s32 team = this.getTeamNum();	  
					CBlob@[] storages;
					if (getBlobsByName( "storage", @storages ))
						for (uint step = 0; step < storages.length; ++step)
						{
							CBlob@ storage = storages[step];
							if (storage.getTeamNum() == team)
							{																
								CBitStream missing;
								if (hasRequirements( caller.getInventory(), storage.getInventory(), s_item.requirements, missing )) 
								{
									@storageReq = storage;
									break;
								}
							}
						}
				}

				if (s_item.ticksToMake > 0)		   // production
					SetItemDescription( button, this, s_item.requirements, s_item.description, this.getInventory() );
				else
					SetItemDescription( button, caller, s_item.requirements, s_item.description, (storageReq !is null) ? storageReq.getInventory() : this.getInventory() );

				//if (s_item.producing) {
				//	button.SetSelected( 1 );
				//	menu.deleteAfterClick = false;
				//}
			}
        }
    }
}

void BuildShopMenu( CBlob@ this, CBlob @caller, string description, Vec2f offset, Vec2f slotsAdd )
{
	if (caller is null || !caller.isMyPlayer())
		return;

    ShopItem[]@ shopitems;

    if (!this.get( SHOP_ARRAY, @shopitems )) { return; }

    {
        CControls@ controls = caller.getControls();
        CGridMenu@ menu = CreateGridMenu( caller.getScreenPos() + offset, this, Vec2f(slotsAdd.x, slotsAdd.y), description );

        if (menu !is null) {
            addShopItemsToMenu(this, menu, caller );
        }
    }
}

void BuildDefaultShopMenu( CBlob@ this, CBlob @caller )
{
	BuildShopMenu( this, caller, "Shop", Vec2f(200,-50), Vec2f( 4,4) );
}

void ShopSendCreateData( CBlob@ this, CBitStream@ stream, const string &in shopArray )
{	 
	ShopItem[]@ items;
	if (this.get( shopArray, @items ))
	{
		stream.write_u8( items.length );
		for (uint i = 0 ; i < items.length; i++)
		{
			ShopItem @item = items[i];	
			item.Serialise(stream);
		}
	}
	else
		stream.write_u8( 0 );
}

bool ShopReceiveCreateData( CBlob@ this, CBitStream@ stream, const string &in shopArray )
{
	u8 itemsCount;
	if (!stream.saferead_u8(itemsCount)) {
		warn("failed to read itemsCount");
		return false;
	}

	// this is received before onInit but we still check for safety and try not to duplicate items

	this.clear( shopArray );

	for (uint i = 0 ; i < itemsCount; i++)
	{
		ShopItem@ item = addShopItem( this, shopArray, "", "", "", "", false, false );		
		if (item !is null)
		{
			if (!item.Unserialise( stream ))	{
				warn("Could not receive shop item for " + this.getName() );
				continue;
			}		
		}
	}

	return true;
}
