
#include "Hitters.as";
#include "BuilderHittable.as"

const f32 speed_thresh = 2.4f;
const f32 speed_hard_thresh = 2.6f;

const string buzz_prop = "drill timer";

const string heat_prop = "drill heat";
const u8 heat_max = 150;
const u8 heat_cooldown_time = 4;
const u8 heat_add = 5;

// sprite

void onInit( CSprite@ this )
{
	CSpriteLayer@ heat = this.addSpriteLayer( "heat", this.getFilename(), 32, 16 );

    if (heat !is null)
    {
        Animation@ anim = heat.addAnimation( "default", 0, true );
        
        int[] frames = {4, 5, 6, 7};
        
        anim.AddFrames(frames);
        heat.SetAnimation( anim );
        heat.SetRelativeZ(0.1f);
        
        heat.SetVisible(false);
        
        heat.setRenderStyle( RenderStyle::light );
    }
}

void onTick( CSprite@ this )
{
	CBlob@ blob = this.getBlob();
	
	bool buzz = blob.get_bool(buzz_prop);
	if (buzz)
	{
		this.SetAnimation("buzz");
	}
	else if (this.isAnimationEnded())
	{
		this.SetAnimation("default");
	}
	
	CSpriteLayer@ heatlayer = this.getSpriteLayer( "heat" );
	if (heatlayer !is null)
	{
		f32 heat = Maths::Min(blob.get_u8(heat_prop),heat_max);
		f32 heatPercent = heat / float(heat_max);
		
		if (heatPercent > 0.1f)
		{
			heatlayer.setRenderStyle( RenderStyle::light );
			
			blob.SetLight( true );
			blob.SetLightRadius( heatPercent * 24.0f );
			SColor lightColor = SColor( 255, 255, Maths::Min(255, 128 + int(heatPercent*128)), 64);
			blob.SetLightColor( lightColor );
    
			heatlayer.SetVisible(true);
			heatlayer.animation.frame = heatPercent * 3;
			
			if (heatPercent > 0.7f && getGameTime() % 3 == 0)
			{
				makeSteamParticle(blob, Vec2f());
			}
			
		}
		else
		{
			blob.SetLight( false );
			heatlayer.SetVisible(false);
		}
	}
}

void makeSteamParticle(CBlob@ this, const Vec2f vel, const string filename = "SmallSteam")
{
	const f32 rad = this.getRadius();
	Vec2f random = Vec2f( Random(128)-64, Random(128)-64 ) * 0.015625f * rad;
	ParticleAnimated( CFileMatcher(filename).getFirst(), this.getPosition() + random, vel, float(Random(360)), 1.0f, 2 + Random(3), -0.1f, false );
}

void makeSteamPuff(CBlob@ this, const f32 velocity = 1.0f, const int smallparticles = 10, const bool sound = true)
{
	if (sound)
	{
		this.getSprite().PlaySound("Steam.ogg");
	}
	
	makeSteamParticle(this, Vec2f(), "MediumSteam");
	for (int i = 0; i < smallparticles; i++)
	{
		f32 randomness = (Random(32) + 32)*0.015625f * 0.5f + 0.75f;
		Vec2f vel = getRandomVelocity( -90, velocity * randomness, 360.0f );
		makeSteamParticle(this, vel);
	}
}


// blob
void onInit( CBlob@ this )
{
    AttachmentPoint@ ap = this.getAttachments().getAttachmentPointByName("PICKUP");

    if (ap !is null)
    {
        ap.SetKeysToTake( key_action1 | key_action2 | key_action3 );
    }

    this.set_u32("hittime", 0);
    
    this.Tag("place45");
    this.set_s8("place45 distance", 1);
	this.Tag("place45 perp");
	
	this.set_u8(heat_prop, 0);

	this.getSprite().SetEmitSound( CFileMatcher("/Drill.ogg").getFirst() );	 	 	
	this.getSprite().SetEmitSoundPaused( true );
}

void onTick( CBlob@ this )
{
	u8 heat = this.get_u8(heat_prop);
	
	const u32 gametime = getGameTime();

	if (heat > 0)
	{
		if (gametime % heat_cooldown_time == 0)
		{
			heat--;
		}
		
		if (this.isInWater() && heat >= heat_add && gametime % (Maths::Max( int(heat_cooldown_time*0.2f), 1 )) == 0)
		{
			
			u8 lim = u8(heat_max * 0.7f);
			if (heat > lim)
			{
				makeSteamPuff(this);
			}
			else
			{
				makeSteamPuff(this, 0.5f, 5, false);
			}
			
			heat -= heat_add;
			
		}

		this.set_u8(heat_prop, heat);
	}
	
	this.getSprite().SetEmitSoundPaused( true );
	
    if (this.isAttached())
    {
		this.getCurrentScript().runFlags &= ~(Script::tick_not_sleeping); 
		
		AttachmentPoint@ point = this.getAttachments().getAttachmentPointByName("PICKUP");
		
        CBlob@ holder = point.getOccupied();

        if (holder is null) { return; }

		// cool faster if holder is moving
		if (heat > 0 && holder.getShape().vellen > 0.01f && getGameTime() % heat_cooldown_time == 0)
		{
			heat--;
		}

		this.getShape().SetRotationsAllowed(false);

		if (int(heat) >= heat_max - (heat_add * 1.5))
		{
			makeSteamPuff(this, 1.5f, 3, false);
			this.server_Hit( holder, holder.getPosition(), Vec2f(), 0.25f, Hitters::burn, true );
			this.server_DetachFrom(holder);
			this.getSprite().PlaySound("DrillOverheat.ogg");
		}

		if (!point.isKeyPressed(key_action1) )
		{
			this.set_bool(buzz_prop,false);
			return;
		}

		this.getSprite().SetEmitSoundPaused( false );
		
		this.set_bool(buzz_prop,true);
		
	
		if (heat < heat_max)
		{
			heat++;
		}

		if (((gametime+this.getNetworkID()) % 8) != 0) return;		
	
		// delay drill

		{
			const bool facingleft = this.isFacingLeft();
			
			Vec2f direction = Vec2f(1,0).RotateBy(this.getAngleDegrees() + (facingleft?180.0f:0.0f));
		
			const f32 sign = (facingleft ? -1.0f : 1.0f);
		
			CMap@ map = getMap();
			const f32 attack_distance = 6.0f;
			Vec2f attackVel = direction * attack_distance;
			const f32 vellen = 16.0f;
        
			const f32 attack_dam = 1.0f;
			bool hitsomething = false;
        
			HitInfo@[] hitInfos;   
			if (this.getMap().getHitInfosFromArc( (this.getPosition() - attackVel), -attackVel.Angle(), 30, vellen, this, false, @hitInfos ))
			{
				//print("angle: "+attackVel.Angle());
			
				for (uint i = 0; i < hitInfos.length; i++)
				{
					HitInfo@ hi = hitInfos[i];

					if (hi.blob !is null) // blob
					{
						if ( (!hi.blob.isCollidable() && !BuilderAlwaysHit(hi.blob)) || hi.blob.getTeamNum() == holder.getTeamNum())
						{
							continue;
						}

						holder.server_Hit( hi.blob, hi.hitpos, attackVel, attack_dam, Hitters::saw );
                    
						if (int(heat) > heat_max * 0.5f)
						{
							holder.server_Hit( hi.blob, hi.hitpos, attackVel, 1.0f, Hitters::burn ); //extra burn damage
						}
                    
						hitsomething = true;
					}
					else // map
					{
						hitsomething = true;
						TileType tile = map.getTile(hi.tileOffset);
                    
						this.server_HitMap( hi.hitpos, attackVel, 1.0f, Hitters::saw );
					}

					if (hitsomething)
					{
						if (heat < heat_max)
						{
							heat += heat_add;
						}
					}
				}
			}	   
		}
        
        this.set_u8(heat_prop, heat);
    }
    else
    {
        this.getShape().SetRotationsAllowed(true);
        this.set_bool(buzz_prop,false);
		if (heat <= 0) {
			this.getCurrentScript().runFlags |= Script::tick_not_sleeping; 
		}
    }
}

f32 onHit( CBlob@ this, Vec2f worldPoint, Vec2f velocity, f32 damage, CBlob@ hitterBlob, u8 customData )
{
    if (customData == Hitters::fire)
    {
        this.set_u8(heat_prop, heat_max);
        makeSteamPuff(this);
    }

    if (customData == Hitters::water)
    {
        this.set_u8(heat_prop, 0);
        makeSteamPuff(this);
    }

    return damage;
}

void onHitMap( CBlob@ this, Vec2f worldPoint, Vec2f velocity, f32 damage, u8 customData )
{
	getMap().server_DestroyTile( worldPoint, damage );
}

void onAttach( CBlob@ this, CBlob@ attached, AttachmentPoint @attachedPoint )
{
	this.getCurrentScript().runFlags &= ~Script::tick_not_sleeping;
}
