﻿// Factory

#include "ShopCommon.as";
#include "ProductionCommon.as";
#include "TechsCommon.as";
#include "Descriptions.as";
#include "MakeScroll.as";
#include "Help.as";
#include "WARCosts.as";
#include "Hitters.as";

void onInit( CBlob@ this )
{							   
	//this.set_Vec2f("shop offset", Vec2f(0, -20));
	//this.set_Vec2f("shop menu size", Vec2f(4,6));	
	this.set_string( "tech name", "");
	this.addCommandID("use scroll");
	this.addCommandID("take scroll");
	this.getShape().getConsts().mapCollisions = false;
	this.Tag("huffpuff production");

	AddIconToken( "$use_scroll$", "GUI/InteractionIcons.png", Vec2f(32,32), 12, 3 );
	AddIconToken( "$take_scroll$", "GUI/InteractionIcons.png", Vec2f(32,32), 19, 1 );
	
	this.addCommandID("kill children");
	AddIconToken( "$kill_children$", "GUI/InteractionIcons.png", Vec2f(32,32), 10, 1 );

	this.set_TileType("background tile", CMap::tile_wood_back);
	this.getSprite().getConsts().accurateLighting = true;

	SetHelp( this, "help use", "", "Factory requires a technology blueprint to run. \n\n         $KEY_E$" ); 
}

bool hasTech( CBlob@ this )
{
	return (this.get_string("tech name").size() > 0);
}

void GetButtonsFor( CBlob@ this, CBlob@ caller )
{
	// add button for adding scroll if caller has it

	CBitStream params;
	params.write_u16( caller.getNetworkID() );

	if (!hasTech(this))
	{	  
		CBlob@ carried = caller.getCarriedBlob();
		if (carried !is null && carried.getName() == "scroll" && carried.hasTag("tech"))
		{
			params.write_u16( carried.getNetworkID() );
			caller.CreateGenericButton( "$use_scroll$", Vec2f(0, 10), this, this.getCommandID("use scroll"), "Use blueprint", params );
		}
		else
		{
			CButton@ button = caller.CreateGenericButton( "$DISABLED$", Vec2f(0, 10), this, 0, "Requires blueprint", params );
			if (button !is null) {
				button.SetEnabled( false );
			}
		}
	}
	else // hasTech
	{									  
		// steal tech
		if (caller.getTeamNum() != this.getTeamNum())
		{
			caller.CreateGenericButton( "$take_scroll$", Vec2f(0, 10), this, this.getCommandID("take scroll"), "Steal blueprint", params );
		}
		else {
			//destroy applicable items
			string[] destructibleChildren = ChildDestruction(this, false);
			uint length = destructibleChildren.length;
			if (length > 0)
			{
				string caption = "Destroy Far Items:";
				for (uint i = 0; i < length; ++i)
				{
					caption += "\n"+destructibleChildren[i];
				}
				
				caller.CreateGenericButton( "$kill_children$", Vec2f(0, -20), this, this.getCommandID("kill children"), caption, params );
			}
		}
	}
}

string[] ChildDestruction( CBlob@ this, bool kill_kids )
{
	bool killed = false;
	bool server = getNet().isServer();
	
	int team = this.getTeamNum();
	
	string[] names;
	
	u16[]@ ids; 
	if (this.get( PRODUCTION_TRACKING_ARRAY, @ids ))
	{
		for (uint i = 0; i < ids.length; i++)
		{
			CBlob@ blob = getBlobByNetworkID( ids[i] );
			if (blob !is null)
			{
				if (!hasTeamiesNear(blob, team))
				{
					if (kill_kids && server)
					{
						this.server_Hit( blob, blob.getPosition(), Vec2f(0,0), 100, Hitters::crush, true);
					}
					
					names.push_back(blob.getInventoryName());
				}
			}
		}
	}
	
	return names;
}

bool hasTeamiesNear( CBlob@ blob, int team )
{
	CMap@ map = blob.getMap();
	if (map is null) return false;
	
	Vec2f pos = blob.getPosition();
	
	CBlob@[] blobs;
	if (map.getBlobsInRadius( pos, 128.0f, @blobs ))
	{
		for (uint step = 0; step < blobs.length; ++step)
		{
			CBlob@ b = blobs[step];
			if (b !is blob && //not us
				b.getTeamNum() == team && //same team
				b.hasTag("player")) 
			{
				return true;
			}
		}
	}
	
	return false;
}

void onCommand( CBlob@ this, u8 cmd, CBitStream @params )
{
	bool isServer = getNet().isServer();

	if (isServer && cmd == this.getCommandID("use scroll"))
	{
		u16 callerID;
		if (!params.saferead_netid(callerID))
			return;
		u16 scrollID;
		if (!params.saferead_netid(scrollID))
			return;

		CBlob@ caller = getBlobByNetworkID( callerID );	 		
		CBlob@ scroll = getBlobByNetworkID( scrollID );	 		
		if (caller !is null && scroll !is null)
		{	
			if (this.server_PutInInventory( scroll ))
			{
			}
		}
	}
	else if (isServer && cmd == this.getCommandID("take scroll"))
	{
		u16 callerID;
		if (!params.saferead_netid(callerID))
			return;

		CBlob@ caller = getBlobByNetworkID( callerID );	 		
		if (caller !is null)
		{	
			CBlob@ scroll = this.server_PutOutInventory( "scroll" );

			if (scroll !is null)
			{
				caller.server_Pickup( scroll );
			}
		}
	}
	else if (cmd == this.getCommandID("kill children"))
	{
		if (isServer) {
			string[] names = ChildDestruction(this,true);
			if (names.length == 0)
				getNet().server_SendMsg("Team mates are near the item you want destroyed.");
		}
	}
}		   

bool isInventoryAccessible( CBlob@ this, CBlob@ forBlob )
{
	return false;
}

void onAddToInventory( CBlob@ this, CBlob@ blob )
{
	if (blob.hasTag("tech"))
	{
		this.getSprite().PlaySound( "/PowerUp.ogg" );
		AddProductionItemsFromTech( this, blob );	 
		this.setInventoryName( this.get_string( "tech name" ) + " factory" );
	}
}

void onRemoveFromInventory( CBlob@ this, CBlob@ blob )
{
	if (blob.hasTag("tech"))
	{
		this.getSprite().PlaySound( "/PowerDown.ogg" );
		RemoveProductionItems( this );
	}
}


void AddProductionItemsFromTech( CBlob@ this, CBlob@ techblob )
{
	if (techblob is null)
		return;

	RemoveProductionItems( this );

	this.set_string( "tech name", techblob.getInventoryName() );

	ShopItem[]@ techItems;
	if (techblob.get( TECH_ARRAY, @techItems ))
	{
		for (uint i = 0 ; i < techItems.length; i++)
		{
			ShopItem @item = techItems[i];
			ShopItem@ s = addProductionItem( this, item.name, item.iconName, item.blobName, item.description, 1, item.spawnInCrate, item.quantityLimit, item.requirements );
			if (s !is null)
			{
				s.ticksToMake = item.ticksToMake;
			}
		}
	}
}

void RemoveProductionItems( CBlob@ this )
{
	this.clear( TECH_ARRAY );
	this.clear( PRODUCTION_ARRAY );
	this.set_string( "tech name", "" );
}

// leave a pile of wood	after death
void onDie(CBlob@ this)
{
	if (getNet().isServer())
	{
		CBlob@ blob = server_CreateBlob( "mat_wood", this.getTeamNum(), this.getPosition() );
		if (blob !is null)
		{
			blob.server_SetQuantity( COST_WOOD_FACTORY/2 );
		}
	}
}
