#include "LayerSetup.as";  
#include "ProductionCommon.as";

//factory

const string texname = "Factory.png";

const u8 rotates	= 0x01;
const u8 forwards	= 0x02;
const u8 damage_an	= 0x04;

void onInit(CSprite@ this)
{
	CBlob@ blob = this.getBlob();

	const int team = blob.getTeamNum();

	//CANT BE SCRIPT GLOBAL, NEEDS TEAMNUM
	int[] wood_cog = {30};
	int[] team_cog = {31};
	int[] dark_cog = {15};
	int[] scroll_anim = {64, 65, 72};
	int[] big_gear = {6, 14};
	int[] frame = {0, 3};
	int[] dispenser = {5, 13};
	
	LayerSetup[] layers = {
					// name,		texture, size,		  		frames,	 		 offset,	team, skin, visible, vars
		//scroll layers
		makeLayerSetup("scroll anim",   texname, Vec2f(16,8),   scroll_anim, 	Vec2f(-2,0),   	0, 0,	    false, 0),
		makeLayerSetup("scroll g 1",	texname, Vec2f(8,8),	wood_cog,	    Vec2f(6,-5), 	0, 0,	    false, rotates|forwards),
		makeLayerSetup("scroll g 2",	texname, Vec2f(8,8),	team_cog,	    Vec2f(-11,3),	team, 0,	false, rotates|forwards),
		//gears layers
		makeLayerSetup("big gear",	    texname, Vec2f(16,16),  big_gear,   	Vec2f(-8,-7),  	0, 0,	    true, rotates | damage_an),

		makeLayerSetup("gear1",		    texname, Vec2f(8,8),	dark_cog,	    Vec2f(-4,7),	0, 0,	    true, rotates),
		makeLayerSetup("gear2",		    texname, Vec2f(8,8),	dark_cog,	    Vec2f(-8,13),	0, 0,	    true, rotates| forwards),
		
		makeLayerSetup("frame",		    texname, Vec2f(40,16),	frame,	   		Vec2f(0,-8),	0, 0,	    true, damage_an ),
		
		makeLayerSetup("dispenser",		texname, Vec2f(16,16),	dispenser,	  	Vec2f(11,-5),	team, 0,	true, damage_an )

	};

	//setup the special anims
	layers[0].frameTime = 3;
	layers[0].loops = true;
	layers[0].z = 0.01f;
	
	layers[7].z = 0.5f;

	addLayersFromSetupArray(this, @layers);

	// cache indices and rot

	for (uint step = 0; step < layers.length; ++step)
	{
		LayerSetup @setup = layers[step];
		bool rot_forwards = (setup.vars & forwards != 0);
		setup.rotationCache = 3.1416f / setup.size.x * (rot_forwards ? -5.0f : 5.0f);
	}

	blob.set("layer setups", @layers);
	
	//setup the dynamic foreground layer :)
	CSpriteLayer@ front = this.addSpriteLayer( "front layer", this.getFilename() , this.getFrameWidth(), this.getFrameHeight(), blob.getTeamNum(), blob.getSkinNum() );

    if (front !is null)
    {
        Animation@ anim = front.addAnimation( "default", 0, false );
        anim.AddFrame(4);
        anim.AddFrame(5);
        front.SetRelativeZ( 1000 );
    }

}

void onTick(CSprite@ this)
{
	CBlob@ blob = this.getBlob();
	LayerSetup[]@ layers;
	if (!blob.get("layer setups", @layers)) return;

	const bool producing = isProducing(blob);
	const bool hasTech = blob.get_string("tech name").size() == 0;	
	this.SetZ(-50);

	//s32 time = (getGameTime() + blob.getNetworkID()*31);   
	f32 hp_frame_ratio = 1.0f-(blob.getHealth() / blob.getInitialHealth());
	for (uint step = 0; step < layers.length; ++step)
	{
		LayerSetup@ setup = layers[step];
		CSpriteLayer@ layer = getLayerFromSetup(this,setup);

		if (hasTech) // !hasTech
		{
			layer.SetVisible(false);
			if (step <= 2)
				continue;
		}
		
		layer.SetVisible(true);

		if (step <= 2)
		{
			if (!producing)
			{
				layer.animation.loop = false;
				continue;
			}
			else
				layer.animation.loop = true;
		}
		
		if (producing && setup.vars & rotates != 0)
		{
			layer.RotateBy( setup.rotationCache , Vec2f_zero );
		}
		if (setup.vars & damage_an != 0)
		{
			layer.animation.frame = hp_frame_ratio * layer.animation.getFramesCount();
		}
	}

}

