﻿// Storage

#include "WARCosts.as";

const Vec2f STORAGE_SIZE(12,5);
string LAST_LABEL;

void onInit( CBlob@ this )
{
	this.set_TileType("background tile", CMap::tile_wood_back);
	this.getSprite().SetZ(-50); //background
	this.getShape().getConsts().mapCollisions = false;
	this.addCommandID("storage in");
	this.addCommandID("storage out");
	this.addCommandID("shipment");
	this.getCurrentScript().tickFrequency = 61;
}

void onTick( CBlob@ this )
{
	PickUpIntoStorage( this );
}
 
// leave a pile of stone after death
void onDie(CBlob@ this)
{
	if (getNet().isServer())
	{
		CBlob@ blob = server_CreateBlob( "mat_wood", this.getTeamNum(), this.getPosition() );
		if (blob !is null)
		{
			blob.server_SetQuantity( COST_WOOD_STORAGE/2 );
		}
	}
}

void onCommand( CBlob@ this, u8 cmd, CBitStream @params )
{
	bool isServer = getNet().isServer();

	if (cmd == this.getCommandID("storage out"))
	{
		CBlob@ caller = getBlobByNetworkID( params.read_netid() );
		CBlob@ item = getBlobByNetworkID( params.read_netid() );
		if (caller !is null && item !is null)
		{
			this.server_PutOutInventory( item );
			item.setPosition( caller.getPosition() ); // put in on the player so it transfers even if it doesn't fit in inv
			if (!caller.server_PutInInventory( item ))
			{
				caller.server_Pickup( item ); // didnt fit in put in hands
			}
		}
	}
	/*else if (cmd == this.getCommandID("storage in"))
	{
		CBlob@ caller = getBlobByNetworkID( params.read_netid() );
		u16 netId = params.read_netid();
		if (netId == 0)
		{
			if (caller !is null) {
				caller.ClearMenus();
			}
			return;
		}
		CBlob@ item = getBlobByNetworkID( netId );
		if (caller !is null && item !is null)
		{	
			this.server_PutInInventory( item );
		}
	}*/ else if( cmd == this.getCommandID("shipment") )
	{
		CBlob@ localBlob = getLocalPlayerBlob();
		if (localBlob !is null && localBlob.getTeamNum() == this.getTeamNum()) {
			client_AddToChat( "Supplies will drop at your storage." );
			Sound::Play("/ShipmentHorn");
		}															  
	}
}

// we use a custom inv
bool isInventoryAccessible( CBlob@ this, CBlob@ forBlob )
{
	return false;
}


void GetButtonsFor( CBlob@ this, CBlob@ caller )
{
	CBitStream params;
	params.write_u16( caller.getNetworkID() );	 
	if ((this.getPosition() - caller.getPosition()).Length() < this.getRadius()*1.0f) 
	{
		BuildSharedInventory( this, caller );
	}
}

CGridButton@ addItem( CGridMenu@ menu, CBlob@ caller, CBlob@ item, u8 cmdID )
{
	CBitStream params;
	params.write_netid( caller.getNetworkID() );
	params.write_netid( item !is null ? item.getNetworkID() : 0 );
	return menu.AddItemButton( item, cmdID, params );
}

void BuildSharedInventory( CBlob@ this, CBlob@ caller, CBlob@ justAddedBlob = null )
{
	if (caller !is null && caller.isMyPlayer())
	{
		caller.ClearMenus();

		CBlob@[] storages;
		getBlobsByName( "storage", @storages );
		uint count = 0;
		for (uint step = 0; step < storages.length; ++step)
		{
			CBlob@ storage = storages[step];
			if (storage.getTeamNum() == this.getTeamNum())
				count++;			
		}

		CControls@ controls = caller.getControls();
		if (count == 1) {
			LAST_LABEL = "Storage";
		}
		else {
			LAST_LABEL = "Shared items from " + count + " storages";
		} 

		CGridMenu@ menu = CreateGridMenu( caller.getScreenPos() + Vec2f(0.0f, 180.0f), this, STORAGE_SIZE, LAST_LABEL );
		if (menu !is null) 
		{
			menu.deleteAfterClick = false;

			// add all storages items

			for (uint step = 0; step < storages.length; ++step)
			{
				CBlob@ storage = storages[step];
				if (storage.getTeamNum() == this.getTeamNum())
				{			
					CInventory@ inv = storage.getInventory();
					for (int i = 0; i < inv.getItemsCount(); i++) {
						CBlob@ item = inv.getItem(i);
						addItem( menu, caller, item, this.getCommandID("storage out") );
					}
				}
			}

			if (justAddedBlob !is null) {
				addItem( menu, caller, justAddedBlob, this.getCommandID("storage out") );
			}

			// click anywhere = put item or close t
			// this is done in engine somehow :/

			//CBlob@ carried = caller.getCarriedBlob();
			////( menu, caller, carried, this.getCommandID("storage in") );
		}
	}
}

void onAddToInventory( CBlob@ this, CBlob@ blob )
{
	CGridMenu@ menu = getGridMenuByName( LAST_LABEL );
	if (menu !is null) 	
	{
		BuildSharedInventory( this, getLocalPlayerBlob(), blob );
	}

	this.getSprite().PlaySound("/StoreSound");
}

void onRemoveFromInventory( CBlob@ this, CBlob@ blob )
{
	CGridMenu@ menu = getGridMenuByName( LAST_LABEL );
	if (menu !is null) 	
	{
		BuildSharedInventory( this, getLocalPlayerBlob() );
	}
}

void PickUpIntoStorage( CBlob@ this )
{
	CBlob@[] blobsInRadius;	   
	CMap@ map = this.getMap();
	if (map.getBlobsInRadius( this.getPosition(), this.getRadius()*3.0f, @blobsInRadius )) 
	{
		for (uint i = 0; i < blobsInRadius.length; i++)
		{
			CBlob @b = blobsInRadius[i];
			const string name = b.getName();
			if (b !is this && !b.isAttached() && b.isOnGround() && b.getShape().vellen < 0.1f
				&& (b.hasTag("material"))
				&& !map.rayCastSolid(this.getPosition(), b.getPosition()))
			{
				this.server_PutInInventory(b);
			}
		}
	}
}