// Trader brain

#define SERVER_ONLY

#include "/Entities/Common/Emotes/EmotesCommon.as"

void onInit( CBrain@ this )
{
    CBlob @blob = this.getBlob();
	this.getCurrentScript().removeIfTag = "dead";
	this.getCurrentScript().runFlags |= Script::tick_not_attached;
}

void onTick( CBrain@ this )
{
    CBlob @blob = this.getBlob();
    u32 gametime = (getGameTime() + blob.getNetworkID()); //!

	// underwater!

	if (blob.isInWater()) 
	{
		blob.setKeyPressed( key_up, true );
		return;

	}

    if (gametime % 211 == 0)  // optimized retarget every couple secs
    {
        FindNewTarget(this, blob);

        if (this.getTarget() is null)
            if (blob.hasTag("at post")) {
                blob.set_u8("emote", Emotes::disappoint);
            }
			else
				this.getCurrentScript().tickFrequency = 1;
    }

    if (this.getTarget() !is null)
    {
        if (!blob.hasTag("at post")) // going to post
        {
			this.getCurrentScript().tickFrequency = 1;
            GoToBlob(this,  this.getTarget() );
        }
        else // at post
		{
			this.getCurrentScript().tickFrequency = 45;						 
			f32 distance = (this.getTarget().getPosition() - blob.getPosition()).Length();	 
			if (distance > 1.2f * blob.getRadius())
			{
				blob.Untag("at post");
				blob.Sync("at post", true);
				this.SetTarget( null );
				blob.set_u8("emote", Emotes::disappoint);
			}

		}
    }
    else // if standing randomly turn
        if (Random(30) == 0) {
            blob.setAimPos( blob.getPosition() + Vec2f( - 100.0f + Random(200), 0.0f) );
        }  
}


bool hasTraderNear( CBlob@ this, CBlob@ post, CBlob@[]@ traders )
{
    Vec2f postPos = post.getPosition();

    for (uint i=0; i < traders.length; i++)
    {
        CBlob@ trader = traders[i];

        if (trader !is this && !trader.hasTag("dead") && (trader.getPosition() - postPos).getLength() < 1.2f * post.getRadius() )
        {
            return true;
        }
    }

    return false;
}

void FindNewTarget( CBrain@ this, CBlob @blob )
{
    CBlob@[] posts;
    getBlobsByName( "tradingpost", @posts );
    CBlob@[] traders;
    getBlobsByName( "trader", @traders );
    CBlob@ closestPost;
    f32 closestDistance = 9999999.9f;
    Vec2f pos = this.getBlob().getPosition();

    for (uint i=0; i < posts.length; i++)
    {
        CBlob@ potential = posts[i];

        if (potential !is blob && potential.getTeamNum() == this.getBlob().getTeamNum() &&
                !potential.isInWater() && !hasTraderNear( this.getBlob(), potential, @traders ) )
        {
            f32 dist = (potential.getPosition() - pos).getLength();

            if (dist < closestDistance)
            {
                closestDistance = dist;
                @closestPost = potential;
            }
        }
    }

    this.SetTarget( closestPost );
										   
    if (closestPost is null)
    {
        this.getBlob().Untag("at post");
        this.getBlob().Sync("at post", true);
    }
}

void GoToBlob( CBrain@ this, CBlob @target )
{
    CBlob @blob = this.getBlob();
    Vec2f targetVector = target.getPosition() - blob.getPosition();
    f32 targetDistance = targetVector.Length();

	if (targetDistance > target.getRadius() * 0.5f)
	{
		// check if we have a clear area to the target
		JustGo( this, target.getPosition() );
										  
		// face the enemy
		blob.setAimPos( target.getPosition() );
		// jump over small blocks
		Vec2f pos = blob.getPosition();

		if ( (blob.isKeyPressed( key_right ) && getMap().isTileSolid( pos + Vec2f(1.3f*blob.getRadius(), 5.0f)*1.2f ) ) ||
			 (blob.isKeyPressed( key_right ) && getMap().isTileSolid( pos + Vec2f(1.3f*blob.getRadius(), -5.0f)*1.2f ) ) ||
			 (blob.isKeyPressed( key_left ) && getMap().isTileSolid( pos + Vec2f(-1.3f*blob.getRadius(), 5.0f)*1.2f ) )	||
			 (blob.isKeyPressed( key_left ) && getMap().isTileSolid( pos + Vec2f(-1.3f*blob.getRadius(), -5.0f)*1.2f ) )
				)
		{
			blob.setKeyPressed( key_up, true );
		}
	}
}

void JustGo( CBrain@ this, Vec2f point )
{
    CBlob @blob = this.getBlob();
    Vec2f mypos = blob.getPosition();
    f32 distance = (point - mypos).Length();

    if (distance > 1.5f * blob.getRadius())
    {
        if (point.x < mypos.x) {
            blob.setKeyPressed( key_left, true );
        }
        else {
            blob.setKeyPressed( key_right, true );
        }

        this.getBlob().Untag("at post");
    }
    else
    {
        this.getBlob().Tag("at post");
    }

    this.getBlob().Sync("at post", true);

    if (distance < 40.0f && point.y + getMap().tilesize + blob.getRadius() < mypos.y) {
        blob.setKeyPressed( key_up, true );
    }
}

