﻿// Trading Post

#include "MakeCrate.as";
#include "MakeSeed.as";
#include "MakeScroll.as";
#include "Requirements.as"
#include "TradingCommon.as"
#include "Descriptions.as";
#include "Help.as"
#include "WARCosts.as";

shared class TradeItem
{
    string name;
    string iconName;
    string configFilename;
    string description;
    CBitStream reqs;
	ShopItem[] techs;
	bool instantShipping;
};

const int DROP_SECS = 15;

void buildFromScrollDef(CBlob@ this, string name, s32 cost, string description)
{
	ScrollDef@ def;
	all_scrolls.get(name, @def);
	if (def !is null)
	{
		addTradeScroll( this, def.name, cost, description, @def.techs );
	}
	else
	{
		error("added missing scroll def by name: "+name);
	}
}

void onInit( CBlob@ this )
{
	scrolls_Setup();
	
    this.addCommandID("stock");
    this.addCommandID("buy");
    this.addCommandID("reload menu");
    AddIconToken( "$"+this.getName()+"$", "TradingPost.png", Vec2f(16,16), 15 );
    AddIconToken( "$parachute$", "Crate.png", Vec2f(32,32), 4 );
    AddIconToken( "$trade$", "MaterialIcons.png", Vec2f(16,16), 5 );
    // shipment vars
    this.set_u32("next drop time", 0 );
    this.set_u32("drop secs", DROP_SECS );
    this.set_u8("drop team", this.getTeamNum() );
    uint[] shipment;
    this.set( "shipment", shipment );
	const string availableAtSpawnText = "\nIf combined with factory provides this item automatically on spawn";
	const string availableAtKnightSpawnText = availableAtSpawnText + " for KNIGHT.";
	const string availableAtArcherSpawnText = availableAtSpawnText + " for ARCHER.";

	//econ techs
	buildFromScrollDef(this, "saw", cost_crappy, descriptions[0]); //TODO
	buildFromScrollDef(this, "drill", cost_crappy, descriptions[0]); //TODO
	
	//siege techs
	buildFromScrollDef(this, "ballista", cost_medium, descriptions[6]);
	buildFromScrollDef(this, "catapult", cost_medium, descriptions[5]);
	buildFromScrollDef(this, "mounted bow", cost_medium, descriptions[31]);
	
	//boats
	
	buildFromScrollDef(this, "longboat", cost_medium, descriptions[33]);
	buildFromScrollDef(this, "armoured boat", cost_medium, descriptions[37]);
	
	//item kits
	buildFromScrollDef(this, "military basics", cost_crappy, descriptions[0]); //TODO
	buildFromScrollDef(this, "explosives", cost_big, descriptions[0]); //TODO
	buildFromScrollDef(this, "pyro", cost_big, descriptions[0]+availableAtArcherSpawnText);
	
	SetHelp( this, "help use", "", "Trader sells technology blueprints.   \n\n         $KEY_E$" ); 

	this.getCurrentScript().tickFrequency = 31;
}

void onTick( CBlob@ this )
{
    // drop crate
    u32 nextDropTime = this.get_u32("next drop time" );	 
    if (getGameTime() >= nextDropTime)
    {
        Ship( this );
    }
}

void GetButtonsFor( CBlob@ this, CBlob@ caller )
{
    CBitStream params;
    params.write_u16(caller.getNetworkID());
    caller.CreateGenericButton( "$trade$", Vec2f(0,-8), this, this.getCommandID("stock"), "Buy technology", params );
}

void BuildTradingMenu( CBlob@ this, CBlob @caller )
{
    TradeItem[]@ items;
    this.get( "items", @items );

    if (caller !is null && caller.isMyPlayer() && items !is null)
    {
        CGridMenu@ menu = CreateGridMenu( this.getScreenPos() + Vec2f(0.0f, 0.0f), this, Vec2f(items.length+3, 1), "Click to buy" );

        if (menu !is null)
        {
            addTradeItemsToMenu(this, menu, caller.getNetworkID());
            menu.deleteAfterClick = false;
        }
    }
}

void addTradeItem( CBlob@ this, const string &in name, int cost, const string &in iconName, const string &in configFilename, const string &in description )
{
    if (!this.exists("items"))
    {
        TradeItem[] items;
        this.set( "items", items );
    }

    TradeItem item;
    item.name = name;
    item.iconName = iconName;
    item.configFilename = configFilename;
    item.description = description;
	item.instantShipping = false;
    AddRequirement( item.reqs, "blob", "mat_gold", "Gold", cost );
    this.push("items", item);
}

void addTradeScroll( CBlob@ this, const string &in name, int cost, const string &in description, ShopItem[]@ techs )
{
	if (!this.exists("items"))
	{
		TradeItem[] items;
		this.set( "items", items );
	}

	TradeItem item;
	item.name = name;
	item.iconName = "$scroll$";
	item.configFilename = "scroll";
	
	string techPrefix = name + "   ";
	for (uint i = 0; i < techs.length; i++)
		techPrefix = techPrefix + "   " + techs[i].iconName;
	techPrefix = techPrefix + "\n\n\n";

	item.description = techPrefix + description;
	item.techs = techs;
	item.instantShipping = true;

	AddRequirement( item.reqs, "blob", "mat_gold", "Gold", cost );
	this.push("items", item);
}


void addTradeItemsToMenu( CBlob@ this, CGridMenu@ menu, u16 callerID)
{
    TradeItem[]@ items;

    if (this.get( "items", @items ))
    {
        CBlob@ caller = getBlobByNetworkID( callerID );

        for (uint i = 0 ; i < items.length; i++)
        {
            TradeItem @item = items[i];
            CBitStream params;
            params.write_u16(callerID);
            params.write_u8( i );
            SetItemDescription( menu.AddButton( item.iconName, item.name, this.getCommandID("buy"), params ), caller, item.reqs, item.description );

			if (i == 1 || i == 4 || i ==6 )
			{
				menu.AddEmptyButton();
			}
        }
    }
}

bool isInRadius( CBlob@ this, CBlob @caller )
{
	return ((this.getPosition() - caller.getPosition()).Length() < this.getRadius()*4.0f + caller.getRadius());
}

void onCommand( CBlob@ this, u8 cmd, CBitStream @params )
{
    if (cmd == this.getCommandID("stock"))
    {
        CBlob@ caller = getBlobByNetworkID( params.read_u16() );
        BuildTradingMenu( this, caller );
    }
    else if (cmd == this.getCommandID("buy"))
    {
        u16 callerid = params.read_u16();
        u8 itemIndex = params.read_u8();
        CBlob@ caller = getBlobByNetworkID( callerid );

        if (caller !is null)
        {
			if (!isInRadius( this, caller ))
			{
				caller.ClearMenus();
				return;
			}

            AddItemToShip( this, caller, itemIndex );
            CBitStream params;
            params.write_u16(callerid);
            //this.server_SendCommandToPlayer( this.getCommandID("reload menu"), params, caller.getPlayer() ); use this for nicer reloading menu - we dont need it for scrolls
			caller.ClearMenus();
        }
    } 
	else if (cmd == this.getCommandID("reload menu"))
    {
        CBlob@ caller = getBlobByNetworkID( params.read_u16() );

        if (caller !is null)
        {
            caller.ClearMenus();
            BuildTradingMenu( this, caller );
        }
	}
}

void AddItemToShip( CBlob@ this, CBlob@ caller, uint itemIndex )
{
	if (caller is null)
		return;

    TradeItem[]@ items;

    if (this.get( "items", @items ) && caller !is null && itemIndex >= 0 && itemIndex < items.length)  {}
    else
    {
        warn("tradingpost.as: no items");
        return;
    }

    TradeItem@ item = items[ itemIndex ];
    CInventory@ inv = caller.getInventory();

    if ( inv is null || item is null)
    {
        warn("tradingpost.as: item or caller inv null");
        return;
    }

    CBitStream missing;

	if (inv.hasRequirements( item.reqs, missing ) )
	{
		inv.server_TakeRequirements( item.reqs );
	}
	else {	  
		//string text, requiredType, name, friendlyName;
		//u16 quantity = 0;
		//missing.ResetBitIndex(); 
		//while (!missing.isBufferEnd())
		//{
		//	ReadRequirement( missing, requiredType, name, friendlyName, quantity );
		//	print("MISSING : " + friendlyName + " = " + quantity );
		//}	   
		return;
	}

	if (item.instantShipping)
	{
		CBlob@ blob = MakeBlobFromItem( item );
		if (blob !is null)
		{	 
			caller.server_Pickup( blob );
		}
	}
	else
	{			   
		uint[]@ shipment;
		this.get( "shipment", @shipment );

		if (shipment.length == 0 && getNet().isServer())
		{
			// new shipment - start countdown
			this.set_u32("next drop time", getGameTime() + this.get_u32("drop secs") * getTicksASecond() );
			this.Sync("next drop time", true );
			this.set_u8("drop team", caller.getTeamNum() );
		}

		shipment.push_back( itemIndex );
	}

	this.getSprite().PlaySound( "ChaChing" );
}

void Ship( CBlob@ this )
{
    uint[]@ shipment;

    if (this.get( "shipment", @shipment ) && shipment.length > 0)
    {
        if (getNet().isServer()) {
            RecursiveCrate( this, shipment, 0, null );
        }

        shipment.clear();
    }
}

void RecursiveCrate( CBlob@ this, uint[]@ shipment, uint index, CBlob@ itemThatDidntFit )
{
    if (index < shipment.length || itemThatDidntFit !is null)
    {
        TradeItem[]@ items;

        if (!this.get( "items", @items ))
        {
            warn("Trading post: no items");
            return;
        }

        CBlob@ crate = server_MakeCrateOnParachute( "", "", 0, this.get_u8("drop team"), getDropPosition( this.getPosition() ) );

        if (crate !is null)
        {
            if (itemThatDidntFit !is null)
                if (!crate.server_PutInInventory( itemThatDidntFit ))
                {
                    // doesn't fit at all - spawn a dedicated crate
                    string configName = itemThatDidntFit.getName();
                    server_MakeCrateOnParachute( configName, itemThatDidntFit.getInventoryName(), 1 + Random(2), this.get_u8("drop team"), getDropPosition( this.getPosition() ) );
                    itemThatDidntFit.server_Die();

                    if (index >= shipment.length) // destroy crates if nothing more left
                    {
                        crate.server_Die();
                    }
                }

            // put stuff in crate recursively until all items used

            for (uint i = index; i < shipment.length; i++)
            {
				TradeItem@ item = items[shipment[i]];  				
				CBlob@ blobItem = MakeBlobFromItem( item );

                if (blobItem !is null)
                {
                    if (!crate.server_PutInInventory( blobItem ))
                    {
                        if (crate.getInventory().getItemsCount() == 0) {
                            crate.server_Die();
                        }

                        RecursiveCrate( this, shipment, i+1, blobItem );
                        return;															
                    }
                }
            }
        }
    }
}

CBlob@ MakeBlobFromItem( TradeItem@ item )
{
	if (item.configFilename == "scroll") {
		return server_MakeTechScroll( Vec2f_zero, item.name, @item.techs );
	}
	else {
		return MakeItem( item.configFilename );
	}
}

CBlob@ MakeItem( const string& in name )
{
    string[]@ tokens = name.split(" ");

    if (tokens.length > 1 && tokens[0] == "seed") { // seed ...
        return server_MakeSeed( Vec2f_zero, tokens[1] );
    } 
    else {
        return server_CreateBlob( name );    // normal blob
    }
}

// render countdown

void onRender( CSprite@ this )
{
    CBlob@ blob = this.getBlob();
    Vec2f pos2d = blob.getScreenPos();
    s32 nextDropTime = blob.get_u32("next drop time" );
    u32 gameTime = getGameTime();
    s32 dropTime = Maths::Max(0,gameTime-60);

    if (nextDropTime > dropTime)
    {
        // draw drop time progress bar
        int top = pos2d.y - 2.5f*blob.getHeight() - 50.0f + 7.0f * Maths::Sin( (getGameTime() + blob.getNetworkID()) / 7.0f );  // bounce a bit
        int margin = 7;
        Vec2f dim;
        int secs = 1 + (nextDropTime - gameTime+60) / getTicksASecond();
        string label = "Next drop in " + secs + "s";
        GUI::GetTextDimensions( label , dim );
        dim.x += margin;
        dim.y += margin;
        dim.y *= 1.8f;
        Vec2f upperleft(pos2d.x-dim.x/2, top - 2*dim.y);
        Vec2f lowerright(pos2d.x+dim.x/2, top - dim.y);
        f32 progress = 1.0f - (float(secs-1) / float(blob.get_u32("drop secs")));
        GUI::DrawProgressBar( upperleft,lowerright, progress );
        GUI::DrawIconByName( "$parachute$", Vec2f(upperleft.x + 26.0f, upperleft.y - 8.0f) );
        // draw items waiting for shipment
        CBlob@ localBlob = getLocalPlayerBlob();

        if (localBlob !is null && (
                    ((localBlob.getPosition() - blob.getPosition()).Length() < (localBlob.getRadius() + blob.getRadius())) &&
                    (getHUD().hasButtons()) ))
        {
            TradeItem[]@ items;

            if (!blob.get( "items", @items ))
            {
                warn("Trading post: no items");
                return;
            }

            uint[]@ shipment;
            blob.get( "shipment", @shipment );

            for (uint i = 0; i < shipment.length; i++)
            {
                int top2 = top - 2*dim.y;
                Vec2f upperleft(pos2d.x - dim.x - dim.x/3.0f , top2 + i*dim.y);
                Vec2f lowerright(upperleft.x + dim.x/2.0f, top2 + dim.y + i*dim.y);
                // above the screen?  temp shitty code
                uint tooMuch = 8;

                if (i > tooMuch)
                {
                    f32 height = lowerright.y - upperleft.y;
                    upperleft.Set(upperleft.x - dim.x/2.0f, top2 + (i-tooMuch-1)*dim.y);
                    lowerright.Set(lowerright.x - dim.x/2.0f, top2 + dim.y + (i-tooMuch-1)*dim.y);
                }

                Vec2f iconDim;
                const string iconName = items[shipment[i]].iconName;
                GUI::GetIconDimensions( iconName, iconDim );
                GUI::DrawSunkenPane( upperleft, lowerright );
                GUI::DrawIconByName( iconName, Vec2f(upperleft.x + 32.0f - iconDim.x, upperleft.y + 16.0f - iconDim.y ) );
            }
        }
    }
}
