// Bomb logic

#include "Hitters.as";
#include "Help.as";

void onInit( CBlob@ this )
{
    s32 bomb_fuse = 120;
    this.set_s32("bomb_timer", getGameTime()+bomb_fuse);
    this.set_u16("explosive_parent",0);
    f32 explRadius = 64.0f;
    this.set_f32("explosive_radius",explRadius);
    this.set_f32("explosive_damage",3.0f);
    this.getSprite().SetEmitSound( CFileMatcher("Sparkle.ogg").getFirst() );
    this.SetLight( true );
    this.SetLightRadius( explRadius*0.5f );
    this.getShape().getVars().waterDragScale = 8.0f;
	this.getShape().getConsts().bullet = true;
    //use the bomb hitter
    this.set_u8("custom_hitter", Hitters::bomb);
    this.set_f32("map_damage_radius", 24.0f);
    this.set_f32("map_damage_ratio", 0.4f);
    this.set_bool("map_damage_raycast", true);
	this.Tag("exploding");
    //
	this.Tag("activated"); // make it lit already and throwable
	SetHelp( this, "help throw", "knight", "$mat_bombs$ AGAIN to throw!\n\n       $KEY_SPACE$", "$mat_bombs$ Throw bomb.\n\n       $KEY_C$",2 ); 
}

//start ugly bomb logic :)

void set_delay( CBlob@ this, string field, s32 delay)
{
    this.set_s32(field,getGameTime()+delay);
}

void onTick( CBlob@ this )
{
    s32 timer = this.get_s32("bomb_timer") - getGameTime();

    //set parent from attached blob

    if (getNet().isServer())
    {
        CBlob@ parent = this.getAttachments().getAttachedBlob("PICKUP", 0);

        if (parent !is null)
        {
            u16 oldParentID = this.get_u16("explosive_parent");
            u16 newParentID = parent.getNetworkID();

            if (oldParentID != newParentID)
            {
                this.set_u16("explosive_parent", newParentID);
                this.SetDamageOwnerPlayer( parent.getPlayer() );
                //this.Sync("explosive_parent", true); we dont need this synced
            }
        }
    }

    //hack, old bomb physics (because they just feel so good)
    Vec2f vel = this.getVelocity();
    const f32 maxVel = 8.0f;

    if (vel.x > maxVel) {
        vel.x = maxVel;
    }

    if (vel.x < -maxVel) {
        vel.x = -maxVel;
    }

    if (this.isOnGround() || this.isOnCeiling()) {
        vel.x *= 0.8;
    }

    if (vel.y > maxVel) {
        vel.y = maxVel;
    }

    if (vel.y < -maxVel) {
        vel.y = -maxVel;
    }

    this.setVelocity(vel);

    if (timer <= 0)
    {
        if (getNet().isServer()) {
            Boom( this );
        }
    }
    else
    {
        SColor lightColor = SColor( 255, 255, Maths::Min(255, 2*timer), 0);
        this.SetLightColor( lightColor );

        if (Random(2) == 0)
        {
            sparks( this.getPosition(), this.getAngleDegrees(), 3.5f+(Random(10)/5.0f), lightColor );
        }
    }
}

void Boom( CBlob@ this )
{
    this.server_SetHealth(-1.0f);
    this.server_Die();
    //this.RemoveScript("Entities/Items/Explosives/Bomb.as");
}

//sprite update

void onTick( CSprite@ this )
{
    CBlob@ blob = this.getBlob();
    Vec2f vel = blob.getVelocity();
    
    s32 timer = blob.get_s32("bomb_timer") - getGameTime();

    if (timer < 0) {
        return;
    }

    if (timer > 30)
    {
        this.SetAnimation("default");
        this.animation.frame = this.animation.getFramesCount() * (1.0f - ((timer-30) / 220.0f));
    }
    else
    {
        this.SetAnimation("shes_gonna_blow");
        this.animation.frame = this.animation.getFramesCount() * (1.0f - (timer / 30.0f));

        if (timer < 15 && timer > 0)
        {
            f32 invTimerScale = (1.0f-(timer / 15.0f));
            Vec2f scaleVec = Vec2f(1,1)*(1.0f + 0.07f*invTimerScale*invTimerScale );
            this.ScaleBy( scaleVec );
        }
    }
}

bool isExplosionHitter(u8 hitter)
{
    return ( hitter == Hitters::bomb );
}

//dont get hurt by anything atm
f32 onHit( CBlob@ this, Vec2f worldPoint, Vec2f velocity, f32 damage, CBlob@ hitterBlob, u8 customData )
{
    if ( isExplosionHitter(customData) )
    {
        return damage; //maybe do something special here
    }

    return 0.0f;
}

void onDie( CBlob@ this )
{
    this.getSprite().SetEmitSoundPaused( true );
}

void onCollision( CBlob@ this, CBlob@ blob, bool solid )
{
    if (!solid) {
        return;
    }

    f32 vellen = this.getVelocity().Length();

    if (vellen > 1.7f)
    {
        Sound::Play( "Entities/Items/Explosives/BombBounce.ogg", this.getPosition(), Maths::Min( vellen/8.0f, 1.1f ) );
    }
}

void sparks(Vec2f at, f32 angle, f32 speed, SColor color )
{
    Vec2f vel = getRandomVelocity(angle+90.0f, speed, 25.0f);
    at.y -= 2.5f;
    ParticlePixel( at, vel, color, true, 119 );
}

// collide with everything except players
//bool doesCollideWithBlob( CBlob@ this, CBlob@ blob )
//{
//    return ( !blob.hasTag("player") || blob.hasTag("shielded") );
//}
//

// run the tick so we explode in inventory
void onThisAddToInventory( CBlob@ this, CBlob@ inventoryBlob )
{
    this.doTickScripts = true;
    //this.getSprite().SetEmitSoundPaused( false );
}
