
#include "Hitters.as";
#include "ShieldCommon.as";
#include "FireParticle.as"

const f32 arrowMediumSpeed = 8.0f;
const f32 arrowFastSpeed = 13.0f;
//maximum is 15 as of 22/11/12 (see ArcherCommon.as)

//Arrow logic

//blob functions
void onInit( CBlob@ this )
{
    this.server_SetTimeToDie( 20 );
    this.getShape().getConsts().mapCollisions = false;	 // weh ave our own map collision
	this.getShape().getConsts().bullet = true;
	
	CSprite@ sprite = this.getSprite();
    //set a random frame
    {
		Animation@ anim = sprite.addAnimation("arrow",0,false);
		anim.AddFrame(Random(4));
		sprite.SetAnimation(anim);
	}

	{
		Animation@ anim = sprite.addAnimation("fire arrow",0,false);
		anim.AddFrame(8);
	}
}

void turnOffFire(CBlob@ this)
{
	this.SetLight(false);
	this.Untag("fire arrow");
	this.Untag("fire source");
	this.getSprite().SetAnimation("arrow");
}

void onTick( CBlob@ this )
{
	f32 angle;
    if (!this.hasTag("collided")) //we haven't hit anything yet!
    {
        angle = (this.getVelocity()).Angle();
        Pierce( this ); //map
        this.setAngleDegrees(-angle);
    }
    else // sticking
    {
        angle = Maths::get360DegreesFrom256(this.get_u8( "angle" ));
        this.setVelocity(Vec2f(0,0));
        this.setPosition(this.get_Vec2f("lock"));
        this.getShape().SetStatic(true);
        this.getShape().getConsts().collidable = false;
    }

    // fire arrow
    if (this.hasTag("fire arrow"))
    {

		if (getGameTime() % 3 == 1)
		{
			CBlob@[] overlapping;
			if (this.getOverlapping( @overlapping ))
			{
				for (uint i = 0; i < overlapping.length; i++)
				{
					CBlob@ overlap = overlapping[i];

					if (overlap.hasTag("player") && overlap.getVelocity().y > 1)
					{
						turnOffFire(this);
						break;
					}
				}
			}
		}
		else if (getGameTime() % 6 == 0)
		{
			this.getSprite().SetAnimation("fire");
			this.Tag("fire source");
			
			Vec2f offset = Vec2f(this.getWidth(),0.0f);
			offset.RotateBy( -angle );
			makeFireParticle( this.getPosition() + offset, 4  );

			if (!this.isInWater())
			{
				this.SetLight(true);
				this.SetLightColor( SColor( 255, 250,215,178 ) );
				this.SetLightRadius( 20.5f );
			}
			else {
				turnOffFire(this);
			}
		}
	}
}

void onCollision( CBlob@ this, CBlob@ blob, bool solid, Vec2f normal, Vec2f point1 )
{
    if (blob !is null && doesCollideWithBlob( this, blob ) && !this.hasTag("collided"))
    {
		if (!solid && !blob.hasTag("flesh"))
			return;

		Vec2f initVelocity = this.getVelocity();
		f32 vellen = initVelocity.Length();

		f32 dmg = blob.getTeamNum() == this.getTeamNum() ? 0.0f : getArrowDamage( this );
			
		// this isnt synced cause we want instant collision for arrow even if it was wrong
		ArrowHitBlob( this, point1, initVelocity, dmg, blob, Hitters::arrow );
		this.server_Hit( blob, point1, initVelocity, dmg, Hitters::arrow);
		if (this.hasTag("fire arrow"))
		{
			this.server_Hit( blob, point1, initVelocity, 0.5f, Hitters::fire);
		}
			
		this.Tag("collided");
	}
}

bool doesCollideWithBlob( CBlob@ this, CBlob@ blob )
{
	return (this.getTeamNum() != blob.getTeamNum() || (blob.getShape().isStatic() && !blob.getShape().getConsts().platform));
}

void Pierce( CBlob @this )
{
    CMap@ map = this.getMap();
	Vec2f end;
	
	if (map.rayCastSolidNoBlobs(this.getShape().getVars().oldpos, this.getPosition() ,end))
	{
		ArrowHitMap( this, end, this.getVelocity(), 0.5f, Hitters::arrow );
	}
}

void ArrowHitBlob( CBlob@ this, Vec2f worldPoint, Vec2f velocity, f32 damage, CBlob@ hitBlob, u8 customData )
{
    if (hitBlob !is null)
    {
        CSprite@ sprite = hitBlob.getSprite();

        if (sprite !is null)
        {
            CSpriteLayer@ arrow = sprite.addSpriteLayer( "arrow", "Entities/Items/Projectiles/Arrow.png", 16, 8, this.getTeamNum(), this.getSkinNum() );

            if (arrow !is null)
            {
                Animation@ anim = arrow.addAnimation( "default", 13, true );

                if (this.getSprite().animation !is null) {
                    anim.AddFrame(4+Random(4));    //always use broken frame
                }
                else
                {
                    warn("exception: arrow has no anim");
                    anim.AddFrame(0);
                }

                arrow.SetAnimation( anim );
				Vec2f normal = worldPoint-hitBlob.getPosition();
				f32 len = normal.Length();
				if (len > 0.0f)
					normal /= len;
                Vec2f soffset = normal * (len + 0);

                if(hitBlob.getMovement() is null) // wow, this is shit
                {
                   // soffset.x *= -1;
                    arrow.RotateBy(180.0f, Vec2f(0,0));
					arrow.SetFacingLeft(true);
                }
				else
				{
					soffset.x *= -1;
					//arrow.SetFacingLeft(false);	// the blob changes facing and reverts the arrow when it is point down or up which is bad :/
				}


                arrow.SetOffset(soffset);
                arrow.SetRelativeZ(-0.01f);

                f32 angle = velocity.Angle();
                arrow.RotateBy(-angle - hitBlob.getAngleDegrees(), Vec2f(0,0));
            }
        }

		// check if shielded
		const bool hitShield = (hitBlob.hasTag("shielded") && blockAttack(hitBlob, velocity, 0.0f));
				
		// play sound
		if (!hitShield)
		{
			if (hitBlob.hasTag("flesh"))
			{
				{
					if (velocity.Length() > arrowFastSpeed) {
						this.getSprite().PlaySound( "ArrowHitFleshFast.ogg" );
					}
					else {
						this.getSprite().PlaySound( "ArrowHitFlesh.ogg" );
					}
				}
			}
			else
			{
				if (velocity.Length() > arrowFastSpeed) {
					this.getSprite().PlaySound( "ArrowHitGroundFast.ogg" );
				}
				else {
					this.getSprite().PlaySound( "ArrowHitGround.ogg" );
				}
			}
		}

        //this.getShape().server_SetActive( false );
        this.getShape().SetStatic(true); // for client

        {
            this.getSprite().SetVisible(false);
            this.doTickScripts = false;
            this.server_Die();
        }
    }
}

void ArrowHitMap( CBlob@ this, Vec2f worldPoint, Vec2f velocity, f32 damage, u8 customData )
{
	if (velocity.Length() > arrowFastSpeed) {
		this.getSprite().PlaySound( "ArrowHitGroundFast.ogg" );
	}
	else {
		this.getSprite().PlaySound( "ArrowHitGround.ogg" );
	}
	
	f32 radius = this.getRadius();
	
	f32 angle = velocity.Angle();
	
	this.set_u8( "angle", Maths::get256DegreesFrom360(angle)  );
	
	Vec2f norm = velocity;
	norm.Normalize();
	norm *= (1.5f * radius);
	Vec2f lock = worldPoint - norm;
	this.set_Vec2f( "lock", lock );
	
	this.Sync("lock",true);
	this.Sync("angle",true);
	
	this.setVelocity(Vec2f(0,0));
	this.setPosition(lock);
	//this.getShape().server_SetActive( false );
	this.getShape().SetStatic(true); // for client
	
	if (this.hasTag("fire arrow"))
		this.set_Vec2f( "fire pos", (worldPoint + (norm * 0.5f) ) );
	
	this.Tag("collided");
	
	this.server_SetTimeToDie( 5 );
    //  this.doTickScripts = false;
}

void FireUp( CBlob@ this )
{
	Vec2f burnpos;
	
	if (this.exists("fire pos"))
		burnpos = this.get_Vec2f( "fire pos" );
	else
	{
		Vec2f head = Vec2f(this.getRadius()*1.2f,0.0f);
		f32 angle = this.getAngleDegrees();
		head.RotateBy( angle );
		
		burnpos = this.getPosition() + head;
	}
	
	// this.getMap() NULL ON ONDIE!
	CMap@ map = getMap();
	if (map !is null)
	{
		getMap().server_setFireWorldspace( burnpos,true );
		getMap().server_setFireWorldspace( this.getPosition() ,true); //burn where i am as well
	}
}

void onDie( CBlob@ this )
{
	// fire arrow!
	if (this.hasTag("fire arrow")) {
		FireUp(this);
	}
}

void onThisAddToInventory( CBlob@ this, CBlob@ inventoryBlob )
{
    if (!getNet().isServer()) {
        return;
    }

    // merge arrow into mat_arrows

    for (int i = 0; i < inventoryBlob.getInventory().getItemsCount(); i++)
    {
        CBlob @blob = inventoryBlob.getInventory().getItem(i);

        if (blob !is this && blob.getName() == "mat_arrows")
        {
            blob.server_SetQuantity( blob.getQuantity() + 1 );
            this.server_Die();
            return;
        }
    }

    // mat_arrows not found
    // make arrow into mat_arrows
    CBlob @mat = server_CreateBlob( "mat_arrows" );

    if (mat !is null)
    {
        inventoryBlob.server_PutInInventory( mat );
        mat.server_SetQuantity(1);
        this.server_Die();
    }
}

f32 onHit( CBlob@ this, Vec2f worldPoint, Vec2f velocity, f32 damage, CBlob@ hitterBlob, u8 customData )
{
	if (customData == Hitters::water) //splash by bucket
	{
		this.Untag("fire arrow");
	}

    return damage;
}


f32 getArrowDamage( CBlob@ this )
{
    f32 vellen = this.getVelocity().Length();

    //print("vellen " + vellen);
    if (vellen >= arrowFastSpeed) {
        return 2.0f;
    }
    else if (vellen >= arrowMediumSpeed) {
        return 1.0f;
    }

    return 0.5f;
}
