
#include "/Entities/Common/Attacks/Hitters.as";
#include "/Entities/Common/Attacks/ShieldCommon.as";

#include "/Entities/Common/Attacks/LimitedAttacks.as";

//Bolt logic

//blob functions
void onInit( CBlob@ this )
{
    this.set_u8( "blocks_pierced", 0 );
    this.set_u8( "state", 0 );
    this.server_SetTimeToDie( 20 );
    this.getShape().getConsts().mapCollisions = false;
	this.getShape().getConsts().bullet = true;
    LimitedAttack_setup(this);
    u32[] tileOffsets;
    this.set( "tileOffsets", tileOffsets );
}

void onTick( CBlob@ this )
{
    u8 state = this.get_u8( "state" );
    f32 angle = 0;

    if (state == 0) //we haven't hit anything yet!
    {
        angle = (this.getVelocity()).Angle();
        Pierce( this, angle );
    }
    else // sticking
    {
        angle = Maths::get360DegreesFrom256(this.get_u8( "angle" ));
        this.setVelocity(Vec2f(0,0));
        this.setPosition(Vec2f(this.get_f32( "lock_x"),this.get_f32( "lock_y")));
        this.getShape().SetStatic(true);
        this.doTickScripts = false;
    }

    this.setAngleDegrees(-angle);
}

bool doesCollideWithBlob( CBlob@ this, CBlob@ blob )	// not used by engine - collides = false
{
	return (this.getTeamNum() != blob.getTeamNum() || (blob.getShape().isStatic() && !blob.getShape().getConsts().platform));
}

void Pierce( CBlob @this, f32 angle )
{
    Vec2f initVelocity = this.getVelocity();
    f32 vellen = this.getShape().vellen;
    HitInfo@[] hitInfos;

	if (vellen > 0.1f)
    if (this.getMap().getHitInfosFromArc( this.getPosition(), -angle, 0, vellen, this, false, @hitInfos ))
    {
        for (uint i = 0; i < hitInfos.length; i++)
        {
            HitInfo@ hi = hitInfos[i];
            f32 dmg = 2.0f;

            if (hi.blob !is null) // blob
            {
                if ( !hi.blob.isCollidable() || !doesCollideWithBlob(this, hi.blob)) {
                    continue;
                }

                if ( LimitedAttack_has_hit_actor(this, hi.blob) ) {
                    continue;
                }

                BallistaHitBlob( this, hi.hitpos, initVelocity, dmg, hi.blob, Hitters::ballista ); // this isnt synced cause we want instant collision for arrow even if it was wrong
                this.server_Hit( hi.blob, hi.hitpos, initVelocity, dmg, Hitters::ballista);
                LimitedAttack_add_actor(this, hi.blob);
            }
            else // map
            {
                BallistaHitMap( this, hi.hitpos,initVelocity, dmg, Hitters::ballista ); // this isnt synced cause we want instant collision for arrow even if it was wrong
                this.server_HitMap( hi.hitpos, initVelocity, dmg, Hitters::ballista );
            }
        }
    }
}

void BallistaHitBlob( CBlob@ this, Vec2f worldPoint, Vec2f velocity, f32 damage, CBlob@ hitBlob, u8 customData )
{
    if (hitBlob !is null)
    {
        // check if shielded
        if (hitBlob.hasTag("flesh"))
        {
            this.getSprite().PlaySound( "ArrowHitFleshFast.ogg" );
        }
        else
        {
            this.getSprite().PlaySound( "ArrowHitGroundFast.ogg" );
        }
    }
}

void BallistaHitMap( CBlob@ this, Vec2f worldPoint, Vec2f velocity, f32 damage, u8 customData )
{
    //check if we've already hit this tile
    u32[]@ offsets;
    this.get( "tileOffsets", @offsets );

    CMap@ map = getMap();
	u32 tileOffset = map.getTile(worldPoint); 


    if( offsets.find(tileOffset) >= 0 ) { return; }

    this.getSprite().PlaySound( "ArrowHitGroundFast.ogg" );
    f32 angle = velocity.Angle();
    TileType t = map.getTile(tileOffset);
    u8 blocks_pierced = this.get_u8( "blocks_pierced" );
    bool stuck = false;

    if (!map.isTileGround(t))
    {
        Vec2f tpos = worldPoint;
        map.server_DestroyTile( tpos, 1.0f );
        Vec2f vel = this.getVelocity();
        this.setVelocity(vel * 0.5f); //damp
        this.setPosition( tpos + Vec2f(4,4) - vel);
        this.push( "tileOffsets", tileOffset );

        if (blocks_pierced < 1)
        {
            blocks_pierced++;
            this.set_u8( "blocks_pierced", blocks_pierced );
        }
        else {
            stuck = true;
        }
    }
    else
    {
        stuck = true;
    }

    if (stuck)
    {
        this.set_u8( "state", 1 );
        this.set_u8( "angle", Maths::get256DegreesFrom360(angle)  );
        Vec2f norm = velocity;
        norm.Normalize();
        norm *= this.getRadius() * 0.5f;
        Vec2f lock = worldPoint - norm;
        this.set_f32( "lock_x", lock.x );
        this.set_f32( "lock_y", lock.y );
        this.Sync("state",true);
        this.Sync("lock_x",true);
        this.Sync("lock_y",true);
        this.setVelocity(Vec2f(0,0));
        this.setPosition(lock);
        this.getShape().SetStatic(true); // for client
        this.doTickScripts = false;
    }
}
