#include "TechsCommon.as"
#include "Descriptions.as"
#include "ScrollCommon.as"

// from def

CBlob@ server_MakePredefinedScroll( Vec2f atpos, const string &in name )
{
	if (!getNet().isServer()) { return null; }
	
	scrolls_Setup();

	ScrollDef@ def;
	all_scrolls.get(name, @def);
	if (def !is null)
	{
		CBlob@ blob = server_CreateBlobNoInit( "scroll" ); 
		if (blob !is null)
		{
			blob.setPosition( atpos );
			blob.set_string( "scroll name", def.name );
			blob.Init();

			for (uint i = 0; i < def.scripts.length; i++) {
				blob.AddScript( def.scripts[i] );
			} 
			
			for (uint i = 0; i < def.techs.length; i++)
			{
				blob.Tag("tech");
				ShopItem@ tech = def.techs[i];
				addTechItem( blob, tech.name, tech.blobName, tech.iconName, "", tech.ticksToMake, tech.spawnInCrate, tech.quantityLimit, @tech.requirements );
			}
			
			blob.Init();
			
		}
		return blob;
	}
	
	return null;
}

// script scroll

CBlob@ server_MakeScriptScroll( Vec2f atpos, const string &in name, string[] scripts  )
{
	if (!getNet().isServer()) { return null; }

	CBlob@ blob = server_CreateBlobNoInit( "scroll" ); 
	if (blob !is null)
	{
		blob.setPosition( atpos );
		blob.set_string( "scroll name", name );
		blob.Init();

		for (uint i = 0; i < scripts.length; i++) {
			blob.AddScript( scripts[i] );
		} 
	}		  
	return blob;
}

// tech scroll

CBlob@ server_MakeTechScroll( Vec2f atpos, const string &in name, ShopItem[]@ techs  )
{
	if (!getNet().isServer()) { return null; }

	CBlob@ blob = server_CreateBlobNoInit( "scroll" ); 
	if (blob !is null)
	{
		blob.setPosition( atpos );
		blob.set_string( "scroll name", name );

		for (uint i = 0; i < techs.length; i++)
		{
			ShopItem@ tech = techs[i];
			addTechItem( blob, tech.name, tech.blobName, tech.iconName, "", tech.ticksToMake, tech.spawnInCrate, tech.quantityLimit, @tech.requirements );
		}


		blob.Tag("tech");
		blob.Init();
	}		  
	return blob;	
}	   

void addScrollTechToArray( const string &in niceName, const string &in blobName, const u16 timeToMakeSecs, const bool spawnInCrate, const u8 quantityLimit, ShopItem[]@ techs, CBitStream @reqs = null )
{		
	ShopItem item;
	
	item.name = niceName;
	item.blobName = blobName;
	item.iconName = "$"+blobName+"$";
	item.ticksToMake = timeToMakeSecs; // ! we take the secs so we don't have to convert later
	item.spawnInCrate = spawnInCrate;
	item.quantityLimit = quantityLimit;
	if (reqs !is null)
		item.requirements = reqs;
	
	item.spawnToInventory = false;
	item.timeCreated = 0;

	techs.push_back( item );
}	  
