
//script for an aquatic animal - attach to:
// blob
// movement
// 		vars:		f32 swimspeed f32 swimforce
// brain
//		vars: 		lots

#define SERVER_ONLY

#include "Hitters.as";
#include "PressOldKeys.as";
#include "AquaticAnimalConsts.as";

//blob
void onInit(CBlob@ this)
{
	this.getShape().getVars().waterDragScale = 1.0f; //water drag == regular drag

	if (!this.exists("swimspeed"))
		this.set_f32("swimspeed", 1.0f);
	if (!this.exists("swimforce"))
		this.set_f32("swimforce", 0.7f);

	// force no team
	this.setTeamNum(-1);

	this.getCurrentScript().runFlags |= Script::tick_not_inwater;
	this.getCurrentScript().runFlags |= Script::tick_onground;
	this.getCurrentScript().runFlags |= Script::tick_not_attached;
	this.getCurrentScript().removeIfTag	= "dead";
}

void onTick(CBlob@ this)
{
	this.setVelocity(Vec2f( this.isKeyPressed(key_left) ? -1.0f : 1.0f, -((10 + Random(30)) * 0.1f)));
}


//brain

enum modes {
	MODE_IDLE = 0,
	MODE_TARGET,
	MODE_FLEE,
}

void onInit( CBrain@ this )
{
    CBlob @blob = this.getBlob();
    blob.set_u8( delay_property , 5+Random(5));
    blob.set_u8(state_property, MODE_IDLE);
    
    blob.set_Vec2f(terr_pos_property, blob.getPosition());
    
    if (!blob.exists(terr_rad_property)) 
	{
		blob.set_f32(terr_rad_property, 32.0f);
	}
	    
    if (!blob.exists(target_searchrad_property))
	{
		blob.set_f32(target_searchrad_property, 32.0f);
	}
	
	if (!blob.exists(personality_property))
	{
		blob.set_u8(personality_property,0);
	}

	this.getCurrentScript().removeIfTag	= "dead";   
	this.getCurrentScript().runFlags |= Script::tick_blob_in_proximity;
	this.getCurrentScript().runFlags |= Script::tick_not_attached;
	this.getCurrentScript().runProximityTag = "player";
	this.getCurrentScript().runProximityRadius = 200.0f;
	//this.getCurrentScript().tickFrequency = 5;
}


void onTick( CBrain@ this )
{
    CBlob @blob = this.getBlob();

	u8 delay = blob.get_u8(delay_property);
	delay--;
	
	if (delay == 0)
	{
		delay = 4+Random(8);

		Vec2f pos = blob.getPosition();
		bool facing_left = blob.isFacingLeft();

		{
			u8 mode = blob.get_u8(state_property);
			u8 personality = blob.get_u8(personality_property);

			//"blind" attacking
			if (mode == MODE_TARGET)
			{
				CBlob@ target = getBlobByNetworkID(blob.get_netid(target_property));

				if (target is null || Random(10) == 0 )
				{
					mode = MODE_IDLE;
				}
				else
				{
					Vec2f tpos = target.getPosition();
					
					f32 search_radius = blob.get_f32(target_searchrad_property);
					
					if ((tpos - pos).LengthSquared() >= (search_radius*search_radius))
					{
						mode = MODE_IDLE;
					}
					{					
						blob.setKeyPressed( (tpos.x < pos.x) ? key_left : key_right, true);
						blob.setKeyPressed( (tpos.y < pos.y) ? key_up : key_down, true);
					}
				}
			}
			//"blind" fleeing
			else if (mode == MODE_FLEE)
			{
				CBlob@ target = getBlobByNetworkID(blob.get_netid(target_property));

				if (target is null)
				{
					mode = MODE_IDLE;
				}
				else
				{
					Vec2f tpos = target.getPosition();
					
					f32 search_radius = blob.get_f32(target_searchrad_property);
					
					if ((tpos - pos).LengthSquared() >= (search_radius*search_radius)*3.0f)
					{
						mode = MODE_IDLE;
					}
					else
					{
						blob.setKeyPressed( (tpos.x > pos.x) ? key_left : key_right, true);
						blob.setKeyPressed( (tpos.y > pos.y) ? key_up : key_down, true);
					}
				}


			}
			else //mode == idle
			{
				if (personality != 0) //we have a special personality
				{
					f32 search_radius = blob.get_f32(target_searchrad_property);
					string name = blob.getName();
					
					CBlob@[] blobs;
					blob.getMap().getBlobsInRadius( pos, search_radius, @blobs );

					for (uint step = 0; step < blobs.length; ++step)
					{
						//TODO: sort on proximity? done by engine?
						CBlob@ other = blobs[step];
						
						if (other is blob) continue; //lets not run away from / try to eat ourselves...
						
						if ( personality & SCARED_BIT != 0 ) //scared
						{
							if (other.getName() != name) //not scared of same type of blob
							{
								mode = MODE_FLEE;
								blob.set_netid(target_property,other.getNetworkID());
								break;
							}
						}
						
						if (personality & AGGRO_BIT != 0 ) //aggressive
						{
							//TODO: flags for these...
							if (other.getName() != name && //dont eat same type of blob
								other.hasTag("flesh")) //attack flesh blobs
							{
								mode = MODE_TARGET;
								blob.set_netid(target_property,other.getNetworkID());
								break;
							}
						}
					}
				}

				Vec2f territory_pos = blob.get_Vec2f(terr_pos_property);
				f32 territory_range = blob.get_f32(terr_rad_property);
				
				Vec2f territory_dir = (territory_pos - pos);
				
				if (territory_dir.LengthSquared() > (territory_range * territory_range))
				{
					//head towards territory
					
					blob.setKeyPressed( (territory_dir.x < 0.0f) ? key_left : key_right, true);
					blob.setKeyPressed( (territory_dir.y > 0.0f) ? key_down : key_up, true);
					
				}
				else
				{
					if (personality & STILL_IDLE_BIT == 0)
					{
						//change direction at random or when on wall
						
						if (Random(2) == 0 || blob.isOnWall())
						{
							blob.setKeyPressed(blob.wasKeyPressed(key_right) ? key_left : key_right, true);
						}
						
						if (Random(2) == 0 || blob.isOnCeiling() || blob.isOnGround())
						{
							blob.setKeyPressed(blob.wasKeyPressed(key_down) ? key_down : key_down, true);
						}
					}
				}

			}

			blob.set_u8(state_property, mode);
		}
	}
	else
	{
		PressOldKeys( blob );
	}

	blob.set_u8(delay_property, delay);
}

//movement

void onInit( CMovement@ this )
{
	this.getCurrentScript().runFlags |= Script::tick_inwater;
	this.getCurrentScript().runFlags |= Script::tick_blob_in_proximity;
	this.getCurrentScript().runFlags |= Script::tick_not_attached;
	this.getCurrentScript().runProximityTag = "player";
	this.getCurrentScript().runProximityRadius = 120.0f;
	this.getCurrentScript().removeIfTag	= "dead";   
}

void onTick( CMovement@ this )
{
    CBlob@ blob = this.getBlob();
									  
    const f32 swimspeed = blob.get_f32("swimspeed");
    const f32 swimforce = blob.get_f32("swimforce");

	//if (inwater)	opt
	{
        Vec2f vel = blob.getVelocity();
        Vec2f waterForce;

        //up and down
        if (blob.isKeyPressed(key_up)
				&& vel.y > -swimspeed )
        {
            waterForce.y -= 1;
        }

        if (blob.isKeyPressed(key_down)
                && vel.y < swimspeed )
        {
            waterForce.y += 1;
        }

        //left and right
        if (blob.isKeyPressed(key_left)
                && vel.x > -swimspeed )
        {
            waterForce.x -= 1;
        }

        if (blob.isKeyPressed(key_right)
                && vel.x < swimspeed )
        {
            waterForce.x += 1;
        }

        waterForce *= swimforce * blob.getMass();
        blob.AddForce(waterForce);
    }
}
