// Pine tree Logic

#include "TreeSync.as"

void onInit( CBlob@ this )
{
    InitVars( this );
    this.setTeamNum(-1);
    TreeVars vars;
    vars.seed = Random(30000);
    vars.growth_time = 150 + Random( vars.seed, 30);
    vars.height = 0;
    vars.max_height = 5 + Random( vars.seed, 3 );
    vars.grown_times = 0;
    vars.max_grow_times = 30;
    vars.last_grew_time = getGameTime()-1; //pretend we started a frame ago ;)
    InitTree( this, vars );
    this.set("TreeVars", vars);
}

void GrowSprite( CSprite@ this, TreeVars@ vars )
{
    CBlob@ blob = this.getBlob();
	if (vars is null)
		return;

    if (vars.height == 0)
    {
        this.animation.frame = 1;
    }
    else if (vars.height == 1)
    {
        this.animation.frame = 2;
    }
    else
    {
        this.animation.frame = 3;
    }

    TreeSegment[]@ segments;
    blob.get("TreeSegments", @segments);
	if (segments is null)
		return;
    for (uint i = 0; i < segments.length; i++)
    {
        TreeSegment@ segment = segments[i];

        if (segment !is null && !segment.gotsprites)
        {
            segment.gotsprites = true;

            if (segment.grown_times == 1)
            {
                CSpriteLayer@ newsegment = this.addSpriteLayer( "segment "+i, "Entities/Natural/Trees/Trees.png" , 16, 16, 0, 0 );

                if (newsegment !is null)
                {
                    Animation@ animGrow = newsegment.addAnimation( "grow", 0, false );

                    if (i == 0)
                    {
                        animGrow.AddFrame(49);
                        animGrow.AddFrame(49);
                        animGrow.AddFrame(65);
                        animGrow.AddFrame(81);
                        animGrow.AddFrame(96);
                        animGrow.AddFrame(112);
                    }
                    else
                    {
                        animGrow.AddFrame(48);
                        animGrow.AddFrame(48);
                        animGrow.AddFrame(64);
                        animGrow.AddFrame(80);
                        animGrow.AddFrame(96);
                    }

                    newsegment.SetAnimation(animGrow);
                    newsegment.ResetTransform();
                    newsegment.SetRelativeZ( -0.1f - (vars.height / 100.0f) );
                    newsegment.RotateBy( segment.angle, Vec2f(0,0) );
                    newsegment.TranslateBy(segment.start_pos);
                }
            }
            else if (segment.grown_times == 2 && segment.height > 2)
            {
                CSpriteLayer@ newsegment = this.addSpriteLayer( "leaves "+i, "Entities/Natural/Trees/Trees.png" , 32, 32, 0, 0 );

                if (newsegment !is null)
                {
                    Animation@ animGrow = newsegment.addAnimation( "grow", 0, false );
                    animGrow.AddFrame(26);
                    animGrow.AddFrame(26);
                    animGrow.AddFrame(27);
                    animGrow.AddFrame(28);

                    if (Random(vars.seed, 2) == 0) {
                        animGrow.AddFrame(19);
                    }
                    else {
                        animGrow.AddFrame(20);
                    }

                    newsegment.SetAnimation(animGrow);
                    newsegment.ResetTransform( );
                    newsegment.SetRelativeZ( 0.1f + (vars.height / 100.0f) );
                    newsegment.RotateBy( segment.angle, Vec2f(0,0) );
                    newsegment.TranslateBy(segment.start_pos);
                }
            }
            else if (segment.grown_times == 5 && segment.height > 2 && segment.height <= vars.max_height )
            {
                f32 flipsign = 1.0f;
                CSpriteLayer@ newsegment = this.addSpriteLayer( "leaves side "+i, "Entities/Natural/Trees/Trees.png" , 32, 32, 0, 0 );

                if (newsegment !is null)
                {
                    Animation@ animGrow = newsegment.addAnimation( "grow", 0, false );
                    animGrow.AddFrame(18);
                    newsegment.SetAnimation(animGrow);
                    newsegment.ResetTransform( );
                    newsegment.SetRelativeZ( -70.0f - (vars.height / 100.0f) );

                    if (Random(vars.seed - i + 124587, 2) == 0) {
                        flipsign = -1.0f;
                    }

                    newsegment.RotateBy( segment.angle + Random(vars.seed, 10) * flipsign, Vec2f(16*flipsign,16) );
                    newsegment.ScaleBy(Vec2f(flipsign,1.0f));
                    newsegment.TranslateBy(segment.start_pos + Vec2f(-((vars.max_height - i) + Random(vars.seed + i + 1568, 8))*0.5 - 8 ,0).RotateBy(segment.angle,Vec2f(0,0)));
                }

                if (Random(vars.seed + i + 124, 2) == 0)
                {
                    CSpriteLayer@ secondnewsegment = this.addSpriteLayer( "leaves doubleside "+i, "Entities/Natural/Trees/Trees.png" , 32, 32, 0, 0 );

                    if (secondnewsegment !is null)
                    {
                        Animation@ animGrow = secondnewsegment.addAnimation( "grow", 0, false );
                        animGrow.AddFrame(18);
                        secondnewsegment.SetAnimation(animGrow);
                        secondnewsegment.ResetTransform( );
                        secondnewsegment.SetRelativeZ( -70.0f - (vars.height / 100.0f) );
                        flipsign *= -1.0f;
                        secondnewsegment.RotateBy( segment.angle + Random(vars.seed, 10) * flipsign, Vec2f(16*flipsign,16) );
                        secondnewsegment.ScaleBy(Vec2f(flipsign,1.0f));
                        secondnewsegment.TranslateBy(segment.start_pos + Vec2f(-((vars.max_height - i) + Random(vars.seed + i + 12124, 8))*0.5 - 8 ,0).RotateBy(segment.angle,Vec2f(0,0)));
                    }
                }
            }

            if (segment.grown_times <= 5)
            {
                CSpriteLayer@ segmentlayer = this.getSpriteLayer("segment "+i);

                if (segmentlayer !is null )
                {
                    segmentlayer.animation.SetFrameIndex(segmentlayer.animation.frame+1);
                }
            }

            if (segment.grown_times - 2 <= 5)
            {
                CSpriteLayer@ leaveslayer = this.getSpriteLayer("leaves "+i);

                if (leaveslayer !is null)
                {
                    leaveslayer.animation.SetFrameIndex(leaveslayer.animation.frame+1);
                }
            }
        }
    }
}
