#include "Vehicle.as"

// Mounted Bow logic

const Vec2f arm_offset = Vec2f(-6,0);

void onInit( CBlob@ this )
{
    Vehicle_Setup( this,
                   0.0f, // move speed
                   0.31f,  // turn speed
                   Vec2f(0.0f, 0.0f), // jump out velocity
                   false  // inventory access
                 );
    Vehicle_SetupWeapon( this,
                         15, // fire delay (ticks)
                         1, // fire bullets amount
                         Vec2f(-6.0f, 2.0f), // fire position offset
                         "mat_arrows", // bullet ammo config name
                         "arrow", // bullet config name
                         "BowFire", // fire sound
                         "EmptyFire" // empty fire sound
                       );
    this.set_u16("charge", 400 );
    //AddIconToken( "$"+this.getName()+"$", CFileMatcher("VehicleIcons.png").getFirst(), Vec2f(32,32), 1 ); // override icon
    // init arm + cage sprites
    CSprite@ sprite = this.getSprite();
    CSpriteLayer@ arm = sprite.addSpriteLayer( "arm", sprite.getConsts().filename, 16, 16 );

    if (arm !is null)
    {
        Animation@ anim = arm.addAnimation( "default", 0, false );
        anim.AddFrame(4);
        anim.AddFrame(5);
        arm.SetOffset( arm_offset );
    }

    CSpriteLayer@ cage = sprite.addSpriteLayer( "cage", sprite.getConsts().filename, 8, 16 );

    if (cage !is null)
    {
        Animation@ anim = cage.addAnimation( "default", 0, false );
        anim.AddFrame(1);
        anim.AddFrame(5);
        anim.AddFrame(7);
        cage.SetOffset( sprite.getOffset() );
        cage.SetRelativeZ(20.0f);
    }

    this.getShape().SetRotationsAllowed( false );
    AddIconToken( "$"+this.getName()+"$", CFileMatcher("MountedBow.png").getFirst(), Vec2f(16,16), 5 );   // override icon
	this.set_string("autograb blob", "mat_arrows");

	sprite.SetZ(-10.0f);

	this.getCurrentScript().runFlags |= Script::tick_hasattached;	

	// auto-load on creation
	if (getNet().isServer())
	{
		CBlob@ ammo = server_CreateBlob( "mat_arrows" );
		if (ammo !is null)	{
			if (!this.server_PutInInventory( ammo ))
				ammo.server_Die();
		}
	}
}

f32 getAimAngle( CBlob@ this )
{
    f32 angle = Vehicle_getWeaponAngle(this);
    bool facing_left = this.isFacingLeft();
    AttachmentPoint@ gunner = this.getAttachments().getAttachmentPointByName("GUNNER");
    bool failed = true;

    if (gunner !is null && gunner.getOccupied() !is null)
    {
        gunner.offsetZ = 5.0f;
        Vec2f aim_vec = gunner.getPosition() - gunner.getAimPos();

        if ( (!facing_left && aim_vec.x < 0) ||
                ( facing_left && aim_vec.x > 0 ) )
        {
            if (aim_vec.x > 0) { aim_vec.x = -aim_vec.x; }

            angle = (-(aim_vec).getAngle() + 180.0f);
            angle = Maths::Max( -80.0f , Maths::Min( angle , 80.0f ) );
        }
        else
        {
            this.SetFacingLeft(!facing_left);
        }
    }

    return angle;
}

void onTick( CBlob@ this )
{
    //set the arm angle based on GUNNER mouse aim, see above ^^^^
    f32 angle = getAimAngle( this );
    Vehicle_SetWeaponAngle( this, angle );
	CSprite@ sprite = this.getSprite();
    CSpriteLayer@ arm = sprite.getSpriteLayer( "arm" );

    if (arm !is null)
    {
        bool facing_left = sprite.isFacingLeft();
        f32 rotation = angle * (facing_left? -1: 1);

        if (this.get_u8("loaded ammo") > 0) {
            arm.animation.frame = 1;
        }
        else {
            arm.animation.frame = 0;
        }

        arm.ResetTransform( );
        arm.SetFacingLeft(facing_left);
        arm.SetRelativeZ( 1.0f );
        arm.SetOffset( arm_offset );
        arm.RotateBy( rotation, Vec2f(facing_left?-4.0f:4.0f,0.0f) );
    }


    Vehicle_Update( this );
    Vehicle_StandardControls( this );
}

void onHealthChange( CBlob@ this, f32 oldHealth )
{

	f32 hp = this.getHealth();
	f32 max_hp = this.getInitialHealth();
	int damframe = hp < max_hp * 0.4f ? 2 : hp < max_hp * 0.9f ? 1 : 0;
	CSprite@ sprite = this.getSprite();
	sprite.animation.frame = damframe;
	CSpriteLayer@ cage = sprite.getSpriteLayer( "cage" );  
	if (cage !is null)	{
		cage.animation.frame = damframe;
	}
}

void GetButtonsFor( CBlob@ this, CBlob@ caller )
{
    if (!Vehicle_AddFlipButton( this, caller))
    {
        Vehicle_AddLoadAmmoButton( this, caller );
    }
}


bool Vehicle_canFire( CBlob@ this, bool isActionPressed, bool wasActionPressed, u8 &out chargeValue ) {return false;}

void Vehicle_onFire( CBlob@ this, CBlob@ bullet, const u8 _unused )
{
    if (bullet !is null)
    {
		u16 charge = this.get_u16("charge");
        f32 angle = Vehicle_getWeaponAngle( this );
        angle = angle * (this.isFacingLeft() ? -1:1);
        angle += ((Random(512) - 256) / 64.0f);
        
        Vec2f vel = Vec2f(charge/21.5f * (this.isFacingLeft() ? -1 : 1), 0.0f).RotateBy(angle);
        bullet.setVelocity( vel );
        Vec2f offset = arm_offset;
        offset.RotateBy(angle);
        bullet.setPosition(this.getPosition() + offset + vel );
    }
}
