// Trampoline logic

namespace Trampoline
{
enum State
{
    folded = 0,
    idle,
    bounce,
    unpack
}

enum msg
{
    msg_pack = 0
}
}

void onInit( CBlob@ this )
{
    this.set_u8("trampolineState", Trampoline::folded);
    this.set_u32("trampolineBounceTime", 0);
    this.getShape().SetOffset( Vec2f(0.0f, 4.0f) );
    this.Tag("no falldamage");

    if (this.hasTag("start unpacked"))
    {
        this.set_u8("trampolineState", Trampoline::unpack);
    }

	this.getCurrentScript().tickFrequency = 2;
}

void onTick( CBlob@ this )
{
    if (this.get_u8("trampolineState") == Trampoline::bounce)
    {
        u32 bouncetime = getGameTime() - this.get_u32("trampolineBounceTime");

        if (bouncetime > 3) //10 ticks after bouncing
        {
            this.set_u8("trampolineState", Trampoline::idle);
        }
    }
}

void GetButtonsFor( CBlob@ this, CBlob@ caller )
{
    u8 state = this.get_u8("trampolineState");

    if (state == Trampoline::folded)
    {
        caller.CreateGenericButton( 6, Vec2f(0,-2), this, Trampoline::msg_pack, "Unpack Trampoline" );
    }
    else
    {
        if (!this.hasTag("static")) {
            caller.CreateGenericButton( 4, Vec2f(0,-2), this, Trampoline::msg_pack, "Pack up to move" );
        }
    }
}

void onCommand( CBlob@ this, u8 cmd, CBitStream @params )
{
    string dbg = "TrampolineLogic.as: Unknown command ";
    u8 state = this.get_u8("trampolineState");

    switch( cmd )
    {
    case Trampoline::msg_pack:
        if (state != Trampoline::folded)
        {
            this.set_u8("trampolineState", Trampoline::folded);
        }
        else
        {
            this.set_u8("trampolineState", Trampoline::unpack); //logic for completion of this this is in anim script
        }

        break;

    default:
        dbg += cmd;
        print( dbg );
        warn( dbg );
    }
}

bool canBePickedUp( CBlob@ this, CBlob@ byBlob )
{
    u8 state = this.get_u8("trampolineState");
    return (state == Trampoline::folded);
}

void onCollision( CBlob@ this, CBlob@ blob, bool solid, Vec2f normal, Vec2f point1 )
{
    if (blob is null) { // map collision?
        return;
    }

    Vec2f pos = this.getPosition();
    Vec2f blobpos = blob.getPosition();
    f32 horizDist = Maths::Abs( (blobpos.x -  pos.x) );
    f32 vely = blob.getVelocity().y;

    if (normal.y > 0.5f && horizDist < 12.0f && vely > 0.5f)
    {
        u8 state = this.get_u8("trampolineState");

        if (state == Trampoline::idle || state == Trampoline::bounce)
        {
            //f32 bounceForce = blob.isKeyPressed( key_jump ) ? 2.0f : 1.1f;
            f32 bounceForce = 2.0f;
            this.set_u8("trampolineState", Trampoline::bounce);
            this.set_u32("trampolineBounceTime", getGameTime() );
            Vec2f vel = blob.getVelocity();
            vel.Normalize();
            vel *= 5.0f;
            vel.y = - Maths::Min(vel.y * bounceForce, 10.0f);
            Vec2f up(0.0f,-1.0f);
            up.RotateBy( this.getAngleDegrees() );
            vel.x += up.x * bounceForce * 5.0f;
            blob.setVelocity(vel);
            f32 baseSound = 3.0f;

            if (vely > baseSound)
            {
                this.getSprite().PlaySound( CFileMatcher("/TrampolineJump").getRandom(), Maths::Min( Maths::Max( 0.4f, vely - baseSound ), 1.0f ), Maths::Min( Maths::Max( 0.5f, 1.0f - (baseSound - vely) ), 1.0f ) );
            }
        }
    }
}

