#include "Vehicle.as"

// Ballista logic

const string charge_time_prop = "ballista charge time";
const u8 charge_time_max = 80;

const string cooldown_time_prop = "ballista cooldown time";
const u8 cooldown_time = 20;

void onInit( CBlob@ this )
{
    Vehicle_Setup( this,
                   30.0f, // move speed
                   0.31f,  // turn speed
                   Vec2f(0.0f, 0.0f), // jump out velocity
                   false  // inventory access
                 );
    Vehicle_SetupWeapon( this,
                         cooldown_time, // fire delay (ticks)
                         1, // fire bullets amount
                         Vec2f(-6.0f, -8.0f), // fire position ffset
                         "mat_bolts", // bullet ammo config name
                         "ballista_bolt", // bullet config name
                         "CatapultFire", // fire sound
                         "EmptyFire", // empty fire sound
                         Vehicle_Fire_Style::custom
                       );
    Vehicle_SetupGroundSound( this, "WoodenWheelsRolling", // movement sound
                              1.0f, // movement sound volume modifier   0.0f = no manipulation
                              1.0f // movement sound pitch modifier     0.0f = no manipulation
                            );
                            
    Vehicle_addWoodenWheel( this, 0, Vec2f(9.0f,14.0f) );
    Vehicle_addWoodenWheel( this, 0, Vec2f(-1.0f,14.0f) );
    Vehicle_addWoodenWheel( this, 0, Vec2f(-11.0f,14.0f) );
    
    AddIconToken( "$"+this.getName()+"$", CFileMatcher("VehicleIcons.png").getFirst(), Vec2f(32,32), 1 );   // override icon
    
    this.getShape().SetOffset(Vec2f(0,10));
    
    // init arm sprites
    CSprite@ sprite = this.getSprite();
    CSpriteLayer@ arm = sprite.addSpriteLayer( "arm", sprite.getConsts().filename, 16, 32 );

    if (arm !is null)
    {
		f32 angle = getAngle(this, 0);
		
        Animation@ anim = arm.addAnimation( "default", 0, false );
        anim.AddFrame(6);
        anim.AddFrame(7);

		CSpriteLayer@ arm = this.getSprite().getSpriteLayer( "arm" );	
		if (arm !is null)
		{
			arm.SetRelativeZ( -0.1f );
			arm.RotateBy( angle, Vec2f(0.0f,-13.0f) );
			arm.SetOffset( Vec2f(8.0f,-8.0f) );
		}
    }
    
    this.set_u8(charge_time_prop, 0);
    Vehicle_SetWeaponAngle( this, 0 );
	this.set_string("autograb blob", "mat_bolts");

	// auto-load on creation
	if (getNet().isServer())
	{
		CBlob@ ammo = server_CreateBlob( "mat_bolts" );
		if (ammo !is null)	{
			if (!this.server_PutInInventory( ammo ))
				ammo.server_Die();
		}
	}
		   
}

f32 getAngle(CBlob@ this, const u8 charge)
{
	if (charge > 0)			   // don't allow rotation while charging
		return Vehicle_getWeaponAngle( this );

    f32 angle = 50.0f;
    bool facing_left = this.isFacingLeft();
    AttachmentPoint@ gunner = this.getAttachments().getAttachmentPointByName("GUNNER");
    bool failed = true;

    if (gunner !is null && gunner.getOccupied() !is null)
    {
        Vec2f aim_vec = gunner.getPosition() - gunner.getAimPos();

        if ( (!facing_left && aim_vec.x < 0) ||
                ( facing_left && aim_vec.x > 0 ) )
        {
            if (aim_vec.x > 0) { aim_vec.x = -aim_vec.x; }

            angle = (-(aim_vec).getAngle() + 270.0f);
            angle = Maths::Max( 50.0f , Maths::Min( angle , 90.0f ) );
        }
    }

    if (facing_left) { angle *= -1; }

    return angle;
}

void onTick( CBlob@ this )
{
	if (this.hasAttached())
	{
		u8 cool = this.get_u8(cooldown_time_prop);
		if (cool > 0)
		{
			cool--;
			this.set_u8(cooldown_time_prop, cool);
		}
    
		f32 angle = getAngle(this, this.get_u8(charge_time_prop));
		Vehicle_SetWeaponAngle( this, angle );
    
		CSpriteLayer@ arm = this.getSprite().getSpriteLayer( "arm" );	 
		if (arm !is null)
		{
			arm.ResetTransform( );
			f32 floattime = getGameTime();
			//arm.TranslateBy( Vec2f(10.0f * sign,-15.0f) );
			arm.SetRelativeZ( -0.1f );
			arm.RotateBy( angle, Vec2f(0.0f,-13.0f) );
			arm.SetOffset( Vec2f(8.0f,-8.0f) );

			if (this.get_u8("loaded ammo") > 0) {
				arm.animation.frame = 1;
			}
			else {
				arm.animation.frame = 0;
			}
		}

		Vehicle_StandardControls( this );
	}

	Vehicle_Update( this );
}

void GetButtonsFor( CBlob@ this, CBlob@ caller )
{
    if (!Vehicle_AddFlipButton( this, caller))
    {
        Vehicle_AddLoadAmmoButton( this, caller );
    }
}//

bool Vehicle_canFire( CBlob@ this , bool isActionPressed, bool wasActionPressed, u8 &out chargeValue )
{
	u8 charge = this.get_u8(charge_time_prop);
	
	if (charge > 0 || isActionPressed)
	{	
		if (charge < charge_time_max && isActionPressed)
		{
			charge++;
			this.set_u8(charge_time_prop,charge);

			u8 t = Maths::Round(float(charge_time_max)*0.66f);
			if ((charge < t && charge % 10 == 0) || (charge >= t && charge % 5 == 0))
				this.getSprite().PlaySound( "/LoadingTick" );

			chargeValue = charge;
			return false;
		}
		chargeValue = charge;
		return true;
	}
	
	return false;
}

void Vehicle_onFire( CBlob@ this, CBlob@ bullet, const u8 _charge )
{
    if (bullet !is null)
    {
		u8 charge_prop = _charge;
		
		f32 charge = 20.0f * (float(charge_prop) / float(charge_time_max));
		
        f32 angle = getAngle( this, _charge );
        Vec2f vel = Vec2f(0.0f, -charge).RotateBy(angle);
        bullet.setVelocity( vel );
        bullet.setPosition(bullet.getPosition() + vel );
    }
    
    this.set_u8(charge_time_prop,0);
    this.set_u8(cooldown_time_prop,cooldown_time);
}

bool doesCollideWithBlob( CBlob@ this, CBlob@ blob )
{
    return Vehicle_doesCollideWithBlob_ground( this, blob );
}


void onCollision( CBlob@ this, CBlob@ blob, bool solid )
{
	if (blob !is null) {
		TryToAttachVehicle( this, blob );
	}
}
