#include "Vehicle.as"

#include "Knocked.as";

// Catapult logic

const u8 baseline_charge = 15;

const string charge_time_prop = "cata charge time";
const u8 charge_time_max = 90;
const u8 charge_contrib = 35;

const u8 cooldown_time = 20;

void onInit( CBlob@ this )
{
    Vehicle_Setup( this,
                   30.0f, // move speed
                   0.31f,  // turn speed
                   Vec2f(0.0f, 0.0f), // jump out velocity
                   false  // inventory access
                 );
    Vehicle_SetupWeapon( this,
                         cooldown_time, // fire delay (ticks)
                         10, // fire bullets amount
                         getMagAttachmentPoint(this).offset, // fire position offset
                         "mat_stone", // bullet ammo config name
                         "cata_rock", // bullet config name
                         "CatapultFire", // fire sound
                         "CatapultFire", // empty fire sound
                         Vehicle_Fire_Style::custom
                       );
    Vehicle_SetupGroundSound( this, "WoodenWheelsRolling", // movement sound
                              1.0f, // movement sound volume modifier   0.0f = no manipulation
                              1.0f // movement sound pitch modifier     0.0f = no manipulation
                            );
    Vehicle_addWheel( this, "WoodenWheels.png", 16, 16, 1, Vec2f(-10.0f,11.0f) );
    Vehicle_addWheel( this, "WoodenWheels.png", 16, 16, 0, Vec2f(8.0f,10.0f) );
    
    AddIconToken( "$"+this.getName()+"$", CFileMatcher("VehicleIcons.png").getFirst(), Vec2f(32,32), 0 );   // override icon
    this.getShape().SetOffset(Vec2f(0,6));
    
	this.set_string("autograb blob", "mat_stone");

	// auto-load on creation
	if (getNet().isServer())
	{
		CBlob@ ammo = server_CreateBlob( "mat_stone" );
		if (ammo !is null)	{
			ammo.server_SetQuantity(100);
			if (!this.server_PutInInventory( ammo ))
				ammo.server_Die();
		}
	}
}

void onTick( CBlob@ this )
{
	if (this.hasAttached())
	{
		//set the arm angle based on how long ago we fired
		f32 delay = float(this.get_u16("fire delay"));	 
		f32 time_til_fire = Maths::Max((this.get_u32("fire time") - getGameTime()), 1);
		f32 rechargeRatio = (time_til_fire / delay);
		f32 angle = 360.0f * (1.0f-rechargeRatio);
		//Vehicle_SetWeaponAngle( this, angle );
		CSpriteLayer@ arm = this.getSprite().getSpriteLayer( "arm" );

		if (arm !is null)
		{
			f32 armAngle = 20 + (angle/9) + (float(this.get_u8(charge_time_prop))/float(charge_time_max))*10;
			arm.ResetTransform();
			f32 floattime = getGameTime();
			f32 sign = this.isFacingLeft() ? -1.0f : 1.0f;
			arm.SetRelativeZ( -10.5f );
			arm.RotateBy( armAngle * -sign, Vec2f(0.0f,-13.0f) );
			
			Vec2f armOffset = Vec2f(-10.0f,-10.0f);
			arm.SetOffset( armOffset );

			if (getMagBlob(this) is null && this.get_u8("loaded ammo") > 0) {
				arm.animation.frame = 1;
			}
			else {
				arm.animation.frame = 0;
			}

			// set the bowl attachment offset
			Vec2f offset(0.0f, 0.0f);
			offset.RotateBy( -armAngle, Vec2f(0.0f,13.0f) );
			
			offset += armOffset + Vec2f(14,-8);
			this.getAttachments().getAttachmentPointByName("MAG").offset = offset;
		}

		// load new item if present in inventory
		Vehicle_StandardControls( this );
	}

	Vehicle_Update( this );	 
}

void GetButtonsFor( CBlob@ this, CBlob@ caller )
{
    if (!Vehicle_AddFlipButton( this, caller))
    {
        Vehicle_AddLoadAmmoButton( this, caller );
    }
}

bool Vehicle_canFire( CBlob@ this , bool isActionPressed, bool wasActionPressed, u8 &out chargeValue )
{
	u8 charge = this.get_u8(charge_time_prop);
	
	if (charge > 0 || isActionPressed)
	{	
		if (charge < charge_time_max && isActionPressed)
		{
			charge++;
			this.set_u8(charge_time_prop,charge);

			u8 t = Maths::Round(float(charge_time_max)*0.66f);
			if ((charge < t && charge % 10 == 0) || (charge >= t && charge % 5 == 0))
				this.getSprite().PlaySound( "/LoadingTick" );

			chargeValue = charge;
			return false;
		}
		chargeValue = charge;
		if (charge < baseline_charge)
			return false;

		return true;
	}
	
	return false;
}

void Vehicle_onFire( CBlob@ this, CBlob@ bullet, const u8 _charge )
{
    f32 charge = baseline_charge + (float(_charge)/float(charge_time_max))*charge_contrib;

    if (bullet !is null)
    {
        f32 angle = this.getAngleDegrees();
        f32 sign = this.isFacingLeft() ? -1.0f : 1.0f;
        
        Vec2f vel = Vec2f(sign, -0.5f) * charge * 0.3f;
        vel += (Vec2f(Random(128) - 64, Random(128) - 64) * 0.01f);
        
        bullet.setVelocity( vel );
        
        if (isKnockable( bullet ))
        {
			SetKnocked( bullet, 30 );
		}
    }

    // we override the default time because we want to base it on charge
    int delay = 30 + (charge / (250/30));
    this.set_u16("fire delay", delay );
    
	this.set_u8(charge_time_prop,0);
}

bool doesCollideWithBlob( CBlob@ this, CBlob@ blob )
{
    return Vehicle_doesCollideWithBlob_ground( this, blob );
}


void onCollision( CBlob@ this, CBlob@ blob, bool solid )
{
	if (blob !is null) {
		TryToAttachVehicle( this, blob );
	}
}
