// Simple chat processing example.
// If the player sends a command, the server does what the command says.
// You can also modify the chat message before it is sent to clients by modifying text_out.

#include "MakeSeed.as";
#include "MakeCrate.as";
#include "MakeScroll.as";

bool onServerProcessChat( CRules@ this, const string& in text_in, string& out text_out, CPlayer@ player )
{
    if (text_in == "!bot") // player is admin...
    {
        CPlayer@ bot = AddBot( "Henry" );
        return true;
    }

    //spawning things
    CBlob@ blob = player.getBlob();

    if (blob is null) {
        return true;
    }

    if (text_in == "!tree" && sv_test)
    {
        server_MakeSeed( blob.getPosition(), "tree_pine", 600, 1, 16 );
    }
    else if (text_in == "!btree" && sv_test)
    {
        server_MakeSeed( blob.getPosition(), "tree_bushy", 400, 2, 16 );
    }
    else if (text_in == "!flowers" && sv_test)
    {
        server_CreateBlob( "Entities/Natural/Flowers/Flowers.cfg", blob.getTeamNum(), blob.getPosition() );
    }
    else if (text_in == "!catapult" && sv_test)
    {
        server_CreateBlob( "Entities/Vehicles/Catapult/Catapult.cfg", blob.getTeamNum(), blob.getPosition() );
    }
    else if (text_in == "!stones" && sv_test)
    {
        CBlob@ b = server_CreateBlob( "Entities/Materials/MaterialStone.cfg", blob.getTeamNum(), blob.getPosition() );

        if (b !is null) {
            b.server_SetQuantity(320);
        }
    }
    else if (text_in == "!arrows" && sv_test)
    {
        for (int i = 0; i < 3; i++)
        {
            CBlob@ b = server_CreateBlob( "Entities/Materials/MaterialArrows.cfg", blob.getTeamNum(), blob.getPosition() );

            if (b !is null) {
                b.server_SetQuantity(30);
            }
        }
    }
    else if (text_in == "!bombs" && sv_test)
    {
        //  for (int i = 0; i < 3; i++)
        CBlob@ b = server_CreateBlob( "Entities/Materials/MaterialBombs.cfg", blob.getTeamNum(), blob.getPosition() );

        if (b !is null) {
            b.server_SetQuantity(30);
        }
    }
    else if (text_in == "!spawnwater" && sv_test)
    {
        getMap().server_setFloodWaterWorldspace(blob.getPosition(),true);
    }
    else if (text_in == "!killall" && sv_test)
    {
        CBlob@[] players;
        getBlobsByTag( "player", @players );

        for (u32 i=0; i < players.length; i++)
        {
            players[i].server_Die();
        }
    }
    else if (text_in == "!kill1" && sv_test)
    {
        CBlob@[] players;
        getBlobsByTag( "player", @players );

        for (u32 i=0; i < players.length; i++)
        {
            CBlob@ blob = players[i];

            if (blob.getTeamNum() == 0) {
                blob.server_Die();
            }
        }
    }
    else if (text_in == "!kill2" && sv_test)
    {
        CBlob@[] players;
        getBlobsByTag( "player", @players );

        for (u32 i=0; i < players.length; i++)
        {
            CBlob@ blob = players[i];

            if (blob.getTeamNum() == 1) {
                blob.server_Die();
            }
        }
    }
    else if (text_in == "!killme")
    {
        blob.server_Hit( blob, blob.getPosition(), Vec2f(0,0), 3.5f, 0);
    }
    else if (text_in == "!crate" && sv_test)
    {
        client_AddToChat( "usage: !crate BLOBNAME [DESCRIPTION]", SColor(255, 255, 0,0));
        server_MakeCrate( "", "", 0, blob.getTeamNum(), Vec2f( blob.getPosition().x, blob.getPosition().y - 30.0f ) );
    }
    else if (text_in == "!debug")
    {
        // print all blobs
        CBlob@[] all;
        getBlobs( @all );

        for (u32 i=0; i < all.length; i++)
        {
            CBlob@ blob = all[i];
            print("["+blob.getName()+" " + blob.getNetworkID() + "] ");            
        }
    }
    else if (text_in.substr(0,1) == "!")
    {
        // check if we have tokens
        string[]@ tokens = text_in.split(" ");

        if (tokens.length > 1)
        {
            if (tokens[0] == "!crate" && sv_test)
            {
                int frame = tokens[1] == "catapult" ? 1 : 0;
                string description = tokens.length > 2 ? tokens[2] : tokens[1];
                server_MakeCrate( tokens[1], description, frame, -1, Vec2f( blob.getPosition().x, blob.getPosition().y ) );
            }
            else if (tokens[0] == "!team" && sv_test)
            {
                int team = parseInt(tokens[1]);
                blob.setTeamNum(team);
            }
			/*else if (tokens[0] == "!scroll" && sv_test)
			{
				string description = tokens.length > 2 ? tokens[2] : tokens[1];
				server_MakeScroll( blob.getPosition(), tokens[1], tokens[1] );
			}*/

            return true;
        }

        // try to spawn an actor with this name !actor
        string name = text_in.substr(1, text_in.size());

        if (sv_test && server_CreateBlob( name, blob.getTeamNum(), blob.getPosition() ) is null) {
            client_AddToChat( "blob " + text_in + " not found", SColor(255, 255, 0,0));
        }
    }

    return true;
}

bool onClientProcessChat( CRules@ this, const string& in text_in, string& out text_out, CPlayer@ player )
{
    if (text_in == "!debug" && !getNet().isServer())
    {
        // print all blobs
        CBlob@[] all;
        getBlobs( @all );

        for (u32 i=0; i < all.length; i++)
        {
            CBlob@ blob = all[i];
            print("["+blob.getName()+" " + blob.getNetworkID() + "] ");

            if (blob.getShape() !is null) {
                print( "     gravity scale = " + blob.getShape().getGravityScale() );
            }
        }
    }

    return true;
}
