// SHOW KILL MESSAGES ON CLIENT

#include "Hitters.as";
#include "TeamColour.as";

int fade_time = 300;


class KillMessage
{
    string victim;
    string attacker;
    int attackerteam;
    int victimteam;
    u8 hitter;
    s16 time;

    KillMessage( ) {} //dont use this

    KillMessage( CPlayer@ _victim, CPlayer@ _attacker, u8 _hitter )
    {
        victim = _victim.getCharacterName();
        victimteam = _victim.getTeamNum();

        if (_attacker !is null)
        {
            attacker = _attacker.getCharacterName();
            attackerteam = _attacker.getTeamNum();
        }
        else
        {
            attacker = "";
            attackerteam = -1;
        }

        hitter = _hitter;
        time = fade_time;
    }
}

class KillFeed
{
    KillMessage[] killMessages;

    void Update()
    {
        while(killMessages.length > 10)
        {
            killMessages.erase(0);
        }

        for (uint message_step = 0; message_step < killMessages.length; ++message_step)
        {
            KillMessage@ message = killMessages[message_step];
            message.time--;

            if (message.time <= 0) //remove
            {
                int pos = killMessages.find(message);

                if (pos >= 0) { killMessages.erase(pos); }
            }
        }
    }

    void Render()
    {
        for (uint message_step = 0; message_step < killMessages.length; ++message_step)
        {
            KillMessage@ message = killMessages[message_step];
            Vec2f dim, ul, lr;

            if (message.attackerteam != -1)
            {
                //draw attacker name

                GUI::GetTextDimensions( message.attacker, dim );
                ul.Set( getScreenWidth() - dim.x - 164, (message_step+1) * 16 );
                lr.Set( ul.x + dim.x, ul.y + dim.y );
                GUI::DrawText( message.attacker, ul, lr, getTeamColor( message.attackerteam ), true, true);
            }

            //draw icon in between based on hitter
            string hitterIcon;

            switch(message.hitter)
            {
            case Hitters::fall:     hitterIcon = "$killfeed_fall$"; break;

            case Hitters::stomp:    hitterIcon = "$killfeed_stomp$"; break;

            case Hitters::builder:  hitterIcon = "$killfeed_builder$"; break;

            case Hitters::sword:    hitterIcon = "$killfeed_sword$"; break;

            case Hitters::shield:   hitterIcon = "$killfeed_shield$"; break;

            case Hitters::bomb:     hitterIcon = "$killfeed_bomb$"; break;

            case Hitters::arrow:    hitterIcon = "$killfeed_arrow$"; break;

            case Hitters::ballista: hitterIcon = "$killfeed_ballista$"; break;

            default: hitterIcon = "";
            }

            if (hitterIcon != "")
            {
                ul.Set( getScreenWidth() - 172, ((message_step+1) * 16) - 8 );
                GUI::DrawIconByName( hitterIcon, ul);
            }

				//draw victim name
			{
				GUI::GetTextDimensions( message.victim, dim );
				ul.Set( getScreenWidth() - 124, (message_step+1) * 16 );
				lr.Set( ul.x + dim.x, ul.y + dim.y );
				GUI::DrawText( message.victim, ul, lr, getTeamColor(message.victimteam), true, true);
			}
        }
    }

}


void onInit( CRules@ this )
{
    AddIconToken( "$killfeed_fall$", "GUI/KillfeedIcons.png", Vec2f(32,16), 1 );
    AddIconToken( "$killfeed_stomp$", "GUI/KillfeedIcons.png", Vec2f(32,16), 4 );
    AddIconToken( "$killfeed_pick$", "GUI/KillfeedIcons.png", Vec2f(32,16), 8 );
    AddIconToken( "$killfeed_axe$", "GUI/KillfeedIcons.png", Vec2f(32,16), 9 );
    AddIconToken( "$killfeed_sword$", "GUI/KillfeedIcons.png", Vec2f(32,16), 12 );
    AddIconToken( "$killfeed_shield$", "GUI/KillfeedIcons.png", Vec2f(32,16), 13 );
    AddIconToken( "$killfeed_bomb$", "GUI/KillfeedIcons.png", Vec2f(32,16), 14 );
    AddIconToken( "$killfeed_arrow$", "GUI/KillfeedIcons.png", Vec2f(32,16), 16 );
    AddIconToken( "$killfeed_ballista$", "GUI/KillfeedIcons.png", Vec2f(32,16), 17 );
}

void onRestart( CRules@ this )
{
    KillFeed feed;
    this.set( "KillFeed", feed );
}

void onPlayerDie( CRules@ this, CPlayer@ victim, CPlayer@ killer, u8 customdata )
{
    if (victim !is null)
    {
        KillFeed@ feed;

        if (this.get( "KillFeed", @feed ))
        {
            KillMessage message = KillMessage(victim, killer, customdata);
            feed.killMessages.push_back(message);
        }
    }
}

void onTick( CRules@ this )
{
    KillFeed@ feed;

    if (this.get( "KillFeed", @feed ))
    {
        feed.Update();
    }
}

void onRender( CRules@ this )
{
    KillFeed@ feed;

    if (this.get( "KillFeed", @feed ))
    {
        feed.Render();
    }
}
