
//The core class of rules

// extend this to provide your own functionality
// and see the ExampleRules, or TDM and WAR scripts for how to do it so it works :)

// defines for debugging:
// DEBUG_PLAYERS
// DEBUG_TEAMS

#include "BaseTeamInfo.as";
#include "PlayerInfo.as";

#include "RespawnSystem.as";

shared class RulesCore
{

    CRules@ rules;

    PlayerInfo@[] players;
    BaseTeamInfo@[] teams;

    RespawnSystem@ respawns;

    //default
    RulesCore() { error("DO NOT DO THIS! INITIALISE WITH RULESCORE(RULES,RESPAWNS)"); }

    //intended constructor, doesn't yell at you
    RulesCore(CRules@ _rules, RespawnSystem@ _respawns ) { Setup(_rules, _respawns); }

    void Setup(CRules@ _rules = null, RespawnSystem@ _respawns = null)
    {
        @rules = _rules;
        @respawns = _respawns;

        if (respawns !is null) {
            respawns.SetCore(this);
        }

        SetupTeams();
        SetupPlayers();
        AddAllPlayersToSpawn();
    }

    void Update()
    {
        if (respawns !is null) {
            respawns.Update();
        }
    }

    //working with teams

    void SetupTeams()
    {
        teams.clear();

        for (int team_step = 0; team_step < rules.getTeamsCount(); ++team_step)
        {
            AddTeam(rules.getTeam(team_step));
        }
    }

    void AddTeam(CTeam@ team)
    {
        BaseTeamInfo t(teams.length, team.getName());
        teams.push_back(@t);
    }

    BaseTeamInfo@ getTeam(int teamNum)
    {
        if (teamNum < 0 || teamNum >= int(teams.length)) {
            return null;
        }

        return teams[teamNum];
    }

    void ChangeTeamPlayerCount(int teamNum, int amountBy)
    {
        BaseTeamInfo@ t = getTeam(teamNum);

        if (t !is null) { t.players_count += amountBy; }
    }

    //working with players

    void SetupPlayers()
    {
        players.clear();

        for (int player_step = 0; player_step < getPlayersCount(); ++player_step)
        {
            AddPlayer(getPlayer(player_step));
        }
    }

    PlayerInfo@ getInfoFromName( string username )
    {
        for (uint k = 0; k < players.length; k++)
        {
            if (players[k].username == username) {
                return players[k];
            }
        }

        return null;
    }

    PlayerInfo@ getInfoFromPlayer( CPlayer@ player )
    {
        if (player !is null) {
            return getInfoFromName( player.getUsername() );
        }
        else {
            return null;
        }
    }

    void AddPlayer(CPlayer@ player, u8 team = 0, string default_config = "")
    {
        PlayerInfo@ check = getInfoFromName( player.getUsername() );

        if (check is null)
        {
            PlayerInfo p(player.getUsername(), team, default_config);
            players.push_back(@p);
            ChangeTeamPlayerCount(p.team, 1);
        }
    }

    void RemovePlayer(CPlayer@ player)
    {
        PlayerInfo@ p = getInfoFromName( player.getUsername() );

        if (p !is null)
        {
            if (respawns !is null) //remove any spawns
            {
                respawns.RemovePlayerFromSpawn(player);
            }

            int pos = players.find(p);

            if (pos != -1) {
                players.erase(pos);
            }

            ChangeTeamPlayerCount(p.team, -1);
        }
    }

    void ChangePlayerTeam(CPlayer@ player, int newTeamNum)
    {
        PlayerInfo@ p = getInfoFromName( player.getUsername() );

        if (p.team != newTeamNum)
        {
            ChangeTeamPlayerCount(p.team, -1);
            ChangeTeamPlayerCount(newTeamNum, 1);
            p.setTeam(newTeamNum);
        }

        if (player.getTeamNum() != s32(p.team) ) {
            player.server_setTeamNum(p.team); //TODO: HAVE SERVER FORCE THIS
        }
    }	

    void AddPlayerSpawn(CPlayer@ player)
    {
		// kill old player
		CBlob@ blob = player.getBlob();
		if (blob !is null) {
			if (blob.getHealth() > 0.0f)
				blob.server_Hit( blob, blob.getPosition(), Vec2f(0,0), blob.getHealth(), 0, true);
		}
		
        PlayerInfo@ p = getInfoFromName( player.getUsername() );

        if (p is null)
        {
            AddPlayer(player);
        }

        if (player.lastBlobName.length() > 0 && p !is null)
        {
            p.blob_name = filterBlobNameToSpawn(player.lastBlobName, player);
        }

        if (respawns !is null)
        {
            respawns.RemovePlayerFromSpawn(player);
            respawns.AddPlayerToSpawn(player);
        }
    }

    /*
     * Overload this to apply some filtering to what blobs
     * players are allowed to respawn as.
     */
    string filterBlobNameToSpawn(string proposed, CPlayer@ player)
    {
        return proposed;
    }

    void AddAllPlayersToSpawn()
    {
		uint len = players.length;
		uint salt = getGameTime() + len;
        for (uint k = 0; k < len; k++)
        {
            AddPlayerSpawn(getPlayerByUsername(players[( (k+salt) * 997) % len].username));
        }
    }



};

