// Rules script to drop trader if team is lacking one

#define SERVER_ONLY

#include "MakeCrate.as";
#include "TradingCommon.as";

const string spawnBlobName = "war_base";
const bool paraSpawn = true;

void onTick( CRules@ this )
{
    if (!this.isMatchRunning())       
      return;


    if (getGameTime() % 303 == 0) // we don't have to do this each tick
    {
        CBlob@[] traders;
        GatherTraders( traders );

        // there's not enough traders
        uint tradersCount = traders.length;

		// decrease dead ones
		for (uint i=0; i < traders.length; i++)
		{
			if (isTraderConscious(traders[i]))
				tradersCount--;
		}

	    TeamSpawnTrader( 0, traders );
		TeamSpawnTrader( 1, traders );
    }
}

void TeamSpawnTrader( int teamNum, CBlob@[] traders )
{
	uint teamTraders = 0;
	for (uint i=0; i < traders.length; i++)
	{
		if (traders[i].getTeamNum() == teamNum)	{
			teamTraders++;
		}
	}

	if (teamTraders == 0)
	{
		CBlob@[] spawns;
		getBlobsByName( "tradingpost", @spawns );
		getBlobsByName( "builder", @spawns );
		for (uint i=0; i < spawns.length; i++)
		{
			CBlob@ post = spawns[i];
			if (post.getTeamNum() == teamNum)
			{
				SpawnTrader( post.getPosition(), teamNum, paraSpawn );
				return;
			}
		}  
	}
}

bool isTraderConscious( CBlob@ trader )
{
	if (trader.isInInventory())
	{
		if (trader.getInventoryBlob().hasTag("parachute"))
			return true;
	}
	else
		false;
	return (!trader.hasTag("dead"));	  //cant put in inv
}

CBlob@ SpawnTrader( Vec2f position, int teamNum,  bool parachute )
{
    // spawn in crate/parachute
    Vec2f dropPos = position;
	//printf("spawning trader");

    if (parachute) {
        dropPos = getDropPosition( position );
    }
    else
    {
        CBlob@ baseSpawn;
        CBlob@[] bases;
        getBlobsByName( spawnBlobName, @bases );

        for (uint i=0; i < bases.length; i++)
        {
            CBlob@ base = bases[i];

            if (base.getTeamNum() == teamNum)
            {
                @baseSpawn = base;
                break;
            }
        }

        if (baseSpawn !is null) {
            dropPos = baseSpawn.getPosition() + Vec2f( baseSpawn.isFacingLeft() ? 32.0f : -32.0f, 0.0f);
        }
    }

    CBlob@ trader = server_CreateBlob( "trader", teamNum, dropPos );

    if (trader !is null)
    {
        trader.setSexNum( Random(2) ); // sex change
    }

    if (parachute)
    {
        CBlob@ crate = server_MakeCrateOnParachute( "", "", 2, teamNum, dropPos );
        crate.server_PutInInventory( trader );
        crate.Tag("destroy on touch");
    }

    return trader;
}

void GatherTraders(  CBlob@[]@ traders )
{
	getBlobsByName( "trader", @traders );
	// crate ones
	CBlob@[] crates;
	getBlobsByName( "crate", @crates );
	for (uint i=0; i < crates.length; i++)
	{
		CBlob@ crate = crates[i];
		if (crate.getInventory() !is null)
		{
			CBlob@ trader = crate.getInventory().getItem("trader");
			if (trader !is null) {
				traders.push_back( trader );
			}
		}
	}  
}
