
//TDM gamemode logic script

#define SERVER_ONLY

#include "TDM_Structs.as";
#include "RulesCore.as";
#include "RespawnSystem.as";

//simple config function - edit the variables below to change the basics

void Config(TDMCore@ this)
{
    //how long to wait for everyone to spawn in?
    s32 warmUpTimeSeconds = 5;
    this.warmUpTime = (getTicksASecond() * warmUpTimeSeconds);
    this.gametime = getGameTime() + this.warmUpTime;
    //how many kills needed to win the match, per player on the smallest team
    this.kills_to_win_per_player = 2;
    //how long for the game to play out?
    s32 gameDurationMinutes = 3;
    this.gameDuration = (getTicksASecond() * 60 * gameDurationMinutes) + this.warmUpTime;
    //spawn after death time - set in gamemode.cfg, or override here
    f32 spawnTimeSeconds = this.rules.playerrespawn_seconds;
    this.spawnTime = (getTicksASecond() * spawnTimeSeconds);
    //how many players have to be in for the game to start
    this.minimum_players_in_team = 1;
    //whether to scramble each game or not
    this.scramble_teams = true;
    // modifies if the fall damage velocity is higher or lower - TDM has lower velocity
    this.rules.set_f32("fall vel modifier", 1.3f);
}

//TDM spawn system

shared class TDMSpawns : RespawnSystem
{
    TDMCore@ TDM_core;

    bool force;

	void SetCore(RulesCore@ _core)
	{
		RespawnSystem::SetCore(_core);
		@TDM_core = cast<TDMCore@>(core);
	}

    void Update()
    {
        for (uint team_num = 0; team_num < TDM_core.teams.length; ++team_num )
        {
            TDMTeamInfo@ team = cast<TDMTeamInfo@>( TDM_core.teams[team_num] );

            for (uint i = 0; i < team.spawns.length; i++)
            {
                TDMPlayerInfo@ info = cast<TDMPlayerInfo@>(team.spawns[i]);
                
                UpdateSpawnTime(info, i);
                
                info.thrownBomb = false;
                DoSpawnPlayer( info );
            }
        }
    }
    
    void UpdateSpawnTime(TDMPlayerInfo@ info, int i)
    {
		//default
		u8 spawn_property = 254;
		
		if ( i == 0 && info !is null && info.can_spawn_time > 0) {
			info.can_spawn_time--;
			spawn_property = u8(Maths::Min(250,(info.can_spawn_time / 30)));
		}

		string propname = "tdm spawn time "+info.username;
		TDM_core.rules.set_u8( propname, spawn_property );
		if (info !is null && info.can_spawn_time >= 0)
		{
			TDM_core.rules.SyncToPlayer( propname, getPlayerByUsername(info.username) );
		}
	}

    void DoSpawnPlayer( PlayerInfo@ p_info )
    {
        if (force || canSpawnPlayer(p_info))
        {
            CPlayer@ player = getPlayerByUsername(p_info.username); // is still connected?

            if (player is null) {
				RemovePlayerFromSpawn(p_info);
                return;
            }
            if (player.getTeamNum() != p_info.team)
            {
				player.server_setTeamNum(p_info.team);
			}

			// remove previous players blob	  			
			if (player.getBlob() !is null)
			{
				CBlob @blob = player.getBlob();
				blob.server_SetPlayer( null );
				blob.server_Die();					
			}

            CBlob@ playerBlob = SpawnPlayerIntoWorld( getSpawnLocation(p_info), p_info);

            if (playerBlob !is null)
            {
                // spawn resources
                p_info.spawnsCount++;
                RemovePlayerFromSpawn(player);
            }
        }
    }

    bool canSpawnPlayer(PlayerInfo@ p_info)
    {
        TDMPlayerInfo@ info = cast<TDMPlayerInfo@>(p_info);

        if (info is null) { warn("TDM LOGIC: Couldn't get player info ( in bool canSpawnPlayer(PlayerInfo@ p_info) ) "); return false; }

        if (force) { return true; }

        return info.can_spawn_time <= 0;
    }

    Vec2f getSpawnLocation(PlayerInfo@ p_info)
    {
        CBlob@[] spawns;

        if (getBlobsByName( "tdm_spawn", @spawns ))
        {
            for (uint step = 0; step < spawns.length; ++step)
            {
                if (spawns[step].getTeamNum() == s32(p_info.team) ) {
                    return spawns[step].getPosition();
                }
            }
        }

        return Vec2f(0,0);
    }

    void RemovePlayerFromSpawn(CPlayer@ player)
    {
        RemovePlayerFromSpawn(core.getInfoFromPlayer(player));
    }
    
    void RemovePlayerFromSpawn(PlayerInfo@ p_info)
    {
        TDMPlayerInfo@ info = cast<TDMPlayerInfo@>(p_info);
        
        if (info is null) { warn("TDM LOGIC: Couldn't get player info ( in void RemovePlayerFromSpawn(PlayerInfo@ p_info) )"); return; }

        string propname = "tdm spawn time "+info.username;
        TDMTeamInfo@ team = cast<TDMTeamInfo@>(TDM_core.teams[info.team]);
        info.can_spawn_time = 0;
        int pos = team.spawns.find(info);

        if (pos != -1) {
            team.spawns.erase(pos);
            TDM_core.rules.set_u8( propname, 255 ); //not respawning
            TDM_core.rules.SyncToPlayer( propname, getPlayerByUsername(info.username) ); 
            // IF WE DONT DO THIS HERE WE GET THE CANT FIND PROP ON JOIN BUG + CAUSES PLAYER TO GLUE TO SPACE ON MAP RESTART
        }
	}

    void AddPlayerToSpawn( CPlayer@ player )
    {
		if (player.getTeamNum() == core.rules.getSpectatorTeamNum())
		{
			RemovePlayerFromSpawn(player);
			//TODO set player info
			return;
		}
		
        s32 tickspawndelay = s32(TDM_core.spawnTime);
        print("ADD SPAWN FOR " + player.getUsername());
        TDMPlayerInfo@ info = cast<TDMPlayerInfo@>(core.getInfoFromPlayer(player));

        if (info is null) { warn("TDM LOGIC: Couldn't get player info  ( in void AddPlayerToSpawn(CPlayer@ player) )"); return; }

        TDMTeamInfo@ team = cast<TDMTeamInfo@>(TDM_core.teams[info.team]);
        RemovePlayerFromSpawn(player);
        info.can_spawn_time = tickspawndelay;
        
        team.spawns.push_back(info);
    }

};

shared class TDMCore : RulesCore
{
    s32 warmUpTime;
    s32 gameDuration;
    s32 spawnTime;
    s32 minimum_players_in_team;
    s32 kills_to_win;
    s32 kills_to_win_per_player;

    s32 players_in_small_team;
    bool scramble_teams;

    TDMSpawns@ tdm_spawns;

    TDMCore() {}

    TDMCore(CRules@ _rules, RespawnSystem@ _respawns )
    {
        super(_rules, _respawns );
    }

    void Setup(CRules@ _rules = null, RespawnSystem@ _respawns = null)
    {
        RulesCore::Setup(_rules, _respawns);
        gametime = getGameTime();
        @tdm_spawns = cast<TDMSpawns@>(_respawns);
     //   server_CreateBlob( "Entities/Meta/TDMMusic.cfg" );
        players_in_small_team = -1;
    }

    int gametime;
    void Update()
    {			
        //HUD
        //dirty hack time...
        // lets save the CPU and do this only once in a while
        if (getGameTime() % 16 == 0)
        {
            updateHUD();
        }

        if (rules.isGameOver()) { return; }

        s32 ticksToStart = gametime - getGameTime();
        tdm_spawns.force = false;

        if (ticksToStart <= 0 && (rules.isWarmup()))
        {
            rules.SetCurrentState(GAME);
        }
        else if (ticksToStart > 0 && rules.isWarmup()) //is the start of the game, spawn everyone + give mats
        {
            rules.SetGlobalMessage( "Match starts in "+((ticksToStart/30)+1) );
            tdm_spawns.force = true;

            //set kills and cache #players in smaller team

            if (players_in_small_team == -1 || (getGameTime() % 30) == 4)
            {
                players_in_small_team = 100;

                for (uint team_num = 0; team_num < teams.length; ++team_num )
                {
                    TDMTeamInfo@ team = cast<TDMTeamInfo@>( teams[team_num] );

                    if (team.players_count < players_in_small_team)
                    {
                        players_in_small_team = team.players_count;
                    }
                }

                kills_to_win = players_in_small_team * kills_to_win_per_player;
            }
        }

        if ((rules.isIntermission() || rules.isWarmup()) && (!allTeamsHavePlayers()) ) //CHECK IF TEAMS HAVE ENOUGH PLAYERS
        {
            gametime = getGameTime() + warmUpTime;
            rules.set("game_end_time", gametime + gameDuration);
            rules.SetGlobalMessage( "Not enough players in each team for the game to start.\nPlease wait for someone to join..." );
            tdm_spawns.force = true;
        }
        else if (rules.isMatchRunning())
        {
            rules.SetGlobalMessage( "" );

            if (getGameTime() % 11 == 0) {
                CheckKnightThrownBomb();
            }
        }

      //  SpawnPowerups();
        RulesCore::Update(); //update respawns
        CheckTeamWon();

		if (getGameTime() % 1000 == 0)
			SpawnBombs();
    }

	void updateHUD()
	{
		bool hidekills = (rules.isIntermission() || rules.isWarmup());
		CBitStream serialised_team_hud;
		serialised_team_hud.write_u16(0x5afe); //check bits

		for (uint team_num = 0; team_num < teams.length; ++team_num )
		{
			TDM_HUD hud;
			TDMTeamInfo@ team = cast<TDMTeamInfo@>( teams[team_num] );
			hud.team_num = team_num;
			hud.kills = team.kills;
			hud.kills_limit = (hidekills ? -1 : kills_to_win);
			string temp = "";

			for ( uint player_num = 0; player_num < players.length; ++player_num )
			{
				TDMPlayerInfo@ player = cast<TDMPlayerInfo@>( players[player_num] );

				if (player.team == team_num)
				{
					CPlayer@ e_player = getPlayerByUsername(player.username);

					if (e_player !is null)
					{
						CBlob@ player_blob = e_player.getBlob();
						bool blob_alive = player_blob !is null && player_blob.getHealth() > 0.0f;

						if (blob_alive)
						{
							string player_char = "k"; //default to sword

							if (player_blob.getName() == "archer") {
								player_char = "a";
							}

							temp += player_char;
						}
						else
						{
							temp += "s";
						}
					}
				}
			}

			hud.unit_pattern = temp;

			if (team.spawns.length > 0 && !rules.isIntermission()) {
				hud.spawn_time = (cast<TDMPlayerInfo@>(team.spawns[0]).can_spawn_time / 30);
			}
			else {
				hud.spawn_time = 255;
			}

			hud.Serialise(serialised_team_hud);
		}

		rules.set_CBitStream("tdm_serialised_team_hud",serialised_team_hud);
		rules.Sync("tdm_serialised_team_hud",true);
	}

    //HELPERS

    int getSmallerTeam()
    {
        s32 lowestTeam = -1;
        s32 lowestCount = 10000;

        for (uint i = 0; i < teams.length; i++)
        {
            if (teams[i].players_count < lowestCount)
            {
                lowestCount = teams[i].players_count;
                lowestTeam = i;
            }
        }

        return lowestTeam;
    }

    bool allTeamsHavePlayers()
    {
        for (uint i = 0; i < teams.length; i++)
        {
            if (teams[i].players_count < minimum_players_in_team)
            {
                return false;
            }
        }

        return true;
    }

    //team stuff

    void AddTeam(CTeam@ team)
    {
        TDMTeamInfo t(teams.length, team.getName());
        teams.push_back(t);
    }

    void AddPlayer(CPlayer@ player, u8 team = 0, string default_config = "")
    {
        TDMPlayerInfo p(player.getUsername(), getSmallerTeam(), player.isBot() ? "knight" : (Random(512) >= 256 ? "knight" : "archer") );
        players.push_back(p);
        ChangeTeamPlayerCount(p.team, 1);
    }

    //setup the TDM bases

    void SetupBase( CBlob@ base )
    {
        if (base is null) {
            return;
        }

        //nothing to do
    }


    void SetupBases()
    {
        const string base_name = "tdm_spawn";
        // destroy all previous spawns if present
        CBlob@[] oldBases;
        getBlobsByName( base_name, @oldBases );

        for (uint i=0; i < oldBases.length; i++) {
            oldBases[i].server_Die();
        }

        //spawn the spawns :D
        CMap@ map = getMap();

        if (map !is null)
        {
            // team 0 ruins
            Vec2f respawnPos;

            if (!getMap().getMarker("blue main spawn", respawnPos ))
            {
				warn("TDM: Blue spawn marker not found on map");
                respawnPos = Vec2f(150.0f, map.getLandYAtX(150.0f/map.tilesize)*map.tilesize - 32.0f);
            }

            respawnPos.y -= 16.0f;
            SetupBase( server_CreateBlob( base_name, 0, respawnPos ) );

            // team 1 ruins
            if (!getMap().getMarker("red main spawn", respawnPos ))
            {
				warn("TDM: Red spawn marker not found on map");
                respawnPos = Vec2f(map.tilemapwidth*map.tilesize - 150.0f, map.getLandYAtX(map.tilemapwidth - (150.0f/map.tilesize))*map.tilesize- 32.0f);
            }

            respawnPos.y -= 16.0f;
            SetupBase( server_CreateBlob( base_name, 1, respawnPos ) );
        }

        rules.SetCurrentState(WARMUP);
    }

    //checks
    void CheckTeamWon( )
    {
        if (!rules.isMatchRunning()) { return; }

        int winteamIndex = -1;
        TDMTeamInfo@ winteam = null;
        s8 team_wins_on_end = -1;
        int highkills = 0;

        for (uint team_num = 0; team_num < teams.length; ++team_num )
        {
            TDMTeamInfo@ team = cast<TDMTeamInfo@>( teams[team_num] );

            if (team.kills >= kills_to_win)
            {
                @winteam = team;
                winteamIndex = team_num;
                break;
            }

            if (team.kills > highkills)
            {
                highkills = team.kills;
                team_wins_on_end = team_num;
            }
            else if (team.kills > 0 && team.kills == highkills)
            {
                team_wins_on_end = -1;
            }
        }

        rules.set_s8("team_wins_on_end", team_wins_on_end);

        if (winteamIndex >= 0)
        {
            rules.SetTeamWon( winteamIndex ); //game over!
            rules.SetCurrentState(GAME_OVER);
            rules.SetGlobalMessage( winteam.name + " wins the game!" );
        }
    }

    void addKill(int team)
    {
        if (team >= 0 && team < int(teams.length))
        {
            TDMTeamInfo@ team_info = cast<TDMTeamInfo@>( teams[team] );
            team_info.kills++;
        }
    }

    void CheckKnightThrownBomb()
    {
        CBlob@[] knights;
        getBlobsByName( "knight", @knights );

        for (uint i=0; i < knights.length; i++) {
            CBlob@ knight = knights[i];

            if (knight.getPlayer() !is null)
            {
                CBlob@ carried = knight.getCarriedBlob();

                if (carried !is null && carried.getName() == "bomb")
                {
                    PlayerInfo@ p_info = getInfoFromName( knight.getPlayer().getUsername() );

                    if (p_info !is null)
                    {
                        TDMPlayerInfo@ tdm_info = cast<TDMPlayerInfo@>(p_info);

                        if (tdm_info !is null) {
                            tdm_info.thrownBomb = true;
                        }
                    }
                }
            }
        }
    }

    void SpawnPowerups()
    {
        if (getGameTime() % 200 == 0 && Random(12) == 0) {
            SpawnPowerup();
        }
    }

    void SpawnPowerup()
    {
        CBlob@ powerup = server_CreateBlob( "powerup", -1, Vec2f( getMap().tilesize * 0.5f * getMap().tilemapwidth, 50.0f ) );
    }

	void SpawnBombs()
	{
		Vec2f[] bombPlaces;
		if (getMap().getMarkers("bombs", bombPlaces )) 
		{
			for (uint i = 0; i < bombPlaces.length; i++)
			{
				server_CreateBlob( "mat_bombs", -1, bombPlaces[i] );
			}
		}
	}
};

//pass stuff to the core from each of the hooks


void onRestart( CRules@ this )
{
    printf("Restarting rules script: " + getCurrentScriptName() );
    TDMSpawns spawns();
    TDMCore core(this, spawns);
    Config(core);
    core.SetupBases();
    this.set("core", @core);
    this.set("start_gametime", getGameTime() + core.warmUpTime);
    this.set("game_end_time", getGameTime() + core.gameDuration); //for TimeToEnd.as
}

void onPlayerJoin( CRules@ this, CPlayer@ player )
{
    TDMCore@ core;
    this.get("core", @core);

    if (core !is null) {
        core.AddPlayer(player);
    }
}

void onPlayerLeave( CRules@ this, CPlayer@ player )
{
    TDMCore@ core;
    this.get("core", @core);

    if (core !is null) {
        core.RemovePlayer(player);
    }
}

void onTick(CRules@ this)
{
    TDMCore@ core;
    this.get("core", @core);

    if (core !is null) {
        core.Update();
    }
}

void onPlayerDie( CRules@ this, CPlayer@ victim, CPlayer@ killer, u8 customData )
{
    if (!this.isMatchRunning()) { return; }

    TDMCore@ core;
    this.get("core", @core);

    if (core is null) { return; }

    if (victim !is null )
    {
        if (killer !is null && killer.getTeamNum() != victim.getTeamNum())
        {
            core.addKill(killer.getTeamNum());
        }
    }
}

void onReceivedSpawnRequest( CRules@ this, CPlayer@ player )
{
    print("TDM RULES: Player " + player.getUsername()+ " requesting spawn" );

    TDMCore@ core;
    this.get("core", @core);

    if (core !is null) {
        core.AddPlayerSpawn(player);
    }
}

void onPlayerInfoChanged( CRules@ this, CPlayer@ player )
{
	print("TDM RULES: Player " + player.getUsername()+ " changing player info (team)" );

	// kill old player
    CBlob@ blob = player.getBlob();
    if (blob !is null) {
		if (blob.getHealth() > 0.0f)
			blob.server_Die();
		else
			blob.server_SetPlayer( null );
	}
	
	TDMCore@ core;
	this.get("core", @core);

	if (core !is null) {
		core.AddPlayerSpawn(player);
	}
	
	
}


// GIVE RESOURCES

shared TDMPlayerInfo@ getTDMPlayerInfoFromName( CRules@ this, string name )
{
    TDMCore@ core;
    this.get("core", @core);

    if (core !is null)
    {
        PlayerInfo@ p_info = core.getInfoFromName( name );

        if (p_info !is null)
        {
            TDMPlayerInfo@ tdm_info = cast<TDMPlayerInfo@>(p_info);
            return tdm_info;
        }
    }

    return null;
}

void GiveSpawnResources( CRules@ this, CBlob@ blob, CPlayer@ player )
{
    int multiplier = 1;

    // give

   /* if (blob.getName() == "knight") // only give bombs on warmup
    {
        // check if knight has thrown bomb
        TDMPlayerInfo@ tdm_info = getTDMPlayerInfoFromName( this, player.getUsername() );
        if (tdm_info !is null && tdm_info.thrownBomb) {
            return;               // thrown - exit
        }

        CBlob@ mat = server_CreateBlob( "mat_bombs" );

        if (mat !is null)
        {
            if (!blob.server_PutInInventory(mat)) {
                mat.setPosition( blob.getPosition() );
            }

            mat.server_SetQuantity(1*multiplier);
        }
    } */

    if (blob.getName() == "archer")
    {
        CBlob@ mat = server_CreateBlob( "mat_arrows" );

        if (mat !is null)
        {
            if (!blob.server_PutInInventory(mat)) {
                mat.setPosition( blob.getPosition() );
            }									   
        }
    }
}

void onSetPlayer( CRules@ this, CBlob@ blob, CPlayer@ player )
{
    if (blob !is null && player !is null) {
        GiveSpawnResources( this, blob, player );
    }
}
