
//war gamemode logic script

#define SERVER_ONLY

#include "WAR_Structs.as";

#include "RulesCore.as";
#include "RespawnSystem.as";

#include "WAR_PopulateSpawnList.as"
#include "MakeCrate.as";

#include "WAR_HUDCommon.as";

//simple config function - edit the variables in the config file
void Config(WarCore@ this)
{
	string configstr = sv_test ? "Rules/WAR/war_vars_test.cfg" : "Rules/WAR/war_vars.cfg";
	if (getRules().exists("warconfig")) {
	   configstr = getRules().get_string("warconfig");
	}
	ConfigFile cfg = ConfigFile( configstr );

	//how long to wait for everyone to spawn in?
    s32 warmUpTimeSeconds = cfg.read_s32("warmup_time",30);
    this.warmUpTime = (getTicksASecond() * warmUpTimeSeconds);
    //how long for the game to play out?
    s32 gameDurationMinutes = cfg.read_s32("game_time",-1);
    if (gameDurationMinutes <= 0)
    {
		this.gameDuration = -1;
		this.rules.set_bool("no timer", true);
	}
    else
    {
		this.gameDuration = (getTicksASecond() * 60 * gameDurationMinutes);
	}
    //how many players have to be in for the game to start
    this.minimum_players_in_team = cfg.read_s32("minimum_players_in_team",2);
    //whether to scramble each game or not
    this.scramble_teams = cfg.read_bool("scramble_teams",true);

	this.shipmentFirstTime = getTicksASecond() * 60 * cfg.read_s32("shipment_first_time",15);
	this.shipmentFrequency = getTicksASecond() * 60 * cfg.read_s32("shipment_frequency",3);

	//how far away counts as raiding?
	f32 raid_percent = cfg.read_f32("raid_percent", 30) / 100.0f;
	
	CMap@ map = getMap();
	if (map is null)
		this.raid_distance = raid_percent * 2048.0f; //presume 256 tile map
	else
		this.raid_distance = raid_percent * map.tilemapwidth * map.tilesize;


	//ho many dead?
	this.alivePercent = float(100 - cfg.read_s32("dead_percent_lose", 80)) / 100.0f;

    //spawn after death time 
    this.spawnTime = (getTicksASecond() * cfg.read_s32("spawn_time", 15));
}


// without "shared" we cannot hot-swap this class :(
// with "shared" we needt o use other function that are "shared" too

shared class WarSpawns : RespawnSystem
{
    WarCore@ war_core;

    WarPlayerInfo@[] spawns;

    bool force;
	s32 nextSpawn;

	void SetCore(RulesCore@ _core)
	{
		RespawnSystem::SetCore(_core);
		@war_core = cast<WarCore@>(core);

		nextSpawn = getGameTime();
	}

    void Update()
    {
		s32 time = getGameTime();
		if (time % 28 == 0 )
		{
			for (uint i = 0; i < spawns.length; i++)
			{
				updatePlayerSpawnTime(spawns[i]);
			}
		}

		if (time > nextSpawn || force)
		{
			for (uint i = 0; i < spawns.length; i++)
			{
				WarPlayerInfo@ info = spawns[i];
				if (info.wave_delay > 0)
					info.wave_delay--;
					
				print("spawn for "+info.username+" , waves to go: "+info.wave_delay);
					
	            DoSpawnPlayer( info ); //check if we should spawn them
			}

			nextSpawn = getGameTime() + war_core.spawnTime;
		}
    }
    
    void updatePlayerSpawnTime(WarPlayerInfo@ w_info)
    {
		WarTeamInfo@ team = cast<WarTeamInfo@>(core.getTeam(w_info.team));
		//sync player time to them directly
		string propname = "time to spawn "+w_info.username;
		s32 time = getSpawnTime( team, w_info );
		war_core.rules.set_s32( propname, time );
		war_core.rules.SyncToPlayer( propname, getPlayerByUsername(w_info.username) );
		propname = "needs respawn hud "+w_info.username;
		war_core.rules.set_bool( propname, (time > getGameTime()) );
		war_core.rules.SyncToPlayer( propname, getPlayerByUsername(w_info.username) );	
	}

    void DoSpawnPlayer( PlayerInfo@ p_info )
    {
        WarPlayerInfo@ w_info = cast<WarPlayerInfo@>(p_info);

        if (canSpawnPlayer(p_info))
        {
            CPlayer@ player = getPlayerByUsername(p_info.username); // is still connected?
            if (player is null) {
                return;
            }
			RemovePlayerBlob( player );

			// check if player wants to respawn from crate/para drop

			Vec2f spawnPos;
			bool blobSpawn = false;
			if (w_info.spawnpoint > 0 && w_info.spawnpoint < 255)
			{
				u16[] respawns;
				PopulateSpawnList( @respawns, player );
				const uint point = w_info.spawnpoint-1; // 0 is side of map;
				if (point < respawns.length)
				{
					CBlob@ respawnBlob = getBlobByNetworkID( respawns[point] );   // get trading barracks
					if (respawnBlob !is null) // if barracks still exists
					{
						spawnPos = respawnBlob.getPosition();
						blobSpawn = true;
					}
				}
			}

			if (!blobSpawn)
			{
				p_info.blob_name = "builder";
				spawnPos = getSpawnLocation(p_info.team);
			}

			CBlob@ playerBlob = SpawnPlayerIntoWorld( spawnPos, p_info);
            RemovePlayerFromSpawn(player);

            if (playerBlob !is null)
            {
				p_info.spawnsCount++;

                string propname = "needs respawn hud "+p_info.username;
                war_core.rules.set_bool( propname, false );
                war_core.rules.SyncToPlayer( propname, getPlayerByUsername(p_info.username) );
            }
        }
    }

    bool canSpawnPlayer(PlayerInfo@ p_info)
    {
		if (force) { return true; }

        WarPlayerInfo@ w_info = cast<WarPlayerInfo@>(p_info);
        WarTeamInfo@ team = cast<WarTeamInfo@>(core.getTeam(w_info.team));

        return ( w_info.wave_delay == 0 );
    }

	s32 getSpawnTime( WarTeamInfo@ team, WarPlayerInfo@ w_info  )
	{
		return nextSpawn + ((w_info.wave_delay-1)*war_core.spawnTime);
	}

    Vec2f getSpawnLocation(int team)
    {
		CMap@ map = getMap();
		f32 side = map.tilesize * 4.0f;
		f32 x = team == 0 ? side : (map.tilesize*map.tilemapwidth - side);
        f32 y = map.tilesize*map.tilemapheight;
        for (int i = 0; i < map.tilemapheight; i++)
        {
			y -= map.tilesize;
			if ( !map.isTileSolid(map.getTile(Vec2f(x,y))) )
				break;
		}
        y -= 32.0f;
        return Vec2f(x,y);
    }

    void RemovePlayerFromSpawn(CPlayer@ player)
    {
        WarPlayerInfo@ info = cast<WarPlayerInfo@>(core.getInfoFromPlayer(player));
        if (info is null) { warn("WAR LOGIC: Couldn't get player info ( in void RemovePlayerFromSpawn(CPlayer@ player) )"); return; }

        int pos = spawns.find(info);
        if (pos != -1) {
            spawns.erase(pos);
        }
    }

    void AddPlayerToSpawn( CPlayer@ player )
    {
		if (player.getTeamNum() == core.rules.getSpectatorTeamNum())
		{
			RemovePlayerFromSpawn(player);
			return;
		}

        u32 tickspawndelay = 0;
      //  print("ADD SPAWN FOR " + player.getUsername());
        WarPlayerInfo@ info = cast<WarPlayerInfo@>(core.getInfoFromPlayer(player));

        if (info is null) { warn("WAR LOGIC: Couldn't get player info  ( in void AddPlayerToSpawn(CPlayer@ player) )"); return; }

        int pos = spawns.find(info);

        if (pos != -1) {
            spawns.erase(pos);
        }

		//default to next wave spawn (not this wave spawn)
        info.wave_delay = 1;
        if (nextSpawn-getGameTime() <= war_core.spawnTime/2)
			info.wave_delay += 1;
        
        if (info.deathDistanceToBase > 100.0f)
		{
			//TODO: some penalty for dying at base?
			//info.wave_delay++;
		}
		if (info.suicide)
		{
			//info.wave_delay += 2; //TODO: config var?
		}

		info.spawnpoint = player.getSpawnPoint();

		//printf("info.spawnpoint " + info.spawnpoint );
        if (info.spawnpoint == 0 || player.getTeamNum() == 255) // side of map or first join
        {
        }
        else
        {
            war_core.ChangePlayerTeam(player, player.getTeamNum()); // TEAM assignement
        }

        spawns.push_back(info);
    }

};

shared class WarCore : RulesCore
{
    string base_name = "war_base";

    s32 warmUpTime;
    s32 gameDuration;
    s32 spawnTime;
    s32 minimum_players_in_team;
    bool scramble_teams;
	s32 shipmentFirstTime;
	s32 shipmentFrequency;
	f32 alivePercent;

	f32 raid_distance;

    WarSpawns@ war_spawns;

    WarCore() {}

    WarCore(CRules@ _rules, RespawnSystem@ _respawns )
    {
        super(_rules, _respawns );
    }

    void Setup(CRules@ _rules = null, RespawnSystem@ _respawns = null)
    {
        RulesCore::Setup(_rules, _respawns);
        gametime = getGameTime();
		startTime = nextShipmentTime = 0;
        @war_spawns = cast<WarSpawns@>(_respawns);
        rules.SetCurrentState(WARMUP);
        server_CreateBlob( "Entities/Meta/WARMusic.cfg" );
    }

    int gametime;
	int startTime, nextShipmentTime;

    void Update()
    {
		//HUD
        if (getGameTime() % 31 == 0)
			updateHUD();

        if (rules.isGameOver()) { return; }

		if (rules.isWarmup())
		{
			const s32 getgametime = getGameTime();
			s32 ticksToStart = gametime - getgametime;
			//setting the game state to running after warmup
			war_spawns.force = false;

			if (ticksToStart <= 0)
			{
				rules.SetCurrentState(GAME);
				printf("WAR STARTED");
				startTime = getgametime;
			}
			else if (ticksToStart > 0) //is the start of the game, spawn everyone + give mats
			{
				rules.SetGlobalMessage( "\nMatch starts in "+((ticksToStart/30)+1) );
				war_spawns.force = true;
			}
		}

        //CHECK IF TEAMS HAVE ENOUGH PLAYERS
        if ((rules.isIntermission() || rules.isWarmup()) && (!allTeamsHavePlayers()) )
        {
            gametime = getGameTime() + warmUpTime;
            rules.set("game_end_time", gametime + gameDuration);
            rules.SetGlobalMessage( "Not enough players in each team for the game to start.\nPlease wait for someone to join..." );
            war_spawns.force = true;
        }
        else
        {
			int tick = 35;
			if (getGameTime() % tick == 0)
			{
				//needs to be updated before the teamwon
				UpdatePlayerCounts();
				//check if the team won
				CheckTeamWon(tick);
				
				if (rules.isMatchRunning())
				{
					rules.SetGlobalMessage( "" );
					updateShipments();
				}
			}
		}
        
		RulesCore::Update(); //update respawns
    }
    
    void updateHUD()
    {
		CBitStream serialised_team_hud;
		serialised_team_hud.write_u16(0x54f3);

		for (uint team_num = 0; team_num < teams.length; ++team_num )
		{
			WAR_HUD team_hud;
			WarTeamInfo@ team = cast<WarTeamInfo@>(teams[team_num]);

			if (team is null) {
				continue;
			}

			//forward dec
			team_hud.team_num = team_num;
			
			//units alive
			team_hud.num_units = team.players_count;
			team_hud.num_alive = team.alive_count;
			
			team_hud.under_raid = team.under_raid;
			
			//write it out
			team_hud.Serialise(serialised_team_hud);
		}

		rules.set_CBitStream("WAR_serialised_team_hud",serialised_team_hud);
		rules.Sync("WAR_serialised_team_hud",true);
	}

    //HELPERS

    int getSmallerTeam()
    {
        s32 lowestTeam = (Random(512) % teams.length);
        s32 lowestCount = teams[lowestTeam].players_count;

        for (uint i = 0; i < teams.length; i++)
        {
            if (teams[i].players_count < lowestCount)
            {
                lowestCount = teams[i].players_count;
                lowestTeam = i;
            }
        }

        return lowestTeam;
    }

    bool allTeamsHavePlayers()
    {
        for (uint i = 0; i < teams.length; i++)
        {
            if (teams[i].players_count < minimum_players_in_team)
            {
                return false;
            }
        }

        return true;
    }

    //team stuff

    void AddTeam(CTeam@ team)
    {
        WarTeamInfo t(teams.length, team.getName());
        teams.push_back(t);
    }

    void AddPlayer(CPlayer@ player, u8 team = 0, string default_config = "")
    {
        WarPlayerInfo p(player.getUsername(), getSmallerTeam(), "builder");
        players.push_back(p);
        ChangeTeamPlayerCount( p.team, 1);
    }

	int lostCount;
	void UpdatePlayerCounts()
	{
		for (uint i = 0; i < teams.length; i++)
		{
			WarTeamInfo@ team = cast<WarTeamInfo@>( teams[i] );
			//"reset"
			team.alive_count = 0;
			team.under_raid = false;
		}

		for (uint step = 0; step < players.length; ++step)
		{
			CPlayer@ p = getPlayerByUsername(players[step].username);
			if (p is null) continue;
			CBlob@ player = p.getBlob();
			if (player is null) continue;
			//whew, actually got a blob now..
			if (!player.hasTag("dead"))
			{
				int teamNum = player.getTeamNum();
				if (teamNum >= 0 && teamNum < teams.length) {
					teams[teamNum].alive_count++;
				}
			}
		}
		
		lostCount = 0;
		for (uint i = 0; i < teams.length; i++)
		{
			WarTeamInfo@ team = cast<WarTeamInfo@>( teams[i] );
			
			// check if enemy is raiding us - near spawn

			const u8 enemyTeam = (i+1) % 2; //hack, assumes 2 teams
			Vec2f spawnPos = war_spawns.getSpawnLocation(i);
			uint enemyRaiders = 0;
			for (uint step = 0; step < players.length; ++step)
			{
				CPlayer@ p = getPlayerByUsername(players[step].username);
				if (p is null) continue;
				CBlob@ player = p.getBlob();
				if (player is null) continue;
				//whew, actually got a blob now..
				if (player.getTeamNum() == enemyTeam && 							//enemy
					!player.hasTag("dead") && 										//not dead
					(spawnPos - player.getPosition()).getLength() < raid_distance ) //near spawn
				{
					enemyRaiders++;
				}
			}
								
			team.under_raid = (enemyRaiders > 0);
			
			int aliveNeeded = Maths::Min( Maths::Max(0.0f,Maths::Floor(team.players_count * alivePercent)+1), team.players_count );
			if (team.alive_count < aliveNeeded && team.under_raid)
			{
				team.lost = true;
				lostCount++;
			}
		}
	}

    //checks
    void CheckTeamWon( int tickFrequency )
    {
		// calc alive players before this function with UpdatePlayerCounts!
		
		// can't lose if the match is running
		
        if (!rules.isMatchRunning()) { return; }

        if ( lostCount > 1 )
        {
            rules.SetCurrentState(GAME_OVER);
            rules.SetGlobalMessage("Both teams lost!" );
        }
		else if (lostCount == 1)
		{
			for (uint i = 0; i < teams.length; i++)
			{
				BaseTeamInfo@ team = teams[i];
				if (!team.lost)
				{
					rules.SetTeamWon( i );
					rules.SetCurrentState(GAME_OVER);
					rules.SetGlobalMessage( team.name + " wins the game!" );
					break;
				}
			}
		}
    }

	void updateShipments()
	{
		if (startTime > 0)
		{
			const s32 time = getGameTime();

		//	printf("ship in  " + " time " + time + " startTime " + startTime + " shipmentFirstTime " + shipmentFirstTime + " nextShipmentTime " + nextShipmentTime);

			// ship?
			if ((nextShipmentTime == 0 && time >= startTime + shipmentFirstTime) ||
				(nextShipmentTime > 0  && time >= nextShipmentTime) )

			{
				// gather barracks

				CBlob@[] bases;
				if (getBlobsByName( "storage", @bases )) {
					for (uint step = 0; step < bases.length; ++step)
					{
						CBlob@ base = bases[step];
						CBitStream params;
						base.SendCommand( base.getCommandID("shipment"), params );
					//	printf("SHIPMENT " + " startTime " + startTime + " " + shipmentFrequency);

						// spawn crate
						Vec2f droppos = base.getPosition();
						droppos.x += (base.getTeamNum() == 0) ? -45.0f : 45.0f;
						CBlob@ crate = server_MakeCrateOnParachute( "", "", 5, base.getTeamNum(), getDropPosition(droppos) );
						if (crate !is null)
						{
							// make unpack button
							crate.Tag("unpackall");
							//crate.Tag("destroy on touch");
							for (uint i = 0; i < 4; i++)
							{
								CBlob@ mat = server_CreateBlob("mat_wood");
								if (mat !is null) {
									crate.server_PutInInventory( mat );
								}
							}
							for (uint i = 0; i < 2; i++)
							{
								CBlob@ mat = server_CreateBlob("mat_stone");
								if (mat !is null) {
									crate.server_PutInInventory( mat );
								}
							}
							for (uint i = 0; i < 1; i++)
							{
								CBlob@ mat = server_CreateBlob("mat_gold");
								if (mat !is null)
								{
									mat.server_SetQuantity(100);
									crate.server_PutInInventory( mat );
								}
							}
						}
					}
				}

				nextShipmentTime = time + shipmentFrequency;
			}
		}
	}

	void NotifyDeathDistance( CPlayer@ player, f32 distance )
	{
		WarPlayerInfo@ info = cast<WarPlayerInfo@>(getInfoFromPlayer(player));
        if (info is null) { return; }
        info.deathDistanceToBase = distance;
	}
};

//pass stuff to the core from each of the hooks

void onRestart( CRules@ this )
{
    printf("Restarting rules script: " + getCurrentScriptName() );
    WarSpawns spawns();
    WarCore core(this, spawns);
    Config(core);
    this.set("core", @core);

    core.gametime = getGameTime() + core.warmUpTime;
    this.set("start_gametime", core.gametime);
    this.set("game_end_time", getGameTime() + core.gameDuration); //for TimeToEnd.as

	// place no build zones at sides
	CMap@ map = getMap();
	f32 space = map.tilesize * 5.0f;
	map.server_AddSector( Vec2f(0.0f,0.0f), Vec2f(space, map.tilemapheight*map.tilesize), "no build" );
	map.server_AddSector( Vec2f(map.tilemapwidth*map.tilesize-space,0.0f), Vec2f(map.tilemapwidth*map.tilesize, map.tilemapheight*map.tilesize), "no build" );
}

void onPlayerJoin( CRules@ this, CPlayer@ player )
{
	print("WAR RULES: Player " + player.getUsername() + " has joined" );

    WarCore@ core;
    this.get("core", @core);

    if (core !is null) {
        core.AddPlayer(player);
    }
}

void onPlayerLeave( CRules@ this, CPlayer@ player )
{
    WarCore@ core;
    this.get("core", @core);

    if (core !is null) {
        core.RemovePlayer(player);
    }
}

void onTick(CRules@ this)
{
    WarCore@ core;
    this.get("core", @core);

    if (core !is null) {
        core.Update();
    }
}

void onPlayerDie( CRules@ this, CPlayer@ victim, CPlayer@ killer, u8 customData )
{
    if (victim !is null )
    {
		CBlob@ blob = victim.getBlob();
		if (blob !is null)
		{
			f32 deathDistanceToBase = 0.0f;

			// check distance to barracks

			WarCore@ core;
			this.get("core", @core);
			if (core !is null) {
				deathDistanceToBase = Maths::Abs( core.war_spawns.getSpawnLocation( blob.getTeamNum() ).x - blob.getPosition().x );
			}

			core.NotifyDeathDistance( victim, deathDistanceToBase );
		}
		else
			warn("Victim blob not found!");
	}
}

void onReceivedSpawnRequest( CRules@ this, CPlayer@ player )
{
    print("WAR RULES: Player " + player.getUsername() + " wants to spawn (at " + player.getSpawnPoint() + ")" );

    WarCore@ core;
    this.get("core", @core);

    if (core !is null) {
        core.AddPlayerSpawn(player);
    }
}

void onPlayerInfoChanged( CRules@ this, CPlayer@ player )
{
	print("WAR RULES: Player " + player.getUsername() + " wants to change team to " + player.getTeamNum() + "" );

	// kill old player
    CBlob@ blob = player.getBlob();
    if (blob !is null) {
		if (blob.getHealth() > 0.0f)
			blob.server_Die();
		else
			blob.server_SetPlayer( null );
	}

	WarCore@ core;
	this.get("core", @core);

	if (core !is null) {
		core.AddPlayerSpawn(player);
	}
}

