// spawn resources

#define SERVER_ONLY

#include "ProductionCommon.as";

bool GiveGroundItems( CBlob@ blob, const string &in name )
{
	CBlob@[] blobsInRadius;
	if (getMap().getBlobsInRadius( blob.getPosition(), blob.getRadius()+32.0f, @blobsInRadius ))
	{
		for (uint i = 0; i < blobsInRadius.length; i++)
		{
			CBlob @b = blobsInRadius[i];
			if (b.getName() == name)
			{
				if (!blob.server_PutInInventory( b ))
					b.setPosition( blob.getPosition() );
				return true;
			}
		}
	}
	return false;
}


bool GiveStorageItem( CBlob@ blob, const string &in name )
{
	// find simply on ground first

	if (GiveGroundItems( blob, name ))
		return true;

	// check in storages

	const s32 team = blob.getTeamNum();

	CBlob@[] bases;
	if (getBlobsByName( "storage", @bases ))
		for (uint step = 0; step < bases.length; ++step)
		{
			CBlob@ base = bases[step];
			if (base.getTeamNum() == team)
			{																
				if (base.hasBlob( name, 1 ))
				{
					CBlob@ item = base.server_PutOutInventory( name );
					if (item !is null)
					{			   
						if (!blob.server_PutInInventory( item ))
							item.setPosition( blob.getPosition() );
						return true;
					}
				}
			}
		}
	return false;
}

// find factory that makes this item and give it
// if not try to find it in base and factory storages

void GiveFactoryItem( CBlob@ blob, const string &in name )
{
	const s32 team = blob.getTeamNum();

	// find in base storage

	if (GiveStorageItem( blob, name ))
		return;


	CBlob@[] factories;
	if (getBlobsByName( "factory", @factories ))
	{
		// find in storages

		for (uint step = 0; step < factories.length; ++step)
		{
			CBlob@ factory = factories[step];
			if (factory.getTeamNum() == team)
			{																
				if (factory.hasBlob( name, 1 ))
				{
					CBlob@ item = factory.server_PutOutInventory( name );
					if (item !is null)
					{			   
						if (!blob.server_PutInInventory( item ))
							item.setPosition( blob.getPosition() );
						return;

					}
				}
			}
		}

		// produce

		for (uint step = 0; step < factories.length; ++step)
		{
			CBlob@ factory = factories[step];
			if (factory.getTeamNum() == team && canProduce( factory, name) )
			{																
				CBitStream params;
				params.write_u16( blob.getNetworkID() );
				params.write_string( name );
				factory.SendCommandOnlyServer( factory.getCommandID("factory give item"), params );
				return;

			}
		}
	}
}

// base makes it 

void MakeOrGiveItem( CBlob@ blob, const string &in name )
{
	const s32 team = blob.getTeamNum();

	// find in base storage

	if (GiveStorageItem( blob, name ))
		return;

	// if not make it

	CBlob@ item = server_CreateBlob( name );
	if (item !is null)
	{
		if (!blob.server_PutInInventory( item ))
			item.setPosition( blob.getPosition() );
	}
}

void GiveSpawnResources( CRules@ this, CBlob@ blob, CPlayer@ player )   
{	 	  	 
	if (blob.getName() == "builder")
	{
		if (sv_test)
		{
			{
				CBlob@ mat = server_CreateBlob( "mat_stone" );
				if (mat !is null)
				{
					mat.server_SetQuantity(250); 
					if (!blob.server_PutInInventory(mat)) {
						mat.setPosition( blob.getPosition() );
					}
				}
			}
			{
				CBlob@ mat = server_CreateBlob( "mat_wood" );

				if (mat !is null)
				{
					mat.server_SetQuantity(250);

					if (!blob.server_PutInInventory(mat)) {
						mat.setPosition( blob.getPosition() );
					}
				}
			}
			{
				CBlob@ mat = server_CreateBlob( "mat_gold" );

				if (mat !is null)
				{
					mat.server_SetQuantity(250);

					if (!blob.server_PutInInventory(mat)) {
						mat.setPosition( blob.getPosition() );
					}
				}
			}
		}
		else  
		{
			GiveStorageItem( blob, "mat_wood" );
			GiveStorageItem( blob, "mat_stone" );
			GiveStorageItem( blob, "mat_gold" );
		}
	}
	else if (blob.getName() == "archer")
	{			
		GiveFactoryItem( blob, "mat_arrows" );
	}
	else if (blob.getName() == "knight")
	{
		GiveFactoryItem( blob, "mat_bombs" );
	}  	
}

void onSetPlayer( CRules@ this, CBlob@ blob, CPlayer@ player )
{
	if (blob !is null && player !is null) {
		GiveSpawnResources( this, blob, player );
	}
}	   