// PNG loader base class - extend this to add your own PNG loading functionality!

// tiles
const SColor color_tile_ground(255, 132, 71, 21);
const SColor color_tile_ground_back(255, 59, 20, 6);
const SColor color_tile_stone(255, 139, 104, 73);
const SColor color_tile_thickstone(255, 66, 72, 75);
const SColor color_tile_bedrock(255, 45, 52, 45);
const SColor color_tile_gold(255, 254, 165, 61);
const SColor color_tile_castle(255, 100, 113, 96);
const SColor color_tile_castle_back(255, 49, 52, 18);
const SColor color_tile_ladder(255, 43, 21, 9);
const SColor color_tile_ladder_ground(255, 66, 36, 11);
const SColor color_tile_ladder_castle(255, 67, 47, 17);
const SColor color_tile_ladder_wood(255, 69, 57, 17);
const SColor color_tile_grass(255, 100, 155, 13);
const SColor color_tile_wood(255, 196, 135, 21);
const SColor color_tile_wood_back(255, 85, 42, 17);
const SColor color_water_air(255, 46, 129, 166);
const SColor color_water_backdirt(255, 51, 85, 102);
// objects
const SColor color_blue_main_spawn(255, 0, 255, 255);
const SColor color_red_main_spawn(255, 255, 0, 0);
const SColor color_blue_spawn(255, 0, 200, 200);
const SColor color_red_spawn(255, 200, 0, 0);
const SColor color_tree(255, 13, 103, 34);
const SColor color_bush(0xff5b7e18);
const SColor color_grain(0xffa2b716);
const SColor color_spikes(255, 180, 42, 17);
const SColor color_spikes_ground(255, 180, 97, 17);
const SColor color_spikes_castle(255, 180, 42, 94);
const SColor color_spikes_wood(255, 200, 42, 94);
const SColor color_trampoline(255, 187, 59, 253);
const SColor color_door_1(255, 26, 78, 131);
const SColor color_door_2(255, 148, 27, 27);
const SColor color_door_noteam(255, 148, 148, 148);
const SColor color_bombs(0xfffbf157);
const SColor color_shark(0xff2cafde);
const SColor color_fish(0xff79a8a3);
const SColor color_lantern(0xfff1e7b1);

enum WAROffset {
	autotile_offset = 0,
	tree_offset,
	bush_offset,
	grain_offset,
	spike_offset,
	offsets_count
};

class PNGLoader
{
	
	PNGLoader()
	{
		offsets = array<array<int>>(offsets_count, array<int>(0));
	}
	
	CFileImage@ image;
	CMap@ map;
	
	array<array<int>> offsets;
	
	int current_offset_count;
	
	bool loadMap( CMap@ _map, const string& in filename)
	{
		@map = _map;

		if (!getNet().isServer())
		{
			SetupBackgrounds();
			return true;
		}

		@image = CFileImage( filename );
		
		if (image.isLoaded())
		{
			SetupMap(image.getWidth(), image.getHeight());
			
			while (image.nextPixel())
			{
				SColor pixel = image.readPixel();
				int offset = image.getPixelOffset();
				
				handlePixel(pixel, offset);
				
				getNet().server_KeepConnectionsAlive();
			}
						
			// late load - after placing tiles

			// load trees
			for (uint i = 0; i < offsets.length; ++i)
			{
				int[]@ offset_set = offsets[i];
				current_offset_count = offset_set.length;
				for (uint step = 0; step < current_offset_count; ++step)
				{
					handleOffset(i,offset_set[step], step, current_offset_count);
					
					getNet().server_KeepConnectionsAlive();
				}
			}

			SetupBackgrounds();	
			return true;
		}
		return false;
	}
	
	//override this to extend functionality per-pixel.
	void handlePixel(SColor pixel, int offset)
	{
		if (pixel == color_tile_ground) {
			map.server_SetTile(offset, CMap::tile_ground );
		}
		else if (pixel == color_tile_ground_back) {
			map.server_SetTile(offset, CMap::tile_ground_back );
		}
		else if (pixel == color_tile_stone) {
			map.server_SetTile(offset, CMap::tile_stone );
		}
		else if (pixel == color_tile_thickstone) {
			map.server_SetTile(offset, CMap::tile_thickstone );
		}
		else if (pixel == color_tile_bedrock) {
			map.server_SetTile(offset, CMap::tile_bedrock );
		}
		else if (pixel == color_tile_gold) {
			map.server_SetTile(offset, CMap::tile_gold );
		}
		else if (pixel == color_tile_castle) {
			map.server_SetTile(offset, CMap::tile_castle );
		}
		else if (pixel == color_tile_castle_back) {
			map.server_SetTile(offset, CMap::tile_castle_back );
		}
		else if (pixel == color_tile_wood) {
			map.server_SetTile(offset, CMap::tile_wood );
		}
		else if (pixel == color_tile_wood_back) {
			map.server_SetTile(offset, CMap::tile_wood_back );
		}
		else if (pixel == color_tile_ladder) {
			map.server_SetTile(offset, CMap::tile_ladder );
		}
		else if (pixel == color_tile_ladder_ground) {
			map.server_SetTile(offset, CMap::tile_ladder_ground );
		}
		else if (pixel == color_tile_ladder_castle) {
			map.server_SetTile(offset, CMap::tile_ladder_castle );
		}
		else if (pixel == color_tile_ladder_wood) {
			map.server_SetTile(offset, CMap::tile_ladder_wood );
		}
		else if (pixel == color_tile_grass) {
			map.server_SetTile(offset, CMap::tile_grass + Random( offset*3, 3) );
		}
		else if (pixel == color_water_air) {
			map.server_setFloodWaterOffset( offset, true );
		}
		else if (pixel == color_water_backdirt)
		{
			map.server_setFloodWaterOffset( offset, true );
			map.server_SetTile( offset, CMap::tile_ground_back );
		}
		else if (pixel == color_blue_main_spawn) {
			AddMarker( map, offset, "blue main spawn" );
		}
		else if (pixel == color_red_main_spawn) {
			AddMarker( map, offset, "red main spawn" );
		}
		else if (pixel == color_blue_spawn) {
			AddMarker( map, offset, "blue spawn" );
		}
		else if (pixel == color_red_spawn) {
			AddMarker( map, offset, "red spawn" );
		}
		else if (pixel == color_door_1)
		{
			spawnBlob( map, "wooden_door", offset, 0, true );
			offsets[autotile_offset].push_back( offset );
		}
		else if (pixel == color_door_2)
		{
			spawnBlob( map, "wooden_door", offset, 1, true );
			offsets[autotile_offset].push_back( offset );
		}
		else if (pixel == color_door_noteam)
		{
			spawnBlob( map, "wooden_door", offset, 255, true );
			offsets[autotile_offset].push_back( offset );
		}
		else if (pixel == color_spikes) {
			offsets[spike_offset].push_back( offset );
		}
		else if (pixel == color_spikes_castle)
		{
			map.server_SetTile(offset, CMap::tile_castle_back );
			offsets[spike_offset].push_back( offset );
		}
		else if (pixel == color_spikes_wood)
		{
			map.server_SetTile(offset, CMap::tile_wood_back );
			offsets[spike_offset].push_back( offset );
		}
		else if (pixel == color_spikes_ground)
		{
			map.server_SetTile(offset, CMap::tile_ground_back );
			offsets[spike_offset].push_back( offset );
		}
		// TREE
		else if (pixel.getRed() == color_tree.getRed() && pixel.getBlue() == color_tree.getBlue() && pixel.getGreen() >= color_tree.getGreen() && pixel.getGreen() <= color_tree.getGreen()+3)
		{
			offsets[tree_offset].push_back( offset );
			offsets[autotile_offset].push_back( offset );
		}
		//NATURE
		else if (pixel == color_bush)
		{
			offsets[bush_offset].push_back( offset );
			offsets[autotile_offset].push_back( offset );
		}
		else if (pixel == color_grain)
		{
			offsets[grain_offset].push_back( offset );
			offsets[autotile_offset].push_back( offset );
		}
		// TRAMPOLINE
		else if (pixel == color_trampoline)
		{
			CBlob@ trampoline = server_CreateBlobNoInit( "trampoline" );

			if (trampoline !is null)
			{
				trampoline.Tag("start unpacked");
				trampoline.Tag("invincible");
				trampoline.Tag("static");
				trampoline.setPosition( getSpawnPosition( map, offset ) );
				trampoline.Init();
			}

			offsets[autotile_offset].push_back( offset );
		}
		else if (pixel == color_shark)
		{
			spawnBlob( map, "shark", offset, -1);
			offsets[autotile_offset].push_back( offset );
		}
		else if (pixel == color_fish)
		{
			CBlob@ fishy = spawnBlob( map, "fishy", offset, -1);
			if (fishy !is null)
			{
				fishy.set_u8("age", (offset * 997) % 4 );
			}
			offsets[autotile_offset].push_back( offset );
		}
		else if (pixel == color_bombs)
		{
			AddMarker( map, offset, "bombs" );
			offsets[autotile_offset].push_back( offset );
		}
		else if (pixel == color_lantern)
		{
			spawnBlob( map, "lantern", offset, -1, true);
			offsets[autotile_offset].push_back( offset );
		}
	}
	
	//override this to add post-load offset types.
	void handleOffset(int type, int offset, int position, int count)
	{
		if (type == autotile_offset)
		{
			PlaceMostLikelyTile(map,offset);
		}
		else if (type == tree_offset)
		{
			if (map.isTileSolid( map.getTile(offset + map.tilemapwidth) )) // load trees only at the ground
			{
				CBlob@ tree = server_CreateBlobNoInit( Random( offset, 35 ) < 21 ? "tree_pine" : "tree_bushy" );

				if (tree !is null)
				{
					tree.Tag("startbig");
					tree.setPosition( getSpawnPosition( map, offset ) );
					tree.Init();
					
					if (map.getTile(offset) == CMap::tile_empty)
						map.server_SetTile(offset, CMap::tile_grass + Random( offset*3, 3) );
				}
			}
		}
		else if (type == bush_offset)
		{
			server_CreateBlob( "bush", -1, map.getTileWorldPosition(offset) + Vec2f(4.0f, 4.0f)  );
		}
		else if (type == grain_offset)
		{
			CBlob@ grain = server_CreateBlobNoInit( "grain_plant" );
			if (grain !is null)
			{
				grain.Tag("instant_grow");
				grain.setPosition( map.getTileWorldPosition(offset) + Vec2f(4.0f, 4.0f) );
				grain.Init();
			}
		}
		else if (type == spike_offset)
		{
			CBlob@ spikes = server_CreateBlob( "spikes", -1, map.getTileWorldPosition(offset) + Vec2f(4.0f, 4.0f)  );

			if (spikes !is null) {
				spikes.getShape().SetStatic( true );
			}
		}
	}

	void SetupMap( int width, int height )
	{
		map.CreateTileMap( width, height, 8.0f, "Sprites/world.png" );
	}

	void SetupBackgrounds()
	{
		// sky

		map.CreateSky( color_black, Vec2f(1.0f,1.0f), 200, "Sprites/Back/cloud", 0);
		map.CreateSkyGradient( "Sprites/skygradient.png" ); // override sky color with gradient

		// plains

		map.AddBackground( "Sprites/Back/BackgroundPlains.png", Vec2f(0.0f, 0.0f), Vec2f(0.3f, 0.3f), color_white ); 
		map.AddBackground( "Sprites/Back/BackgroundTrees.png", Vec2f(0.0f,  19.0f), Vec2f(0.4f, 0.4f), color_white ); 
		//map.AddBackground( "Sprites/Back/BackgroundIsland.png", Vec2f(0.0f, 50.0f), Vec2f(0.5f, 0.5f), color_white ); 
		map.AddBackground( "Sprites/Back/BackgroundCastle.png", Vec2f(0.0f, 50.0f), Vec2f(0.6f, 0.6f), color_white ); 

		// fade in 				   
		SetScreenFlash( 255, 0, 0, 0 );
	}
}

void PlaceMostLikelyTile( CMap@ map, int offset )
{
    TileType up = map.getTile( offset - map.tilemapwidth);
    TileType down = map.getTile( offset + map.tilemapwidth);
    TileType left = map.getTile( offset - 1);
    TileType right = map.getTile( offset + 1);

	bool upEmpty = (up == CMap::tile_empty);

    if (!upEmpty &&
			( up == CMap::tile_castle || up == CMap::tile_castle_back ||
			down == CMap::tile_castle || down == CMap::tile_castle_back ||
			left == CMap::tile_castle || left == CMap::tile_castle_back ||
			right == CMap::tile_castle || right == CMap::tile_castle_back))
	{
		map.server_SetTile(offset, CMap::tile_castle_back );
	}
	else if (!upEmpty &&
			( up == CMap::tile_wood || up == CMap::tile_wood_back ||
			 down == CMap::tile_wood || down == CMap::tile_wood_back ||
			 left == CMap::tile_wood || left == CMap::tile_wood_back ||
			 right == CMap::tile_wood || right == CMap::tile_wood_back))
	{
		map.server_SetTile(offset, CMap::tile_wood_back );
	}
	else if (!upEmpty &&
			( up == CMap::tile_ground || up == CMap::tile_ground_back ||
			 down == CMap::tile_ground || down == CMap::tile_ground_back ||
			 left == CMap::tile_ground || left == CMap::tile_ground_back ||
			 right == CMap::tile_ground || right == CMap::tile_ground_back) )
	{
		map.server_SetTile(offset, CMap::tile_ground_back );
	}
	else if ( map.isTileSolid(down) &&
			 (map.isTileGrass(left) || map.isTileGrass(right)) )
	{
		map.server_SetTile(offset, CMap::tile_grass + 2 + Random(offset*3, 2) );
	}
}


Vec2f getSpawnPosition( CMap@ map, int offset )
{
    Vec2f pos = map.getTileWorldPosition(offset);
    f32 tile_offset = map.tilesize * 0.5f;
    pos.x += tile_offset;
    pos.y += tile_offset;
    return pos;
}

CBlob@ spawnBlob( CMap@ map, const string& in name, int offset, int team, bool attached_to_map = false )
{
    CBlob@ blob = server_CreateBlob( name, team, getSpawnPosition( map, offset) );

    if (blob !is null && attached_to_map) {
        blob.getShape().SetStatic( true );
    }

    return blob;
}

void AddMarker( CMap@ map, int offset, const string& in name)
{
    map.AddMarker( map.getTileWorldPosition( offset ), name );
    PlaceMostLikelyTile( map, offset );
}
